/* $Id: ac3mode.c,v 1.4 2004/11/29 01:21:08 rcochran Exp $ */
/*
 *  Copyright (c) 2003, 2004 Richard Cochran (cochran@cs.umass.edu)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 */
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include "exaudio_remote.h"

#define DEVICE "/dev/ir"
#define STRLEN 1024

static void usage(char* progname)
{
  fprintf(stderr,
          "Sets the AC3 playback mode, usage: %s [options] \n"
          " -d [file]  IR device (default %s) \n"
          " -h         prints this message \n"
          " -m [1-8]   which mode: \n"
          "           L          \n"
          "           F          \n"
          "mode Rs Ls E  C  R  L \n"
          "--------------------- \n"
          "1    0  0  0  0  1  1 \n"
          "2    0  0  0  0  1  1 \n"
          "3    0  0  1  0  1  1 \n"
          "4    1  1  0  0  1  1 \n"
          "5    1  1  1  0  1  1 \n"
          "6    1  1  0  1  1  1 \n"
          "7    1  1  1  1  1  1 \n"
	  "8  -- pass through -- \n"
          "  Channel names: \n"
          "  Rs = Right surround, Ls = Left surround, \n"
          "  LFE = Low frequency enhancement, C = Center front, \n"
          "  R = Right front, L = Left front \n"
          ,progname,DEVICE);
}

int main(int argc,char* argv[])
{
  int c, mx, new=0, sel=1, val=1;
  char* progname;
  char mstr[STRLEN] = DEVICE;

  /* Handle the command line arguments. */
  progname = strrchr(argv[0],'/');
  progname = progname ? 1+progname : argv[0];
  while (EOF != (c = getopt(argc,argv,"d:hm:"))) {
    switch (c) {
    case 'd': snprintf(mstr,STRLEN,"%s",optarg); break;
    case 'm': val = atoi(optarg); break;
    case 'h': usage(progname); return(0);
    case '?': usage(progname); return(-1);
    default:  usage(progname); return(-1);
    }
  }
  mx = open(mstr,O_RDWR);
  if (mx < 0) {
    fprintf(stderr,"Cannot open %s : %s\n",mstr,strerror(errno));
    return(-1);
  }
  if (val < 1 || val > 8) {
    usage(progname);
    return(-1);
  }

  if (8 == val) {
    /* Set SU29 to pass the AC3 stream through. */
    sel = 2;
    if (ioctl(mx,EXAUDIO_SET_SU29,&sel) < 0)
      perror("error setting selector pin");
  } else {
    /* Set SU29 to decode the AC3 stream. */
    if (ioctl(mx,EXAUDIO_SET_SU29,&sel) < 0)
      perror("error setting selector pin");
    if (ioctl(mx,EXAUDIO_SET_AC3_MODE,&val) < 0)
      perror("error setting mode");
    if (ioctl(mx,EXAUDIO_GET_AC3_MODE,&new) < 0)
      perror("error getting mode");
    if (new != val) {
      fprintf(stderr,"tried mode %d, but got mode %d\n",val,new);
    }
  }

  close(mx);
  return(0);
}
