/* $Id: irxmms.c,v 1.5 2003/06/14 15:46:02 cochran Exp $
 *
 *  Copyright (c) 2002 Richard Cochran and Ilia Lobsanov
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 */

#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <xmms/xmmsctrl.h> /* provided by xmms-devel */
#include "ircodes.h"

static int find_xmms(void)
{
  int session;

  for (session = 0 ; session < 16 ; session++) 
    if (xmms_remote_is_running(session)) break;

  if (session == 16)
    return(-1);
  else
    return(session);
}

/*
 * go to previous song, wrap to the last song if the position
 * in the play list is the first. Note that in case of CD,
 * the first position is 1 otherwise it is 0.
 */
static void play_prev (gint session)
{
  const gchar DEV[] = "/dev/";
  gint first = 0;
  
  if (!strncmp(DEV,
	       xmms_remote_get_playlist_file
	       (session,xmms_remote_get_playlist_pos(session)),
	       5)) {
    /*puts ("It appears you're playing a CD");*/
    first = 1;
  }
  
  if (first != xmms_remote_get_playlist_pos (session))
    xmms_remote_playlist_prev(session);
  else
    xmms_remote_set_playlist_pos(session,
				 xmms_remote_get_playlist_length(session) - 1);
}

/*
 * vol command: needs number | +number | -number
 */
static void set_vol(gint session,int arg)
{
  gint vol = xmms_remote_get_main_volume(session);
  vol += arg;
  /* check bounds */
  if (vol < 0) 
    vol = 0;
  else if (vol > 100)
    vol = 100;
  xmms_remote_set_main_volume(session,vol);
}

/* skips given number of milliseconds from current file offset */
static void skip(gint session,int ms)
{
  if (xmms_remote_is_playing(session))
    xmms_remote_jump_to_time(session,ms+xmms_remote_get_output_time(session));
}

int main(int argc,char* argv[])
{  
  gint session;
  gint volq;
  gint savedvol;
  int fd;
  unsigned int tmp;
  unsigned short code;
  pid_t pid;

  session = find_xmms();
  if (session < 0) {
    pid = fork();
    if (pid) { /* parent */
      sleep(2);
      session = find_xmms();
      if (session < 0) {
	fprintf(stderr,"Cannot launch xmms\n");
	kill(pid,SIGKILL);
	return(-1);
      }
    } else { /* child */
      execlp("xmms","xmms",NULL);
    }
  }

  fd = open("/dev/ir",O_RDONLY);
  if (fd < 0) {
    perror("Cannot open IR device");
    return(-1);
  }

  while (1) {
    if (sizeof(int) != read(fd,&tmp,sizeof(int))) {
      perror("Cannot read IR device");
      return(-1);
    } else {
      code = tmp & 0xFF;
      switch (code) {
      case EXTIGY_MUTE:
	volq = xmms_remote_get_main_volume(session);
	if (volq == 0) {
	  xmms_remote_set_main_volume(session,savedvol);
	} else {
	  savedvol = volq;
	  xmms_remote_set_main_volume(session,0);
	}
	break;
      case EXTIGY_VOL_MINUS:
	set_vol(session,-5);
	break;
      case EXTIGY_VOL_PLUS:
	set_vol(session,5);
	break;
      case EXTIGY_STOP_EJECT:
	xmms_remote_stop(session);
	break;
      case EXTIGY_PLAY_PAUSE:
	if (xmms_remote_is_playing(session)) {
	  xmms_remote_pause(session);
	} else {
	  xmms_remote_play(session);
	}
	break;
      case EXTIGY_PREVIOUS:
	play_prev(session);
	break;
      case EXTIGY_NEXT:
	xmms_remote_playlist_next(session);
	break;
      case EXTIGY_SLOW:
	skip(session,10000);
	break;
      case EXTIGY_STEP:
	skip(session,60000);
	break;
      case EXTIGY_RIGHT_ARROWS:
	skip(session,2000);
	break;
	case EXTIGY_LEFT_ARROWS:
	  skip(session,-2000);
	  break;
      case EXTIGY_CANCEL:
	xmms_remote_quit(session);
	return(0);
      }
    }
  }
  return(0);
}
