/*
 * V4L2 USB driver for Fujifilm FinePix cameras
 *
 * Copyright (C) 2004 Frank Zago
 * Copyright (C) 2004 Luc Willems
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation, version 2.
 *
 */

#include "finepix.h"

/* taken from the CPiA webcam driver for Linux */

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,19))

/**********************************************************************
 * This is a shameless copy from the USB-cpia driver (linux kernel
 * version 2.3.29 or so, I have no idea what this code actually does ;).
 * Actually it seems to be a copy of a shameless copy of the bttv-driver.
 * Or that is a copy of a shameless copy of ... (To the powers: is there
 * no generic kernel-function to do this sort of stuff?)
 *
 * Yes, it was a shameless copy from the bttv-driver. IIRC, Alan says
 * there will be one, but apparentely not yet - jerdfelt
 *
 **********************************************************************/

/* Given PGD from the address space's page table, return the kernel
 * virtual mapping of the physical memory mapped at ADR.
 */
static inline unsigned long uvirt_to_kva(pgd_t * pgd, unsigned long adr)
{
	unsigned long ret = 0UL;
	pmd_t *pmd;
	pte_t *ptep, pte;

	if (!pgd_none(*pgd)) {
		pmd = pmd_offset(pgd, adr);
		if (!pmd_none(*pmd)) {
			ptep = pte_offset(pmd, adr);
			pte = *ptep;
			if (pte_present(pte)) {
				ret =
				    (unsigned long)page_address(pte_page(pte));
				ret |= (adr & (PAGE_SIZE - 1));
			}
		}
	}
	return ret;
}

/* Here we want the physical address of the memory.
 * This is used when initializing the contents of the
 * area and marking the pages as reserved.
 */
unsigned long kvirt_to_pa(unsigned long adr)
{
	unsigned long va, kva, ret;

	va = VMALLOC_VMADDR(adr);
	kva = uvirt_to_kva(pgd_offset_k(va), va);
	ret = __pa(kva);
	return ret;
}

void *rvmalloc(unsigned long size)
{
	void *mem;
	unsigned long adr, page;

	/* Round it off to PAGE_SIZE */
	size += (PAGE_SIZE - 1);
	size &= ~(PAGE_SIZE - 1);

	mem = vmalloc_32(size);
	if (!mem)
		return NULL;

	memset(mem, 0, size);	/* Clear the ram out, no junk to the user */
	adr = (unsigned long)mem;
	while (size > 0) {
		page = kvirt_to_pa(adr);
		mem_map_reserve(virt_to_page(__va(page)));
		adr += PAGE_SIZE;
		if (size > PAGE_SIZE)
			size -= PAGE_SIZE;
		else
			size = 0;
	}

	return mem;
}

void rvfree(void *mem, unsigned long size)
{
	unsigned long adr, page;

	if (!mem)
		return;

	size += (PAGE_SIZE - 1);
	size &= ~(PAGE_SIZE - 1);

	adr = (unsigned long)mem;
	while (size > 0) {
		page = kvirt_to_pa(adr);
		mem_map_unreserve(virt_to_page(__va(page)));
		adr += PAGE_SIZE;
		if (size > PAGE_SIZE)
			size -= PAGE_SIZE;
		else
			size = 0;
	}
	vfree(mem);
}
#else				/*(LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,19)) */

/* Here we want the physical address of the memory.
 * This is used when initializing the contents of the area.
 */
static unsigned long kvirt_to_pa(unsigned long adr)
{
	unsigned long kva, ret;

	kva = (unsigned long)page_address(vmalloc_to_page((void *)adr));
	kva |= adr & (PAGE_SIZE - 1);	/* restore the offset */
	ret = __pa(kva);
	return ret;
}

void *rvmalloc(unsigned long size)
{
	void *mem;
	unsigned long adr;

	size = PAGE_ALIGN(size);
	mem = vmalloc_32(size);
	if (!mem)
		return NULL;

	memset(mem, 0, size);	/* Clear the ram out, no junk to the user */
	adr = (unsigned long)mem;
	while (size > 0) {
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,69))
		mem_map_reserve(vmalloc_to_page((void *)adr));
#else				/* LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,69) */
		SetPageReserved(vmalloc_to_page((void *)adr));
#endif
		adr += PAGE_SIZE;
		size -= PAGE_SIZE;
	}

	return mem;
}

void rvfree(void *mem, unsigned long size)
{
	unsigned long adr;

	if (!mem)
		return;

	adr = (unsigned long)mem;
	while ((long)size > 0) {
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,69))
		mem_map_unreserve(vmalloc_to_page((void *)adr));
#else				/* LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,69) */
		ClearPageReserved(vmalloc_to_page((void *)adr));
#endif
		adr += PAGE_SIZE;
		size -= PAGE_SIZE;
	}
	vfree(mem);
}
#endif				/*(LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,19)) */

/* Queue a new frame. frame_num is valid.
 * Returns <0 on error. */
int queue_frame(struct usb_fpix *dev, int frame_num)
{
	struct fpix_frame *frame = &dev->frame[frame_num];
	unsigned long flags;
	int ret;

	spin_lock_irqsave(&dev->lists_lock, flags);

	if (frame->flags & V4L2_BUF_FLAG_QUEUED) {
		/* Already queued */
		dev_err(&dev->udev->dev, "App is trying to queue an already queued buffer (index=%d)\n", frame_num);
		ret = -EINVAL;
	} else {
		/* Remove from either the done or free list */
		list_del(&frame->item);
		list_add_tail(&frame->item,
					  &dev->queued_buffers);

		frame->flags =
			V4L2_BUF_FLAG_MAPPED |
			V4L2_BUF_FLAG_QUEUED;
		ret = 0;
	}

	spin_unlock_irqrestore(&dev->lists_lock, flags);

	if (ret == 0) {
		start_streaming(dev);
	}

	return ret;
}

/* Start streaming */
void start_streaming(struct usb_fpix *dev)
{
	down(&dev->sem);

	if (dev->state == FPIX_NOP) {

		init_completion(&dev->can_close);

		dev_new_state(FPIX_REQ_FRAME_1);

		schedule_delayed_work(&dev->wqe, 1);
	}

	up(&dev->sem);
}

/* Stop streaming */
void stop_streaming(struct usb_fpix *dev)
{
	/* Stop the state machine */
	down(&dev->sem);

	if (dev->state != FPIX_NOP) {
		dev->must_stop = 1;
		up(&dev->sem);

		wait_for_completion(&dev->can_close);
	} else {
		up(&dev->sem);
	}
}

/*--------------------------------------------------------------------------*/

int fpix_mmap_jpeg(struct file *file, struct vm_area_struct *vma)
{
	struct usb_fpix *dev = file->private_data;
	unsigned long start = vma->vm_start;
	unsigned long size = vma->vm_end - vma->vm_start;
	void *pos;
	unsigned int index;
	unsigned long offset = vma->vm_pgoff << PAGE_SHIFT;

	dev_dbg(&dev->udev->dev, "%s\n", __FUNCTION__);

	down(&dev->sem);

	index = offset / FPIX_FRAME_SIZE;

	if (index > FPIX_MAX_FRAMES) {
		up(&dev->sem);
		dev_err(&dev->udev->dev, "app error: invalid index (%d > %d)\n", index,
		    FPIX_MAX_FRAMES);
		return -EINVAL;
	}

	/* Compute the offset in the frame */
	offset %= FPIX_FRAME_SIZE;

	if (offset + size > FPIX_FRAME_SIZE) {
		up(&dev->sem);
		dev_err(&dev->udev->dev, "app error: invalid size (%lx, %lx, %lx)\n", start, size,
		    offset);
		return -EINVAL;
	}

	vma->vm_flags |= VM_LOCKED;
	pos = &dev->frame[index].buffer[offset];
	dev->frame[index].buffer[0] = 'F';

	if (remap_page_range
	    (vma, start, virt_to_phys(pos), size, vma->vm_page_prot)) {
		up(&dev->sem);
		return -EAGAIN;
	}

	dev->mmapped = 1;

	up(&dev->sem);

	return 0;
}

int fpix_mmap_rgb(struct file *file, struct vm_area_struct *vma)
{
	struct usb_fpix *dev = file->private_data;
	unsigned long start = vma->vm_start;
	unsigned long size = vma->vm_end - vma->vm_start;
	unsigned long page, pos;

	dev_dbg(&dev->udev->dev, "%s\n", __FUNCTION__);

	down(&dev->sem);
	vma->vm_flags |= VM_LOCKED;
	pos = (unsigned long)(dev->rgb24buf);
	while (size > 0) {
		page = kvirt_to_pa(pos);
		//dbg("%s start=%lx page=%lx size=%ld pos=%d ",__FUNCTION__,start,page,size,pos);   
		if (remap_page_range(vma, start, page, PAGE_SIZE, PAGE_SHARED)) {
			up(&dev->sem);
			return -EAGAIN;
		}
		start += PAGE_SIZE;
		pos += PAGE_SIZE;
		if (size > PAGE_SIZE)
			size -= PAGE_SIZE;
		else
			size = 0;
	}
	up(&dev->sem);
	return 0;
}
