/*
 * V4L2 USB driver for Fujifilm FinePix cameras
 *
 * Copyright (C) 2004 Frank Zago
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation, version 2.
 *
 */

#include "finepix.h"

int fpix_do_ioctl_v4l2(struct usb_fpix *dev,
		       struct inode *inode, struct file *file,
		       unsigned int cmd, void *arg)
{
	int ret;

      dqbuf_again:

	if (dev->present == 0) {
		/* TODO: correct return code? */
		return -ENODEV;
	}

	switch (cmd) {

	case VIDIOC_ENUMINPUT:{
			struct v4l2_input *input = arg;

			if (input->index != 0) {
				ret = -EINVAL;
			} else {
				strlcpy(input->name, "USB",
					sizeof(input->name));
				input->type = V4L2_INPUT_TYPE_CAMERA;
				input->audioset = 0;
				input->tuner = 0;
				input->std = V4L2_STD_UNKNOWN;
				input->status = 0;
				memset(input->reserved, 0,
				       sizeof(input->reserved));

				ret = 0;
			}
		}
		break;

	case VIDIOC_ENUMSTD:{
			struct v4l2_standard *standard = arg;

			if (standard->index != 0) {
				ret = -EINVAL;
			} else {
				standard->id = V4L2_STD_UNKNOWN;
				strlcpy(standard->name, "FinePix JPEG",
					sizeof(standard->name));
				standard->frameperiod.numerator = 1;
				standard->frameperiod.denominator = 8;
				standard->framelines = 240;
				memset(standard->reserved, 0,
				       sizeof(standard->reserved));

				ret = 0;
			}
		}
		break;

	case VIDIOC_ENUM_FMT:{
			struct v4l2_fmtdesc *fmtdesc = arg;

			/* There only one format supported. */
			if (fmtdesc->index != 0) {
				ret = -EINVAL;
			} else {

				fmtdesc->type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
				fmtdesc->flags = V4L2_FMT_FLAG_COMPRESSED;
				strlcpy(fmtdesc->description, "JPEG",
					sizeof(fmtdesc->description));
				fmtdesc->pixelformat = V4L2_PIX_FMT_JPEG;
				memset(fmtdesc->reserved, 0,
				       sizeof(fmtdesc->reserved));
				ret = 0;
			}
		}
		break;

	case VIDIOC_G_PARM:{
			struct v4l2_streamparm *streamparm = arg;
			struct v4l2_captureparm *captureparm =
			    &streamparm->parm.capture;

			captureparm->capability = 0;
			captureparm->capturemode = V4L2_MODE_HIGHQUALITY;
			captureparm->extendedmode = 0;
			captureparm->readbuffers = 1;
			memset(captureparm->reserved, 0,
			       sizeof(captureparm->reserved));

			ret = 0;
		}
		break;

	case VIDIOC_QUERYCTRL:
		/* This device doesn't support anything */
		ret = -EINVAL;
		break;

	case VIDIOC_G_STD:{
			v4l2_std_id *std_id = arg;

			*std_id = V4L2_STD_UNKNOWN;
			ret = 0;
		}
		break;

	case VIDIOC_G_INPUT:{
			int *input = arg;

			*input = 0;

			ret = 0;
		}
		break;

	case VIDIOC_G_FMT:{
			struct v4l2_format *format = arg;

			if (format->type == V4L2_BUF_TYPE_VIDEO_CAPTURE) {
				struct v4l2_pix_format *pix_format =
				    &format->fmt.pix;

				pix_format->width = 320;
				pix_format->height = 240;
				pix_format->pixelformat = V4L2_PIX_FMT_JPEG;
				pix_format->field = V4L2_FIELD_NONE;
				pix_format->bytesperline = 320 * 3;
				pix_format->sizeimage = 320 * 240 * 3;
				pix_format->colorspace = V4L2_COLORSPACE_JPEG;
				pix_format->priv = 0;

				ret = 0;
			} else {
				ret = -EINVAL;
			}
		}
		break;

	case VIDIOC_S_FMT:{
			struct v4l2_format *format = arg;

			if (format->type == V4L2_BUF_TYPE_VIDEO_CAPTURE) {
				struct v4l2_pix_format *pix_format =
				    &format->fmt.pix;

				/* Force the app to use our parameters. */
				pix_format->width = 320;
				pix_format->height = 240;
				pix_format->pixelformat = V4L2_PIX_FMT_JPEG;
				pix_format->field = V4L2_FIELD_NONE;
				pix_format->bytesperline = 320 * 3;
				pix_format->sizeimage = 320 * 240 * 3;
				pix_format->colorspace = V4L2_COLORSPACE_JPEG;
				pix_format->priv = 0;

				ret = 0;
			} else {
				ret = -EINVAL;
			}
		}
		break;

	case VIDIOC_TRY_FMT:{
			struct v4l2_format *format = arg;

			if (format->type != V4L2_BUF_TYPE_VIDEO_CAPTURE) {
				ret = -EINVAL;
			} else {
				struct v4l2_pix_format *pix_format =
				    &format->fmt.pix;

#if 0
				dbg("  FMT: %d %d %x %d %d %d %d %d",
				    pix_format->width,
				    pix_format->height,
				    pix_format->pixelformat,
				    pix_format->field,
				    pix_format->bytesperline,
				    pix_format->sizeimage,
				    pix_format->colorspace, pix_format->priv);
#endif

				/* Force the app to use our parameters. */
				pix_format->width = 320;
				pix_format->height = 240;
				pix_format->pixelformat = V4L2_PIX_FMT_JPEG;
				pix_format->field = V4L2_FIELD_NONE;
				pix_format->bytesperline = 320 * 3;
				pix_format->sizeimage = 320 * 240 * 3;
				pix_format->colorspace = V4L2_COLORSPACE_JPEG;
				pix_format->priv = 0;

				ret = 0;
			}
		}
		break;

	case VIDIOC_S_INPUT:{
			ret = 0;
		}
		break;

	case VIDIOC_CROPCAP:
		ret = -EINVAL;
		break;

	case VIDIOC_REQBUFS:{
			struct v4l2_requestbuffers *requestbuffers = arg;

			if (requestbuffers->memory == V4L2_MEMORY_MMAP &&
			    requestbuffers->type ==
			    V4L2_BUF_TYPE_VIDEO_CAPTURE) {

				/* Diregard what the application wants. */
				requestbuffers->count = FPIX_MAX_FRAMES;
				ret = 0;

			} else {
				/* Method not supported */
				ret = -EINVAL;
			}
		}
		break;

	case VIDIOC_QUERYBUF:{
			struct v4l2_buffer *buffer = arg;

			if (buffer->type == V4L2_BUF_TYPE_VIDEO_CAPTURE &&
			    buffer->index < FPIX_MAX_FRAMES) {

				buffer->memory = V4L2_MEMORY_MMAP;
				buffer->m.offset =
				    buffer->index * FPIX_FRAME_SIZE;
				buffer->length = FPIX_FRAME_SIZE;
				buffer->flags = dev->frame[buffer->index].flags;

				ret = 0;
			} else {
				dev_err(&dev->udev->dev, "unsupported VIDIOC_QUERYBUF (%d, %d)\n",
				    buffer->type, buffer->index);
				ret = -EINVAL;
			}
		}
		break;

	case VIDIOC_QBUF:{
			struct v4l2_buffer *buffer = arg;

			if (buffer->type == V4L2_BUF_TYPE_VIDEO_CAPTURE &&
			    buffer->index < FPIX_MAX_FRAMES) {
				ret = queue_frame(dev, buffer->index);
			} else {
				dev_err(&dev->udev->dev, "unsupported VIDIOC_QBUF (%d, %d)\n",
				    buffer->type, buffer->index);
				ret = -EINVAL;
			}
		}
		break;

	case VIDIOC_DQBUF:{
			struct v4l2_buffer *buffer = arg;

			if (buffer->type == V4L2_BUF_TYPE_VIDEO_CAPTURE) {

				unsigned long flags;

				spin_lock_irqsave(&dev->lists_lock, flags);

				if (list_empty(&dev->done_buffers)) {
					if (dev->nonblocking || dev->state == FPIX_NOP) {
						ret = -EAGAIN;
					} else {
						spin_unlock_irqrestore(&dev->
								       lists_lock,
								       flags);
						wait_for_completion(&dev->
								    frame_done);
						goto dqbuf_again;
					}
				} else {
					/* Remove the oldest frame */
					struct fpix_frame *frame =
					    list_entry(dev->done_buffers.next,
						       struct fpix_frame, item);

					list_del(dev->done_buffers.next);
					list_add_tail(&frame->item,
						      &dev->free_buffers);

					frame->flags = buffer->flags =
					    V4L2_BUF_FLAG_MAPPED;

					buffer->index = frame->index;
					buffer->bytesused = frame->offset;

					ret = 0;
				}

				spin_unlock_irqrestore(&dev->lists_lock, flags);
			} else {
				dev_err(&dev->udev->dev, "unsupported VIDIOC_DQBUF (%d, %d)\n",
				    buffer->type, buffer->index);
				ret = -EINVAL;
			}
			ret = 0;
		}
		break;

	case VIDIOC_STREAMON:
		start_streaming(dev);
		ret = 0;
		break;

	case VIDIOC_STREAMOFF:
		stop_streaming(dev);
		ret = 0;
		break;

	default:
		/* Let's say everything else is not supported. */
		ret = -EINVAL;
	}

	return ret;
}
