#!/usr/bin/env python
import array
import fcntl
import sys

v4lfile = "/dev/video0"
BESProcfile = "/proc/mgavideo/mga0/bes"

def guessVals():
    """Try to get the colorkey by running the 'xvinfo' program."""
    import os
    import re
    print "Scanning for current expected xv colorkey.."
    f = os.popen('xvinfo')
    lines = f.read()
    f.close()
    m = re.search(r'depth (\d+), visualID ', lines)
    if not m:
        print "Unable to determine screen depth"
        sys.exit(1)
    depth = int(m.group(1))
    m = re.search(r'client gettable attribute \(current value is (\d+)\)'
                  , lines)
    if not m:
        print "Unable to determine colorkey"
        sys.exit(1)
    key = int(m.group(1))
    if depth == 16:
        r = (key>>11) & 0x1F
        g = (key>>5) & 0x3F
        b = key & 0x1F
    elif depth == 15:
        r = (key>>10) & 0x1F
        g = (key>>5) & 0x1F
        b = key & 0x1F
    elif depth == 24:
        r = (key>>16) & 0xFF
        g = (key>>8) & 0xFF
        b = key & 0xFF
    else:
        print "Unknown depth", depth
        sys.exit(1)
    print "Setting rgb value of: %d %d %d" % (r,g,b)
    return r,g,b

def setmgatv(red, green, blue):
    """Set chroma key using mgatv ioctl method."""
    rgb = red << 24 | green << 16 | blue << 8
    f = open(v4lfile, 'rwb')
    # ioctl number
    mgatv_forcechroma = -2147191096
    # type, left, top, width, height
    data = array.array('i', [1, rgb])
    fcntl.ioctl(f.fileno(), mgatv_forcechroma, data.tostring())

def setdriverfb(red, green, blue):
    """Set chroma key using /proc interface of driver_fb/mgavideo."""
    try:
        f = open(BESProcfile, 'w')
    except IOError:
        return -1
    f.write("color_key_rgb %d %d %d\n" % (red, green, blue))
    f.close()
    return 0;

def main():
    if len(sys.argv) == 1:
        red, green, blue = guessVals()
    elif len(sys.argv) == 4:
        red = int(sys.argv[1])
        green = int(sys.argv[2])
        blue = int(sys.argv[3])
    else:
        print "Usage:\n%s [<red> <green> <blue>]" % (sys.argv[0],)
        sys.exit(1)
    ret = setdriverfb(red, green, blue)
    if ret:
        setmgatv(red, green, blue)

if __name__ == '__main__':
    main()


