/*
 *
 *    Connectix USB QuickCam VC Video Camera driver
 *
 *    Copyright 2001 De Marchi Daniele
 *    Copyright 2004 Terry Mohan
 *    Copyright 2005 Troy Rollo
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of the
 *    License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Jan 2001 This driver develop started on the linux 
 *            kernel 2.4.0.
 *
 */

#include <linux/config.h>
#include <linux/version.h>

#if (LINUX_VERSION_CODE <= KERNEL_VERSION(2,5,0))
#include <linux/wrapper.h>
#endif
#ifdef CONFIG_KMOD
#include <linux/kmod.h>
#endif

#include <linux/proc_fs.h>
#include <linux/vmalloc.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/sched.h>
#include <linux/ctype.h>
#include <linux/parport.h>
#include <linux/delay.h>

#include "qcamvc.h"

//*#define _QCAMVC_DEBUG_
//#define CONFIG_VIDEO_QCAMVC_PP_MODULE
//#undef CONFIG_PROC_FS

static int video_nr = -1;

#ifdef MODULE
MODULE_PARM(video_nr,"i");
MODULE_AUTHOR("De Marchi Daniele <demarchidaniele@libero.it>");
MODULE_DESCRIPTION("V4L-driver for QuickCam VC cameras");
MODULE_LICENSE("GPL");
MODULE_SUPPORTED_DEVICE("video");
#endif

#ifdef CONFIG_VIDEO_QCAMVC_PP
extern int qcamvc_pp_init(void);
#endif
#ifdef CONFIG_VIDEO_QCAMVC_USB
extern int qcamvc_usb_init(void);
#endif

static int		qcamvc_vopen(struct inode *inode, struct file *file);
static int		qcamvc_vrelease(struct inode *inode, struct file *file);
static int		qcamvc_vioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);
static ssize_t	qcamvc_vread(struct file *file, char *data, size_t count, loff_t *ppos);
static int		qcamvc_vmmap(struct file *file, struct vm_area_struct *vma);
static int		qcamvc_do_ioctl(struct inode *inode, struct file *file, unsigned int cmd, void *arg);

static void		qcamvc_parse(struct qcamvc_data *qcamvc, unsigned char *outbuf);
static int		qcamvc_camera_init(struct qcamvc_data *qcamvc);

static struct file_operations qcamvc_fops =
{
	.owner		= THIS_MODULE,
	.open		= qcamvc_vopen,
	.release	= qcamvc_vrelease,
	.ioctl		= qcamvc_vioctl,
	.llseek		= no_llseek,
	.read		= qcamvc_vread,
	.mmap		= qcamvc_vmmap,
};

static struct video_device qcamvc_template =
{
	.name		= "UNSET",
	.type		= VID_TYPE_CAPTURE,
	.hardware	= VID_HARDWARE_QCAM_C,
	.fops		= &qcamvc_fops,
	.minor		= -1,
};

#if LINUX_VERSION_CODE>=KERNEL_VERSION(2,6,10) 
 /* For kernel 2.6.10 and above -- Vikram */ 
int remap_pfn_range(struct vm_area_struct *vma, unsigned long uvaddr, 
		  unsigned long pfn, unsigned long size, pgprot_t prot); 
  
static inline int remap_page_range(struct vm_area_struct *vma, 
				unsigned long uvaddr, 
				unsigned long paddr, 
				unsigned long size, pgprot_t prot) 
{ 
	return remap_pfn_range(vma, uvaddr, paddr >> PAGE_SHIFT, size, prot); 
} 
#endif 

/**********************************************************************
 *
 * Memory management
 *
 **********************************************************************/

/* Here we want the physical address of the memory.
 * This is used when initializing the contents of the area.
 */
static inline unsigned long kvirt_to_pa(unsigned long adr)
{
	unsigned long kva, ret;

	kva = (unsigned long) page_address(vmalloc_to_page((void *)adr));
	kva |= adr & (PAGE_SIZE-1);
	ret = __pa(kva);
	return ret;
}

static void *rvmalloc(unsigned long size)
{
	void *mem;
	unsigned long adr;

	size = PAGE_ALIGN(size);
	mem = vmalloc_32(size);
	if (!mem)
		return NULL;

	memset(mem, 0, size);
	adr = (unsigned long) mem;
	while (size > 0) {
		SetPageReserved(vmalloc_to_page((void *)adr));
		adr += PAGE_SIZE;
		size -= PAGE_SIZE;
	}

	return mem;
}

static void rvfree(void *mem, unsigned long size)
{
	unsigned long adr;

	if (!mem)
		return;

	adr = (unsigned long) mem;
	while ((long) size > 0) {
		ClearPageReserved(vmalloc_to_page((void *)adr));
		adr += PAGE_SIZE;
		size -= PAGE_SIZE;
	}
	vfree(mem);
}

static inline int allocate_raw_frame(struct qcamvc_data *qcamvc)
{
	if (!qcamvc)
		return -EFAULT;
	
	if (!qcamvc->raw_frame)
	{
		qcamvc->raw_frame = kmalloc(MAX_RAW_IMAGE_SIZE, GFP_KERNEL);
		if (!qcamvc->raw_frame)
			return -ENOBUFS;
	}
	
	/*if ( (qcamvc->bpc == 6) && !qcamvc->raw_frame_temp )
	{
		qcamvc->raw_frame_temp = kmalloc(MAX_RAW_IMAGE_SIZE, GFP_KERNEL);
		if (!qcamvc->raw_frame_temp)
			return -ENOBUFS;
	}*/

	return 0;
}

static void free_raw_frame(struct qcamvc_data *qcamvc)
{
	if (!qcamvc) return;
	
	if (qcamvc->raw_frame)
	{
		kfree(qcamvc->raw_frame);
		qcamvc->raw_frame = NULL;
	}

	/*if (qcamvc->raw_frame_temp)
	{
		kfree(qcamvc->raw_frame_temp);
		qcamvc->raw_frame_temp = NULL;
	}*/
}

static inline int allocate_frame_buf(struct qcamvc_data *qcamvc)
{
	if (!qcamvc) return -EFAULT;
	if (qcamvc->frame_buf) return 0; //already allocated

	/* tries to allocate all the memory needed for frame buffers */
	qcamvc->frame_buf = rvmalloc(QCAMVC_NUMFRAMES * MAX_FRAME_SIZE);
	if (!qcamvc->frame_buf)
		return -ENOBUFS;

	int i;
	for (i = 0; i < QCAMVC_NUMFRAMES; i++)
	{
		/* stores the starting addresses of each frame buffer in the
		qcamvc->frame_buf */
		qcamvc->frame[i].data = qcamvc->frame_buf + i * MAX_FRAME_SIZE;
		qcamvc->frame[i].state = FRAME_READY;
	}
	
	return 0;
}

static void free_frame_buf(struct qcamvc_data *qcamvc)
{
	if (!qcamvc) return;
	
	if (qcamvc->frame_buf)
	{
		rvfree(qcamvc->frame_buf, QCAMVC_NUMFRAMES * MAX_FRAME_SIZE);
		qcamvc->frame_buf = NULL;
	}

	int i;
	for (i=0; i < QCAMVC_NUMFRAMES; i++)
	{
		qcamvc->frame[i].state = FRAME_UNUSED;
		qcamvc->frame[i].data = NULL;
	}

}

/**********************************************************************
 *
 * Camera Address Read/Write helpers
 *
 **********************************************************************/

static int get_register(struct qcamvc_data *qcamvc, unsigned char reg, unsigned char *buf)
{
	if (!qcamvc || !qcamvc->ops || !buf) return -1;

	if (qcamvc->ops->qcamvc_get_reg(qcamvc->lowlevel_data, reg, buf, 1) != 1)
		return -1;
		
	return 0;
}

static int set_register(struct qcamvc_data *qcamvc, unsigned char reg, unsigned char buf)
{
	if (!qcamvc || !qcamvc->ops) return -1;

	if (qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, reg, &buf, 1) != 1)
		return -1;
		
	return 0;
}

static int set_registers(struct qcamvc_data *qcamvc, unsigned char reg, unsigned char *buf, size_t size)
{
	if (!qcamvc || !qcamvc->ops || !buf) return -1;

	if (qcamvc->ops->qcamvc_set_reg(qcamvc->lowlevel_data, reg, buf, size) != size)
		return -1;
		
	return 0;
}

/**********************************************************************
 *
 * Image Grabbing
 *
 **********************************************************************/

static int send_grab_image(struct qcamvc_data *qcamvc, int new_frame)
{
	if (new_frame)
		qcamvc->frame_count++; /*fresh frame */
	
	if (set_register(qcamvc, QCAM_VC_GET_FRAME, qcamvc->frame_count))
		return -ENODEV;
	
	return 0;
}
 
/* interrogate the camera every 10ms while waiting for an image to be ready. */
/* return 0 not ready, or >0 ready */
static int frame_is_ready(struct qcamvc_data *qcamvc)
{
#if 0
	if (!qcamvc) return 0;

	struct qcamvc_misc misc = {0};
	int attempt = 200; /* 2 second timeout should be plenty */

	/* send grab frame command with current frame count */
	if (send_grab_image(qcamvc, 0))
		return 0;
	
	do
	{
		if (!get_register(qcamvc, QCAM_VC_SET_MISC, (unsigned char*)&misc))
		{
			/* bit 8 of MISC register is set if the camera has finished capturing a frame */
			if (misc.frame_ready)
			{
				break;
			}
		}
		else
		{
			/* failed reading from the camera. can't do much else here. */
			return 0;
		}

		/* sleep 10ms. ie. about 1/3 of the time it takes to snap an image at 30fps */
		__set_current_state (TASK_INTERRUPTIBLE);
		schedule_timeout ((HZ + 99) / 100);
		if (signal_pending (current))
		{
			return 0;
		}

	}while (--attempt);
	
	return attempt;
#else
	return 1;
#endif
}

/* gets a single raw frame from the camera - waits for it if necessary */
static int get_raw_frame(struct qcamvc_data *qcamvc)
{ 
	if (!qcamvc || !qcamvc->ops)
		return -EINVAL;

	unsigned long oldjif, rate, diff;
	int err;
	size_t size;

	if ( (err=allocate_raw_frame(qcamvc)) )
		return err;

	oldjif = jiffies;
	
	/* wait for it to become ready */
	if (!frame_is_ready(qcamvc))
	{ /* about 2 secounds has elapsed without the camera telling us a frame is ready... */
		printk("Camera timed-out while grabbing frame #%d.\n", qcamvc->frame_count);
		return -ENODEV;
	}

	/* now suck the image data from the camera */
	if (set_register(qcamvc, QCAM_VC_GET_FRAME, qcamvc->frame_count))
		return -ENODEV;
	
	size = qcamvc->ops->qcamvc_stream_read(qcamvc->lowlevel_data, qcamvc->raw_frame, qcamvc->packet_len);

	if (size == 0)
	{
		printk("Failed to read frame #%d from the camera.\n", qcamvc->frame_count);
		return -1;
	}

	/* calc frame rate */
	rate = qcamvc->packet_len * HZ / 1024;
	diff = jiffies-oldjif;
	qcamvc->transfer_rate = diff==0 ? rate : rate/diff;

	return size;
}

/* V4L capture frame using mmap double buffering */
static int capture_frame(struct qcamvc_data *qcamvc, int frame)
{
	if (frame < 0 || frame >= QCAMVC_NUMFRAMES)
		return -EINVAL;
	
	qcamvc->curframe = frame;

	if (qcamvc->frame[qcamvc->curframe].state == FRAME_READY)
	{
		/* nothing queued... send grab frame to camera now */
		if (!qcamvc->frame_waiting)
		{
			if (send_grab_image(qcamvc, 1))
				return -ENODEV;
		}
		else
		{
			/* queue this so that it can be captured immediately next */
			qcamvc->frame_waiting++;
		}
		
		qcamvc->frame[qcamvc->curframe].state = FRAME_GRABBING;
	}
	else if (qcamvc->frame[qcamvc->curframe].state == FRAME_GRABBING)
	{
		int count = get_raw_frame(qcamvc);

		/* send grab frame now for next frame */
		if (qcamvc->frame_waiting)
		{
			qcamvc->frame_waiting--;
			send_grab_image(qcamvc, 1);
		}

		if (count < 0)
		{
			qcamvc->frame_waiting = 0; /* clear queue on errors */
			qcamvc->frame[qcamvc->curframe].state = FRAME_READY;
			return count;
		}
		
		qcamvc_parse(qcamvc, qcamvc->frame[qcamvc->curframe].data);
		qcamvc->frame[qcamvc->curframe].state = FRAME_DONE;
	}

	return 0;
}

/**********************************************************************
 *
 * Camera Settings
 *
 **********************************************************************/

static inline void make_valid_res(int *width, int *height)
{
	if (*width > MAX_WIDTH) *width = MAX_WIDTH;
	else if (*width < MIN_WIDTH) *width = MIN_WIDTH;
	
	if (*height > MAX_HEIGHT) *height = MAX_HEIGHT;
	else if (*height < MIN_HEIGHT) *height = MIN_HEIGHT;
}
 
static inline int res_changed(struct qcamvc_data *qcamvc, int width, int height)
{
	if (!qcamvc) return 0;
	
	if (qcamvc->width != width) return 1;
	if (qcamvc->height != height) return 1;

	return 0;
}

static inline int valid_res(int width, int height)
{
	if (width > MAX_WIDTH) return 0;
	else if (width < MIN_WIDTH) return 0;
	
	if (height > MAX_HEIGHT) return 0;
	else if (height < MIN_HEIGHT) return 0;
	
	return 1;
}

static int mode_depth(__u32 mode)
{
	switch (mode)
	{
	case V4L2_PIX_FMT_RGB24:
	case V4L2_PIX_FMT_BGR24:
		return 24;

	case V4L2_PIX_FMT_RGB32:
	case V4L2_PIX_FMT_BGR32:
		return 32;

	case V4L2_PIX_FMT_RGB555:
	case V4L2_PIX_FMT_RGB565:
		return 16;

	case V4L2_PIX_FMT_GREY:
		return 8;

	case V4L2_PIX_FMT_YUYV:
	case V4L2_PIX_FMT_UYVY:
		return 16;
	}
	return 0;
}

static int mode_bpc(__u32 mode)
{
	switch (mode)
	{
	case V4L2_PIX_FMT_RGB555:
	case V4L2_PIX_FMT_RGB565:
		return 6;
	}
	return 8;
}

static enum v4l2_colorspace mode_colorspace(__u32 mode)
{
	switch (mode)
	{
	case V4L2_PIX_FMT_RGB24:
	case V4L2_PIX_FMT_BGR24:
	case V4L2_PIX_FMT_RGB32:
	case V4L2_PIX_FMT_BGR32:
	case V4L2_PIX_FMT_RGB555:
	case V4L2_PIX_FMT_RGB565:
		return V4L2_COLORSPACE_SRGB;

	case V4L2_PIX_FMT_GREY:
	case V4L2_PIX_FMT_YUYV:
	case V4L2_PIX_FMT_UYVY:
		return V4L2_COLORSPACE_SMPTE170M;
	}
	return 0;
}

/* calculate the CCD area rows & cols, return final width and height */
static unsigned int calc_CCD_area(int width, int height, struct qcamvc_ccd_area *ccd_area)
{
	if (!ccd_area) return 0;
	
	/* allow only even number of row/columns */
	width &= ~1;
	height &= ~1;
	
	ccd_area->multiplier = 0;
	
	/* sane values for width & height */
	make_valid_res(&width, &height);
	
	/* anything over 176x144 will be doubled by the camera */
	if ( (width > 176) || (height > 144) )
	{
		ccd_area->multiplier = 1;
		width = width >> 1;
		height = height >> 1;
		make_valid_res(&width, &height);
	}
	
	/* width center point is 180/2, +2 because of the left two optically black CCD cells */
	ccd_area->first_col = 92 - (width >> 1);
	ccd_area->last_col = ccd_area->first_col + width;
	
	/* height center point is 144/2, +1 because of 1's based rows */
	/* camera divides the start row when the height is multiplied, so double it here */
	ccd_area->first_row = (73 - (height >> 1)) << ccd_area->multiplier;
	ccd_area->last_row = ccd_area->first_row + height;
	
	/* save the new width and height for no particular reason */
	ccd_area->width = width << ccd_area->multiplier;
	ccd_area->height = height << ccd_area->multiplier;

	return ((width << 8)|(height));
}

/* calculate the packet size for the current image size & bpc */
static inline int calc_packetlength(struct qcamvc_data *qcamvc, int bpc)
{
	if (!qcamvc) return 0; /* let's not panic */
	
	/* plus 2 is for the two packet length bytes at the start of the image data */
	int pl = ((qcamvc->width * qcamvc->height * bpc) / 8) + 64;
	
	if (pl > MAX_PACKET_LENGTH)
	{ // this should never happen
		pl = MAX_PACKET_LENGTH;
	}
	
	return pl;
}

/* raw is a pointer to the start of raw image data */
static inline int header_packetlength(unsigned char *raw)
{
	if (!raw) return 0;
	
	/* the pl in raw data header is half the actual packet length. so x2. */
	int pl = (int)(((int)raw[1] << 8) | raw[0]) * 2;

	if (pl > MAX_PACKET_LENGTH)
	{ // bad raw packet received?
		return 0;
	}
	
	return pl;
}

/* raw is a pointer to the start of raw image data. packetsize to verify. */
static inline int valid_packet(unsigned char *raw, int packetSize)
{
	if (!raw) return 0;
	
	return ( header_packetlength(raw) == packetSize );
}

static int qcamvc_set_ccd_area(struct qcamvc_data *qcamvc)
{
	if (!qcamvc) return -1;
	
	calc_CCD_area(qcamvc->width, qcamvc->height, &qcamvc->ccd_area);
	qcamvc->width = qcamvc->ccd_area.width;
	qcamvc->height = qcamvc->ccd_area.height;

	/* new packet lengths for new image size & bpc */
	qcamvc->packet_len = calc_packetlength(qcamvc, qcamvc->bpc);
	qcamvc->packet_len_bpc8 = calc_packetlength(qcamvc, 8);

	/* write out 1st 4 bytes of ccd_area */
	if (set_registers(qcamvc, QCAM_VC_SET_CCD_AREA, (unsigned char*)&qcamvc->ccd_area, 4))
		return -1;
	
	return 0;
}

static int qcamvc_set_res(struct qcamvc_data *qcamvc, int width, int height)
{
	/* make sure the width and height are valid, or bad things will happen */
	make_valid_res(&width, &height);
	
	if (down_interruptible(&qcamvc->busy_lock))
		return -1;

	qcamvc->width = width;
	qcamvc->height = height;
	
	/* set CCD columns & rows */
	qcamvc_set_ccd_area(qcamvc);

	/* set multiplier bit after res change */
	qcamvc->misc_reg.multiplier = (qcamvc->ccd_area.multiplier ? 0 : 1);
	set_registers(qcamvc, QCAM_VC_SET_MISC, (unsigned char*)&qcamvc->misc_reg, 1);

	/* clear the current frame by asking for a new one */
	send_grab_image(qcamvc, 1);

	up(&qcamvc->busy_lock);
	
	return 0;
}

static void qcamvc_set_exposure(struct qcamvc_data *qcamvc,  unsigned char val)
{
	unsigned char buffer[2];
	int ival;

	/* No idea why exposure range is 294 -> 1, 295 -> 16383, but that's what it is. */
	/* Exposure zero and one, appear to be the same thing. */

	ival = val + 1;
	ival = (ival < 4) ? ival : ((ival * ival) >> 2); /* linear below 4 */
	
	if(ival < 295)
	{
		ival = 295 - ival;
	}
	else if(ival > 16383)
	{
		ival = 16383;
	}

	buffer[0] = ival;
	buffer[1] = ival >> 8;

	set_registers(qcamvc, QCAM_VC_SET_EXPOSURE, buffer, 2);
}

static void qcamvc_set_brightness(struct qcamvc_data *qcamvc,  unsigned char val)
{
	short i,j;
	unsigned char buffer[21]=
		{
		0x58, 0xd8, 0x58, 0xd8, 0x58,
		0xd8, 0x58, 0xd8, 0x58, 0xd8,
		0x58, 0xd8, 0x58, 0xd8, 0x58,
		0xd8, 0x58, 0xd8, 0x58, 0xd8, 0x5c
		};
	
	for(i=1, j=18 ; i<=0x80 ; i*=2 , j-=2)
	{
		if( val & i)
		{
			buffer[j] = buffer[j] | 1;
			buffer[j+1] = buffer[j+1] | 1;
		}
	}

	set_registers(qcamvc, QCAM_VC_SET_BRIGHTNESS, buffer, sizeof(buffer));
}

static void qcamvc_set_light_sensitivity(struct qcamvc_data *qcamvc,  unsigned char val)
{
	unsigned char buffer[2];
	
	buffer[0] = 255-val;	/* attenuation */
	buffer[1] = val;		/* gain */
	
	set_registers(qcamvc, QCAM_VC_SET_LIGHTSENS_LO, buffer, 2);
}

/*
	have to get this checked on a USB & quickcam Pro to make sure 
	address 0x00 & 0x01 are in fact, camera model/type.
*/
static int get_camera_model(struct qcamvc_data *qcamvc)
{
	unsigned char model, type;

	if (get_register(qcamvc, 0x00, &model))
		return -1;
	if (get_register(qcamvc, 0x01, &type))
		return -1;
	
	/* 
	for quickcam vc...
	address 0x00 always 0x02 (2)
	address 0x01 always 0x0A (16)
	*/
	
	qcamvc->camera_model = model;
	qcamvc->camera_type = type;
	
	//printk("Camera: %d / %d\n", model, type);
	
	return 0;
}

/* Table of ln(x/256) * 256 for x going from 0 -> 255 */
static const int gamma_helper[256] =
{
	0, -1420, -1242, -1138, -1065, -1007, -961, -921, -887, -857, -830, -806, -783, -762, -744, -726,
	-710, -694, -679, -666, -652, -640, -628, -617, -606, -596, -586, -576, -567, -577, -549, -541,
	-532, -525, -517, -509, -502, -495, -488, -482, -475, -469, -463, -457, -451, -455, -439, -434,
	-429, -423, -418, -413, -408, -403, -398, -394, -389, -385, -380, -376, -371, -367, -363, -359,
	-355, -351, -347, -343, -339, -336, -332, -328, -325, -321, -318, -314, -311, -308, -304, -301,
	-298, -295, -291, -288, -285, -282, -279, -276, -273, -271, -268, -265, -262, -259, -257, -254,
	-251, -248, -246, -243, -241, -238, -236, -233, -231, -228, -226, -223, -221, -219, -216, -214,
	-212, -209, -207, -205, -203, -200, -198, -196, -194, -192, -190, -188, -186, -184, -182, -180,
	-178, -176, -174, -172, -170, -168, -166, -164, -162, -160, -158, -156, -155, -153, -151, -149,
	-147, -146, -144, -142, -140, -139, -137, -135, -134, -132, -130, -128, -127, -125, -124, -122,
	-120, -119, -117, -116, -114, -112, -111, -109, -108, -106, -105, -103, -102, -100, -99, -97,
	-96, -95, -93, -92, -90, -89, -87, -86, -85, -83, -82, -80, -79, -78, -76, -75,
	-74, -72, -71, -70, -68, -67, -66, -65, -63, -62, -61, -59, -58, -57, -56, -54,
	-53, -52, -51, -50, -48, -47, -46, -45, -44, -42, -41, -40, -39, -38, -37, -35,
	-34, -33, -32, -31, -30, -29, -27, -26, -25, -24, -23, -22, -21, -20, -19, -18,
	-17, -16, -14, -13, -12, -11, -10, -9, -8, -7, -6, -5, -4, -3, -2, -1
};

/*
 * Copyright (c) 2002 S. Fuerst
 *
 * This software may be copied and distributed for educational, research, and
 * not for profit purposes provided that this copyright and statement are
 * included in all such copies.

 * Build the gamma table so that floating point isn't needed.
 *
 *  ANGBAND_X11_GAMMA is
 * 256 * (1 / gamma), rounded to integer. A recommended value
 * is 183, which is an approximation of the Macintosh hardware
 * gamma of 1.4.
 *
 *   gamma      ANGBAND_X11_GAMMA
 *   -----      -----------------
 *   1.2        213
 *   1.25       205
 *   1.3        197
 *   1.35       190
 *   1.4        183
 *   1.45       177
 *   1.5        171
 *   1.6        160
 *   1.7        151
 *   ...
 *
 * XXX XXX The environment variable, or better,
 * the interact with colours command should allow users
 * to specify gamma values (or gamma value * 100).
 */
void build_gamma_table(struct qcamvc_data *qcamvc)
{
	if (!qcamvc || qcamvc->gamma_done) return;

	qcamvc->gamma_done = 1;
	int gamma = 170;//qcamvc->contrast;
	
	int i, n;

	/*
	* value is the current sum.
	* diff is the new term to add to the series.
	*/
	long value, diff;

	/* Paranoia */
	if (gamma < 0) gamma = 0;
	else if (gamma > 255) gamma = 255;

	/* Hack - convergence is bad in these cases. */
	qcamvc->gamma_table[0] = 0;
	qcamvc->gamma_table[255] = 255;

	for (i = 1; i < 255; i++)
	{
		/*
		* Initialise the Taylor series
		*
		* value and diff have been scaled by 256
		*/
		n = 1;
		value = 256 * 256;
		diff = ((long)gamma_helper[i]) * (gamma - 256);

		while (diff)
		{
			value += diff;
			n++;
			/*
			* Use the following identiy to calculate the gamma table.
			* exp(x) = 1 + x + x^2/2 + x^3/(2*3) + x^4/(2*3*4) +...
			*
			* n is the current term number.
			*
			* The gamma_helper array contains a table of
			* ln(x/256) * 256
			* This is used because a^b = exp(b*ln(a))
			*
			* In this case:
			* a is i / 256
			* b is gamma.
			*
			* Note that everything is scaled by 256 for accuracy,
			* plus another factor of 256 for the final result to
			* be from 0-255.  Thus gamma_helper[] * gamma must be
			* divided by 256*256 each itteration, to get back to
			* the original power series.
			*/
			diff = (((diff / 256) * gamma_helper[i]) * (gamma - 256)) / (256 * n);
		}
		/*
		* Store the value in the table so that the
		* floating point pow function isn't needed.
		*/
		qcamvc->gamma_table[i] = ((long)(value / 256) * i) / 256;
	}

}

#define LIMIT(x) ((x)>255?255 : ((x)<0?0:x))

static int ybr_convert(struct qcamvc_data *qcamvc,  unsigned char *pout, int Y, int Cb, int Cr, int odd)
{
	unsigned char red=0, green=0, blue=0;
	unsigned bytes = 0;

	Y = qcamvc->gamma_table[ Y ];
	//Y = LIMIT(Y);
	
	Y = Y * qcamvc->contrast * qcamvc->contrast / 16384;
	Cr = Cr * qcamvc->saturation * qcamvc->saturation / 16384;
	Cb = Cb * qcamvc->saturation * qcamvc->saturation / 16384;
	
	switch (qcamvc->pixelformat)
	{
	case V4L2_PIX_FMT_RGB24:
	case V4L2_PIX_FMT_RGB32:
	case V4L2_PIX_FMT_BGR24:
	case V4L2_PIX_FMT_BGR32:
	case V4L2_PIX_FMT_RGB555:
	case V4L2_PIX_FMT_RGB565:
		red =   LIMIT( (Y) + (Cr * 1402 / 1000) );
		green = LIMIT( (Y) - (Cb * 3441 / 10000)  - (Cr * 7141 / 10000) );
		blue =  LIMIT( (Y) + (Cb * 1772 / 1000) );
		break;
	}

	if (qcamvc->flip_bgr)
	{
		/* There has been some confusion regarding why this is necessary.
		 * It is not because of a problem in the driver, but because of
		 * a problem in the V4L2 standard. In V4L1, the V4L_PALETTE_RGB*
		 * entries referred to a B,G,R order. In the first versions of the
		 * V4L2 spec this was also the order, but it was changed fairly
		 * early on. To complicate matters, some specs were published which
		 * showed V4L2_PIX_FMT_RGB* and V4L2_PIX_FMT_BGR* having the
		 * opposite meanings to those they have now. Since some apps will
		 * be using the wrong meanings, we have no choice but to have a
		 * switch, but at least we can switch via settings to
		 * /proc/qcamvc/video0 rather than at module initialisation time.
		 */
		unsigned tmp;

		switch (qcamvc->pixelformat)
		{
		case V4L2_PIX_FMT_RGB24:
		case V4L2_PIX_FMT_RGB32:
		case V4L2_PIX_FMT_BGR24:
		case V4L2_PIX_FMT_BGR32:
			tmp = red;
			red = blue;
			blue = tmp;
			break;
		}
	}

	switch (qcamvc->pixelformat)
	{
	case V4L2_PIX_FMT_BGR32:
		*pout-- = 0;
		++bytes;
	case V4L2_PIX_FMT_BGR24:
		*pout-- = red;
		*pout-- = green;
		*pout-- = blue;
		bytes += 3;
		break;

	case V4L2_PIX_FMT_RGB32:
		*pout-- = 0;
		++bytes;
	case V4L2_PIX_FMT_RGB24:
		*pout-- = blue;
		*pout-- = green;
		*pout-- = red;
		bytes += 3;
		break;

	case V4L2_PIX_FMT_GREY:
		*pout-- = LIMIT(Y);
		++bytes;
		break;

	case V4L2_PIX_FMT_RGB555:
		*pout-- = (green >> 6) | ((blue >> 1) &0x7c);
		*pout-- = (red >> 3) | ((green << 2) & 0xe0);
		bytes += 2;
		break;

	case V4L2_PIX_FMT_RGB565:
		*pout-- = (green >> 5) | (blue & 0xf8);
		*pout-- = (red >> 3) | ((green << 3) & 0xe0);
		bytes += 2;
		break;

	case V4L2_PIX_FMT_YUYV:
		if (odd)
			*pout-- = LIMIT(Cb + 128);
		else
			*pout-- = LIMIT(Cr + 128);
		*pout-- = (Y > 203) ? 219 : (Y + 16);
		bytes += 2;
		break;

	case V4L2_PIX_FMT_UYVY:
		*pout-- = (Y > 203) ? 219 : (Y + 16);
		if (odd)
			*pout-- = LIMIT(Cb + 128);
		else
			*pout-- = LIMIT(Cr + 128);
		bytes += 2;
		break;
	}
	
	return bytes;
}

static void codec(struct qcamvc_data *qcamvc, unsigned char *inbuf, unsigned char *outbuf)
{
	if (!qcamvc) return;
	
	int row,col;
	int start_row, start_col, end_col, end_row, width;
	unsigned char *pin, *pout;
	int rgb_offset;
	char Y, Cb, Cr;

	rgb_offset = qcamvc->colordepth / 8;

	pin			= inbuf;
	pout		= outbuf + (qcamvc->width * qcamvc->height * rgb_offset) - 1;
	start_col	= 0;//qcamvc->ccd_area.first_col;
	start_row	= 1;//(qcamvc->ccd_area.first_row >> qcamvc->ccd_area.multiplier);
	end_col		= start_col + qcamvc->width;
	end_row		= start_row + qcamvc->height;
	width		= qcamvc->width;

	/* do gamma table */
	if (!qcamvc->gamma_done)
		build_gamma_table(qcamvc);

#define ADD(a,b) (((int)*(pin+a) + (int)*(pin+b)))
#define SUB(a,b) (((int)*(pin+a) - (int)*(pin+b)))

	/*
	Demosaic the CCD cells and convert to some usable colourspace...
	Connectix uses just the odd or even fields. Not both.
		So that the max vertical resolution of the 362x582 CCD is 582/2= 291.
		Effectively 290 max, using the one field.
	Each field is given to us as
		odd row: (Mg+Ye), (Gr+Cy)
		even row: (Gr+Ye), (Mg+Cy)
		So 576 CCD cell rows are used for 352x288 resolution.
	
	For this codec to work, the width and height must be even.
	*/
//printk("start row=%d, start col=%d\n", start_row, start_row);	
	for(row = start_row ; row < end_row ; row++)
	{
		//reverse = (row % 4) >> 1;
		
		for (col = start_col ; col < end_col ; col++)
		{

			#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))
			if (current->need_resched)
				schedule ();
			#else
			cond_resched();
			#endif

			
			if (col % 2 == qcamvc->ccd_area.multiplier)
			{ //even column
				
				Y = ADD(0, -1);
				
				if (row % 2 == qcamvc->ccd_area.multiplier)
				{ //odd row
					Cb = -SUB(width, width-1);
					Cr = SUB(0, -1);
				}
				else
				{ //even row
					Cb = -SUB(0, -1);
					Cr = SUB(-width, -width-1);
				}
			}
			else
			{ //odd column
				
				Y = ADD(0, 1);
				
				if (row % 2 == qcamvc->ccd_area.multiplier)
				{ //odd row
					Cb = -SUB(width+1, width);
					Cr = SUB(1, 0);
				}
				else
				{ //even row
					Cb = -SUB(1, 0);
					Cr = SUB(-width+1, -width);
				}
			}

			/* YCbCr -> output format */
			pout -= ybr_convert(qcamvc, pout, Y, Cb, Cr, col & 1);
			
			pin++;
		}
	} 

#undef ADD
#undef SUB

}

static void qcamvc_parse(struct qcamvc_data *qcamvc, unsigned char *outbuf)
{
	unsigned char *inbuf;
	
	if (qcamvc->bpc == 6)
	{
		/* convert 6bit packed data to 8bit data. */
		
		unsigned char *pTo, *pFrom;
		int eop = ((qcamvc->width * qcamvc->height * 3) / 4) + 2;
		int eop8 = qcamvc->width * qcamvc->height + 2;
		
		/* starting at the end of the buffer */
		pFrom = qcamvc->raw_frame + eop - 1;
		
		/* +4 bytes to avoid overlap by the end of this conversion. */
		pTo = qcamvc->raw_frame + eop8 + 4 - 1;

		while(pFrom > qcamvc->raw_frame + 2)
		{
			*(pTo--) = *(pFrom) >> 2;
			*(pTo--) = ((*(pFrom - 1) >> 4) & 0x0f) | ((*(pFrom) << 4) & 0x30);
			*(pTo--) = ((*(pFrom - 2) >> 6) & 0x03) | ((*(pFrom - 1) & 0x0f) << 2);
			*(pTo--) = *(pFrom - 2) & 0x3f;
			
			pFrom -= 3;
		}
		
		/* the start of data is now +4 bytes away +2 bytes
			to skip header data */
		inbuf = qcamvc->raw_frame + 4 + 2;
	}
	else
	{
		inbuf = qcamvc->raw_frame + 2; /* skip first 2 bytes */
	}

#if 1
	codec(qcamvc, inbuf, outbuf);
#else
	/* test - view the raw image */
	size_t i;
	for( i=0; i < qcamvc->width*qcamvc->height*3; i++ ){
		*(outbuf+i) = *(inbuf + (i/3));
	}
#endif

}

static void put_cam(struct qcamvc_camera_ops* ops)
{
	if (ops->owner)
		module_put(ops->owner);
}

/****************************************************************************
 *
 * /proc interface
 *
 ***************************************************************************/

#ifdef CONFIG_PROC_FS

static struct proc_dir_entry *qcamvc_proc_root = NULL;

#define YES_NO(x) ((x) ? "yes" : "no")

static int qcamvc_read_proc(char *page, char **start, off_t off, int count,
			   int *eof, void *data)
{
	char *out = page;
	int len;
	struct qcamvc_data *qcamvc = data;
	
	if (!qcamvc) return 0;
	
	/* ok. no floating point */
	unsigned int fps = qcamvc->packet_len ? ((qcamvc->transfer_rate * 102400) / qcamvc->packet_len) : 0;
	
	// Stay under PAGE_SIZE or bad things will happen :(
	out+=sprintf(out, "driver_version   : %s\n", QCAMVCVERSION);
	out+=sprintf(out, "model            : %s\n", qcamvc->camtypedesc);
	out+=sprintf(out, "max resolution   : %dx%d\n", MAX_WIDTH, MAX_HEIGHT);
	out+=sprintf(out, "min resolution   : %dx%d\n\n", MIN_WIDTH, MIN_HEIGHT);
	out+=sprintf(out, "Settings\n");
	out+=sprintf(out, "resolution       : %dx%d\n", qcamvc->width, qcamvc->height);
	out+=sprintf(out, "bit per component: %d\n", qcamvc->bpc);
	out+=sprintf(out, "contrast         : %d\n", (int)qcamvc->contrast);
	out+=sprintf(out, "brightness       : %d\n", qcamvc->brightness);
	out+=sprintf(out, "exposure         : %d\n", (int)qcamvc->exposure);
	out+=sprintf(out, "hue              : %d\n", (int)qcamvc->hue);
	out+=sprintf(out, "saturation       : %d\n", (int)qcamvc->saturation);
	out+=sprintf(out, "transfer rate    : %dkB/s (%d.%dfps)\n", 
		qcamvc->transfer_rate, fps/100, fps ? (fps-((fps/100)*100)) : 0);
	out+=sprintf(out, "rgb to bgr hack  : %s\n", qcamvc->flip_bgr ? "on" : "off");
	
	out+=sprintf(out, "\n");
	
	len = out - page;
	len -= off;
	if (len < count)
	{
		*eof = 1;
		if (len <= 0) return 0;
	}
	else
	{
		len = count;
	}
	
	*start = page + off;
	
	return len;	
}

static void setparam (struct qcamvc_data *qcamvc, char *name, char *sval)
{
	if (down_interruptible(&qcamvc->busy_lock))
		return;
	
	unsigned int val, err;
	
	if (!qcamvc || !sval) return;
	if (!isdigit(sval[0])) return;

	err = 1;
	val = simple_strtoul(sval, NULL, 0);

#define MATCH(x) (strcmp(name, x)==0)
	
	if (MATCH("brightness") && (val <= 255))
	{
		qcamvc->brightness = val;
		qcamvc_set_brightness(qcamvc,qcamvc->brightness);
		err=0;
	}
	else if (MATCH("exposure") && (val <= 255))
	{		
		qcamvc->exposure = val;
		qcamvc_set_exposure(qcamvc,qcamvc->exposure);
		err=0;
	}
	else if (MATCH("contrast") && (val <= 255))
	{
		//qcamvc->blue_hue = val;
		qcamvc->contrast = val;
		qcamvc->gamma_done=0;
		err=0;
	}
	else if (MATCH("hue") && (val <= 255))
	{
		qcamvc->hue = val;
		err=0;
	}
	else if (MATCH("colour") && (val <= 255))
	{
		qcamvc->saturation = val;
		err=0;
	}
	else if (MATCH("bpc"))
	{
		if ((val == 6) || (val == 8))
		{
			qcamvc->bpc = val;
			qcamvc_camera_init(qcamvc);
			err=0;
		}
		else
		{
			printk("Invalid Range: bpc can only be 6 or 8\n");
		}
	}
	else if (MATCH("flipbgr"))
	{
		if (val == 0 || val == 1)
		{
			qcamvc->flip_bgr = val;
			err = 0;
		}
		else
		{
			printk("Invalid setting: flipbgr can only be 0 or 1\n");
		}
	}
	else if (MATCH("compression"))// && (val <= 255))
	{
		qcamvc->compression=val;
		qcamvc_camera_init(qcamvc);
		err=0;
	}
	else if (MATCH("sensitivity") && (val <= 255))
	{
		qcamvc->light_sens=val;
		qcamvc_set_light_sensitivity(qcamvc,qcamvc->light_sens);
		err=0;
	}

	if (!err)
		printk("Set: %s=%i\n", name, val);
	else
		printk("Couldn't Set: %s=%i\n", name, val);

#undef MATCH
 
	up(&qcamvc->busy_lock);
}

/* copied from zoran_procfs */
static int qcamvc_write_proc(struct file *file, const char *buffer,
                           unsigned long count, void *data)
{
	char *string, *sp;
	char *line, *ldelim, *varname, *svar, *tdelim;

	struct qcamvc_data* qcamvc = (struct qcamvc_data*) data;

	string = sp = vmalloc(count + 1);
	if (!string) {
		printk("write_proc: can not allocate memory\n");
		return -ENOMEM;
	}
	if (copy_from_user(string, buffer, count)) {
		vfree (string);
		return -EFAULT;
	}
	string[count] = 0;

	ldelim = " \t\n";
	tdelim = "=";
	line = strpbrk(sp, ldelim);
	while (line) {
		*line = 0;
		svar = strpbrk(sp, tdelim);
		if (svar) {
			*svar = 0;
			varname = sp;
			svar++;
			setparam(qcamvc, varname, svar);
		}
		sp = line + 1;
		line = strpbrk(sp, ldelim);
	}
	vfree(string);

	return count;
}

static void create_proc_qcamvc_cam(struct qcamvc_data *qcamvc)
{
	char name[8];
	struct proc_dir_entry *ent;
	
	if (!qcamvc_proc_root || !qcamvc)
		return;

	printk(KERN_INFO "QuickCam VC: Creating a camera entry in /proc/qcamvc\n");
	//printk("QuickCam VC: QuickCam dev struct in create_proc_qcamvc %p\n",qcamvc);

	sprintf(name, "video%d", qcamvc->vdev->minor);
	
	ent = create_proc_entry(name, S_IFREG|S_IRUGO|S_IWUSR, qcamvc_proc_root);
	if (!ent)
	{
		printk(KERN_ERR "QuickCam VC: couldn't create /proc/qcamvc/%s\n", name);
		return;
	}

	ent->data = qcamvc;
	ent->read_proc = qcamvc_read_proc;
	ent->write_proc = qcamvc_write_proc;

	ent->size = 0;
	qcamvc->proc_entry = ent;
}

static void destroy_proc_qcamvc_cam(struct qcamvc_data *qcamvc)
{
	char name[8];
	
	if (!qcamvc || !qcamvc->proc_entry)
		return;
	
	sprintf(name, "video%d", qcamvc->vdev->minor);
	remove_proc_entry(name, qcamvc_proc_root);
	qcamvc->proc_entry = NULL;
}

static void proc_qcamvc_create(void)
{
	qcamvc_proc_root = create_proc_entry("qcamvc", S_IFDIR, 0);

	if (qcamvc_proc_root)
		qcamvc_proc_root->owner = THIS_MODULE;
	else
		LOG("Unable to initialise /proc/qcamvc\n");
}

static void __exit proc_qcamvc_destroy(void)
{
	remove_proc_entry("qcamvc", 0);
}
#endif /* CONFIG_PROC_FS && CONFIG_VIDEO_PROC_FS */

/*****************************************************************************/
/*                                                                           */
/* Video 4 Linux                                                             */
/*                                                                           */
/*****************************************************************************/

static int qcamvc_vopen(struct inode *inode, struct file *file)
{
	struct video_device *dev = video_devdata(file);
	struct qcamvc_data *qcamvc = (struct qcamvc_data *)dev->priv;

#ifdef DBG_CAM
	printk("QuickCam qcamvc_vopen:\n");
#endif

	if (!qcamvc) {
		printk("Internal error, qcamvc_data not found!\n");
		return -ENODEV;
	}

	if (qcamvc->open_count > 0) {
		printk("Camera already open\n");
		return -EBUSY;
	}

	if (!try_module_get(qcamvc->ops->owner))
	{
		printk("qcamvc_vopen: try_module_get() FAILED\n");
		return -ENODEV;
	}

	/* Init the frame buffers to nothing */
	qcamvc->curframe = 0;
	qcamvc->frame_count = 0;
	qcamvc->frame_waiting = 0;
	free_frame_buf(qcamvc);

	if (qcamvc->ops->qcamvc_open(qcamvc->lowlevel_data))
		return -ENODEV;

	/* if the camera hasn't already been initialised, do it now */
	if (!qcamvc->camera_init)
	{
		if (qcamvc_camera_init(qcamvc) == -1)
			return -ENODEV;
	}

	/* Set ownership of /proc/qcamvc/videoX to current user */
	if(qcamvc->proc_entry)
		qcamvc->proc_entry->uid = current->uid;

	file->private_data = dev;
	qcamvc->open_count++;

	return 0;
}

static int qcamvc_vrelease(struct inode *inode, struct file *file)
{
	struct video_device *dev = video_devdata(file);
	struct qcamvc_data *qcamvc = (struct qcamvc_data *)dev->priv;

	if (qcamvc->ops)
	{
		/* Return ownership of /proc/qcamvc/videoX to root */
		if(qcamvc->proc_entry)
			qcamvc->proc_entry->uid = 0;
	
		qcamvc->ops->qcamvc_close(qcamvc->lowlevel_data);
		put_cam(qcamvc->ops);
	}

	if (--qcamvc->open_count == 0)
	{
		free_frame_buf(qcamvc);
		free_raw_frame(qcamvc);

		if (!qcamvc->ops)
			kfree(qcamvc);
	}
	
	file->private_data = NULL;

	return 0;
}

static ssize_t qcamvc_vread(struct file *file, char *data, size_t count, loff_t *ppos)
{ 
	if (!file)
		return -EINVAL;

	struct video_device *dev = file->private_data;
	struct qcamvc_data *qcamvc = dev->priv;
	char *tmpbuf;

	if (!qcamvc || !data || !count)
		return -EINVAL;
		
	if ( ((qcamvc->colordepth / 8) * qcamvc->width * qcamvc->height) > count )
	{
		printk("QuickCam VC: Image buffer (%ld) too small for bit count (%d bites, %d bytes required). Forcing to 24bit.\n",
				count, qcamvc->colordepth, (qcamvc->colordepth / 8) * qcamvc->width * qcamvc->height);
		/* force it to 24. If it still fails next time, there's not much we can do about it. */
		qcamvc->colordepth = 24;
		qcamvc->pixelformat = V4L2_PIX_FMT_RGB24;
		return -EFAULT;
	}

	if (down_interruptible(&qcamvc->busy_lock))
		return -EINTR;

	/* grab a raw frame from the camera */
	if (send_grab_image(qcamvc, 1))
	{
		up(&qcamvc->busy_lock);
		return 0;
	}
	
	count = get_raw_frame(qcamvc);
	
	if (count < 0)
	{ /* error or no data from camera? */
		up(&qcamvc->busy_lock);
		return 0;
	}

	/* convert the raw CMYG image to something usable */
	count = qcamvc->colordepth / 8 * qcamvc->width * qcamvc->height;
	tmpbuf = vmalloc(count);
	if (!tmpbuf)
	{
		printk("QCamVC: Could not allocate %ld bytes\n", count);
		count = -ENOMEM;
	}
	else
	{
		qcamvc_parse(qcamvc, tmpbuf);
		if (copy_to_user(data, tmpbuf, count))
			count = -EFAULT;
		vfree(tmpbuf);
	}

	up(&qcamvc->busy_lock);

	return count;
}

static int qcamvc_vmmap(struct file *file, struct vm_area_struct *vma)
{
	struct video_device *dev = file->private_data;
	unsigned long start = vma->vm_start;
	unsigned long size  = vma->vm_end - vma->vm_start;
	unsigned long page, pos;
	struct qcamvc_data *cam = dev->priv;
	int retval;

	if (!cam || !cam->ops)
		return -ENODEV;
	
	if (size > QCAMVC_NUMFRAMES*MAX_FRAME_SIZE)
		return -EINVAL;

	if (!cam || !cam->ops)
		return -ENODEV;
	
	if (down_interruptible(&cam->busy_lock))
		return -EINTR;

	if ( (retval = allocate_frame_buf(cam)) )
	{
		up(&cam->busy_lock);
		return retval;
	}

	pos = (unsigned long)(cam->frame_buf);
	while (size > 0) {
		page = kvirt_to_pa(pos);
		if (remap_page_range(vma, start, page, PAGE_SIZE, PAGE_SHARED)) {
			up(&cam->busy_lock);
			return -EAGAIN;
		}
		start += PAGE_SIZE;
		pos += PAGE_SIZE;
		if (size > PAGE_SIZE)
			size -= PAGE_SIZE;
		else
			size = 0;
	}

	DBG("qcamvc_mmap: %ld\n", size);
	up(&cam->busy_lock);

	return 0;
}

static int qcamvc_vioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
	return video_usercopy(inode, file, cmd, arg, qcamvc_do_ioctl);
}

static int qcamvc_querycap(struct qcamvc_data *qcamvc, struct v4l2_capability *cap)
{
	memset(cap, 0, sizeof(*cap));
	strcpy(cap->driver,"qcamvc");
	strlcpy(cap->card,qcamvc->camtypedesc,sizeof(cap->card));
	cap->version = QCAMVC_VERSION_CODE;
	cap->capabilities = 
		V4L2_CAP_VIDEO_CAPTURE |
		V4L2_CAP_READWRITE;
	return 0;
}

static int qcamvc_enum_fmt(struct v4l2_fmtdesc *f)
{
	if (f->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;

	switch(f->index)
	{
	case 0:
		f->pixelformat = V4L2_PIX_FMT_RGB24;
		strcpy(f->description, "RGB 8-8-8");
		break;

	case 1:
		f->pixelformat = V4L2_PIX_FMT_BGR24;
		strcpy(f->description, "BGRA 8-8-8");
		break;

	case 2:
		f->pixelformat = V4L2_PIX_FMT_RGB32;
		strcpy(f->description, "RGBA 8-8-8-8");
		break;

	case 3:
		f->pixelformat = V4L2_PIX_FMT_BGR32;
		strcpy(f->description, "BGR 8-8-8-8");
		break;

	case 4:
		f->pixelformat = V4L2_PIX_FMT_RGB555;
		strcpy(f->description, "RGB 5-5-5");
		break;

	case 5:
		f->pixelformat = V4L2_PIX_FMT_RGB565;
		strcpy(f->description, "RGB 5-6-5");
		break;

	case 6:
		f->pixelformat = V4L2_PIX_FMT_GREY;
		strcpy(f->description, "Greyscale");
		break;

	case 7:
		f->pixelformat = V4L2_PIX_FMT_YUYV;
		strcpy(f->description, "YUV 4:2:2 (YUYV)");
		break;

	case 8:
		f->pixelformat = V4L2_PIX_FMT_UYVY;
		strcpy(f->description, "YUV 4:2:2 (UYVY)");
		break;

	default:
		return -EINVAL;
	}
	f->flags = 0;
	return 0;
}

static int qcamvc_try_fmt(struct v4l2_format *f)
{
	int depth;
	int width;
	int height;

	if (f->type != V4L2_BUF_TYPE_VIDEO_CAPTURE ||
	    (f->fmt.pix.field != V4L2_FIELD_INTERLACED &&
	     f->fmt.pix.field != V4L2_FIELD_ANY))
		return -EINVAL;

	depth = mode_depth(f->fmt.pix.pixelformat);

	if (!depth)
		return -EINVAL;

	width = f->fmt.pix.width;
	height = f->fmt.pix.height;

	make_valid_res(&width, &height);
	f->fmt.pix.width = width;
	f->fmt.pix.height = height;
	f->fmt.pix.field = V4L2_FIELD_INTERLACED;
	f->fmt.pix.bytesperline = f->fmt.pix.width * depth / 8;
	f->fmt.pix.sizeimage = f->fmt.pix.bytesperline * f->fmt.pix.height;
	f->fmt.pix.colorspace = mode_colorspace(f->fmt.pix.pixelformat);
	return 0;
}

static int qcamvc_s_fmt(struct qcamvc_data *qcamvc, struct v4l2_format *f)
{
	int depth;
	int newbpc;

	if (f->type != V4L2_BUF_TYPE_VIDEO_CAPTURE ||
	    (f->fmt.pix.field != V4L2_FIELD_INTERLACED &&
	     f->fmt.pix.field != V4L2_FIELD_ANY))
		return -EINVAL;

	printk("Setting format to %dx%d %c%c%c%c\n",
			f->fmt.pix.width, f->fmt.pix.height,
			f->fmt.pix.pixelformat,
			f->fmt.pix.pixelformat >> 8,
			f->fmt.pix.pixelformat >> 16,
			f->fmt.pix.pixelformat >> 24);

	depth = mode_depth(f->fmt.pix.pixelformat);

	if (!depth)
		return -EINVAL;

	newbpc = mode_bpc(f->fmt.pix.pixelformat);
	if (newbpc != qcamvc->bpc)
	{
		if (down_interruptible(&qcamvc->busy_lock))
			return -EBUSY;
	}

	qcamvc_set_res(qcamvc, f->fmt.pix.width, f->fmt.pix.height);
	qcamvc->colordepth = depth;
	qcamvc->pixelformat = f->fmt.pix.pixelformat;
	f->fmt.pix.width = qcamvc->width;
	f->fmt.pix.height = qcamvc->height;
	f->fmt.pix.field = V4L2_FIELD_INTERLACED;
	f->fmt.pix.bytesperline = f->fmt.pix.width * depth / 8;
	f->fmt.pix.sizeimage = f->fmt.pix.bytesperline * f->fmt.pix.height;
	f->fmt.pix.colorspace = mode_colorspace(f->fmt.pix.pixelformat);
	if (newbpc != qcamvc->bpc)
	{
		qcamvc->bpc = newbpc;
		qcamvc_camera_init(qcamvc);
		up(&qcamvc->busy_lock);
	}
	return 0;
}

static int qcamvc_g_fmt(struct qcamvc_data *qcamvc, struct v4l2_format *f)
{
	if (f->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;

	f->fmt.pix.width = qcamvc->width;
	f->fmt.pix.height = qcamvc->height;
	f->fmt.pix.pixelformat = qcamvc->pixelformat;
	f->fmt.pix.field = V4L2_FIELD_INTERLACED;
	f->fmt.pix.bytesperline = f->fmt.pix.width * qcamvc->colordepth / 8;
	f->fmt.pix.sizeimage = f->fmt.pix.bytesperline * f->fmt.pix.height;
	f->fmt.pix.colorspace = mode_colorspace(f->fmt.pix.pixelformat);
	f->fmt.pix.priv = 0;
	return 0;
}

static int qcamvc_cropcap(struct qcamvc_data *qcamvc, struct v4l2_cropcap *cap)
{
	if (cap->type != V4L2_BUF_TYPE_VIDEO_CAPTURE)
		return -EINVAL;
	memset(cap, 0, sizeof(*cap));
	cap->bounds.left = 0;
	cap->bounds.top = 0;
	cap->bounds.width = qcamvc->width;
	cap->bounds.height = qcamvc->height;
	cap->defrect.left = 0;
	cap->defrect.top = 0;
	cap->defrect.width = qcamvc->width;
	cap->defrect.height = qcamvc->height;
	cap->pixelaspect.numerator = 1;
	cap->pixelaspect.denominator = 1;
	return 0;
}

#define IGNORE_IOCTL(x) case x: printk("QCamVC: Ignoring " #x "\n"); return -EINVAL;
#define NOACTION_IOCTL(x) case x: printk("QCamVC: No action taken for " #x "\n"); return 0;

static int qcamvc_enuminput(struct v4l2_input *pin)
{
	if (pin->index != 0)
		return -EINVAL;

	strcpy(pin->name, "Camera");
	pin->type = V4L2_INPUT_TYPE_CAMERA;
	pin->audioset = 0;
	pin->tuner = 0;
	pin->std = 0;
	pin->status = 0;
	memset(pin->reserved, 0, sizeof(pin->reserved));
	return 0;
}

static int qcamvc_queryctrl(struct v4l2_queryctrl *qc)
{
	switch (qc->id)
	{
	case V4L2_CID_BRIGHTNESS:
		strcpy(qc->name, "Brightness");
		qc->type = V4L2_CTRL_TYPE_INTEGER;
		qc->maximum = 256 << 8;
		qc->default_value = 175 << 8;
		qc->step = 256;
		break;

	case V4L2_CID_WHITENESS:
		strcpy(qc->name, "Exposure");
		qc->type = V4L2_CTRL_TYPE_INTEGER;
		qc->maximum = 255 << 8;
		qc->default_value = 57 << 8;
		qc->step = 256;
		break;

	case V4L2_CID_CONTRAST:
		strcpy(qc->name, "Contrast");
		qc->type = V4L2_CTRL_TYPE_INTEGER;
		qc->maximum = 255 << 8;
		qc->default_value = 128 << 8;
		qc->step = 256;
		break;

	case V4L2_CID_SATURATION:
		strcpy(qc->name, "Saturation");
		qc->type = V4L2_CTRL_TYPE_INTEGER;
		qc->maximum = 255 << 8;
		qc->default_value = 128 << 8;
		qc->step = 256;
		break;

	case V4L2_CID_HUE:
		strcpy(qc->name, "Hue");
		qc->type = V4L2_CTRL_TYPE_INTEGER;
		qc->maximum = 255 << 8;
		qc->default_value = 128 << 8;
		qc->step = 256;
		break;

	case V4L2_CID_DO_WHITE_BALANCE:
		strcpy(qc->name, "Auto Brightness");
		qc->type = V4L2_CTRL_TYPE_BOOLEAN;
		qc->maximum = 1;
		qc->default_value = 0;
		qc->step = 1;
		break;

	default:
		return -EINVAL;
	}
	qc->minimum = 0;
	qc->flags = 0;
	memset(qc->reserved, 0, sizeof(qc->reserved));
	return 0;
}

static int qcamvc_s_ctrl(struct qcamvc_data *qcamvc, struct v4l2_control *c)
{
	__u16 value = c->value; /* Works around a bug in V4L1-V4L2 conversion */

	switch (c->id)
	{
	case V4L2_CID_BRIGHTNESS:
		if (value > 255 << 8)
			return -ERANGE;
		if (down_interruptible(&qcamvc->busy_lock))
			return -EBUSY;
		qcamvc->brightness = value >> 8;
		qcamvc_set_brightness(qcamvc, qcamvc->brightness);
		up(&qcamvc->busy_lock);
		return 0;

	case V4L2_CID_WHITENESS:
		if (value > 255 << 8)
			return -ERANGE;
		if (down_interruptible(&qcamvc->busy_lock))
			return -EBUSY;
		qcamvc->exposure = value >> 8;
		qcamvc_set_exposure(qcamvc, qcamvc->exposure);
		up(&qcamvc->busy_lock);
		return 0;

	case V4L2_CID_CONTRAST:
		if (value > 255 << 8)
			return -ERANGE;
		qcamvc->contrast = value >> 8;
		qcamvc->gamma_done = 0;
		return 0;

	case V4L2_CID_SATURATION:
		if (value > 255 << 8)
			return -ERANGE;
		qcamvc->saturation = value >> 8;
		return 0;

	case V4L2_CID_HUE:
		if (value > 255 << 8)
			return -ERANGE;
		qcamvc->hue = value >> 8;
		return 0;

	case V4L2_CID_DO_WHITE_BALANCE:
		return 0;
	}
	return -EINVAL;
}

static int qcamvc_g_ctrl(struct qcamvc_data *qcamvc, struct v4l2_control *c)
{
	switch (c->id)
	{
	case V4L2_CID_BRIGHTNESS:
		c->value = qcamvc->brightness << 8;
		break;

	case V4L2_CID_WHITENESS:
		c->value = qcamvc->exposure << 8;
		break;

	case V4L2_CID_CONTRAST:
		c->value = qcamvc->contrast << 8;
		break;

	case V4L2_CID_SATURATION:
		c->value = qcamvc->saturation << 8;
		break;

	case V4L2_CID_HUE:
		c->value = qcamvc->hue << 8;
		break;

	case V4L2_CID_DO_WHITE_BALANCE:
		c->value = 0;
		break;

	default:
		return -EINVAL;
	}
	return 0;
}

static int qcamvc_do_ioctl(struct inode *inode, struct file *file, unsigned int cmd, void *arg)
{
	struct video_device *dev = file->private_data;
	struct qcamvc_data *qcamvc = dev->priv;

	int retval = 0;

	if (!qcamvc || !qcamvc->ops)
		return -ENODEV;

	if ((cmd & 0xff00) == 0x7600)
	{
		/* V4L version 1 - call the compatibility routine */
		retval = v4l_compat_translate_ioctl(inode, file, cmd, arg, qcamvc_do_ioctl);

		if (retval >= 0)
		{
			// Fix things that v4l_compat can't handle
			
			switch (cmd)
			{
			case VIDIOCGPICT:
				((struct video_picture *)arg)->depth = mode_depth(qcamvc->pixelformat);
				break;

			case VIDIOCGCAP:
				{
					struct video_capability *cap = (struct video_capability *) arg;

					cap->minwidth = MIN_WIDTH;
					cap->minheight = MIN_HEIGHT;
				}
				break;
			}
		}
		return retval;
	}

	switch(cmd)
	{
#if 0
	/* mmap interface for V4L1 - kept here in case we need to implement it
	 * for V4L2, although we shouldn't do so unless an app requires it.
	 * This implementation seems somewhat hackish since it still causes the
	 * caller to sleep, which the mmap interface should not do.
	 */
	case VIDIOCGMBUF:
	{
		struct video_mbuf *vm = arg;
		int i;

		DBG("VIDIOCGMBUF\n");
		memset(vm, 0, sizeof(*vm));
		vm->size = QCAMVC_NUMFRAMES*MAX_FRAME_SIZE;
		vm->frames = QCAMVC_NUMFRAMES;
		for (i = 0; i < QCAMVC_NUMFRAMES; i++)
			vm->offsets[i] = MAX_FRAME_SIZE * i;
		break;
	}
      
	case VIDIOCMCAPTURE:
	{
		struct video_mmap *vm = arg;

//printk("VIDIOCMCAPTURE: %d / %d / %dx%d\n", vm->format, vm->frame, vm->width, vm->height);
		if (vm->frame<0||vm->frame>=QCAMVC_NUMFRAMES)
		{
			retval = -EINVAL;
			break;
		}

		switch(vm->format)
		{
		case V4L2_PIX_FMT_GREY:
			qcamvc->vpic.depth=8;
			break;
		/*
		case V4L2_PIX_FMT_RGB555:
		case V4L2_PIX_FMT_RGB565:
		case V4L2_PIX_FMT_YUYV:
		case V4L2_PIX_FMT_UYVY:
			qcamvc->vpic.depth = 16;
			break;*/
		case V4L2_PIX_FMT_RGB24:
			qcamvc->vpic.depth = 24;
			break;
		case V4L2_PIX_FMT_RGB32:
			qcamvc->vpic.depth = 32;
			break;
		default:
			retval = -EINVAL;
			break;
		}
		if (retval)
			break;
			
		qcamvc->vpic.palette = vm->format;

		if (res_changed(qcamvc, vm->width, vm->height))
		{
printk("size changing\n");
			//memset(&qcamvc->vc, 0, sizeof(qcamvc->vc));
			
			qcamvc_set_res(qcamvc, vm->width, vm->height);
		}
		
		retval = capture_frame(qcamvc, vm->frame);

		break;
	}

	case VIDIOCSYNC:
	{
		int *frame = arg;

		if (*frame<0 || *frame >= QCAMVC_NUMFRAMES )
		{
			retval = -EINVAL;
			break;
		}
	
		switch (qcamvc->frame[*frame].state) 
		{
			case FRAME_GRABBING:
			{
				/* parse the frame now */
				if (capture_frame(qcamvc, *frame))
					retval = -EINVAL;
				
				qcamvc->frame[*frame].state = FRAME_READY;
				break;
			}
			
			case FRAME_READY:
			case FRAME_UNUSED:
			{
				printk("QuickCam VC: Sync to unused frame\n");
				retval = -EINVAL;
				break;
			}
			
			case FRAME_DONE:
			/* if the frame has already been captured & parsed, just mark it as ready */
	    	qcamvc->frame[*frame].state = FRAME_READY;
			break;
			
			default:
			retval = -EINVAL;
			break;
		}
		break;
	}
#endif
	
	/* ***  v4l2  *** ************************************************ */
	case VIDIOC_QUERYCAP:
		return qcamvc_querycap(qcamvc, (struct v4l2_capability *) arg);

	case VIDIOC_S_FMT:
		return qcamvc_s_fmt(qcamvc, (struct v4l2_format *) arg);
	
	case VIDIOC_G_FMT:
		return qcamvc_g_fmt(qcamvc, (struct v4l2_format *) arg);

	case VIDIOC_ENUM_FMT:
		return qcamvc_enum_fmt((struct v4l2_fmtdesc *) arg);
	
	case VIDIOC_TRY_FMT:
		return qcamvc_try_fmt((struct v4l2_format *) arg);

	case VIDIOC_CROPCAP:
		return qcamvc_cropcap(qcamvc, (struct v4l2_cropcap *) arg);

	case VIDIOC_ENUMINPUT:
		return qcamvc_enuminput((struct v4l2_input *) arg);
	
	case VIDIOC_G_INPUT:
		*(int *) arg = 0;
		return 0;
	
	case VIDIOC_S_INPUT:
		if (*(int *) arg != 0)
			return -EINVAL;
		return 0;

	case VIDIOC_QUERYCTRL:
		return qcamvc_queryctrl((struct v4l2_queryctrl *) arg);
	
	case VIDIOC_S_CTRL:
		return qcamvc_s_ctrl(qcamvc, (struct v4l2_control *) arg);
	
	case VIDIOC_G_CTRL:
		return qcamvc_g_ctrl(qcamvc, (struct v4l2_control *) arg);

	case VIDIOC_G_STD: /* Should return -EINVAL but SANE fails if we do */
	case VIDIOC_QUERYSTD: /* for broken apps - ideally we should return -EINVAL */
		*(v4l2_std_id *) arg = 0;
		return 0;

	/* This relates to both memory mapping and user pointers. Implementing user
	 * pointers may be a reasonable thing to do if an application needs it
	 */
	IGNORE_IOCTL(VIDIOC_REQBUFS)
	NOACTION_IOCTL(VIDIOC_STREAMOFF)
	NOACTION_IOCTL(VIDIOC_STREAMON)
	IGNORE_IOCTL(VIDIOC_QBUF)
	IGNORE_IOCTL(VIDIOC_DQBUF)

	/* This relates to memory mapping. It is probably not a good idea to implement
	 * the mmap interface unless an application is found that requires them - memory
	 * mapping is * required to allocate DMA-able memory for the buffers (that is,
	 * kalloc memory * rather than vmalloc memory), which is a scarce resource and
	 * highly prone to * failure with blocks the size of the ones we would be
	 * allocating. All this for little to no performance improvement in our case
	 * since we need to translate the pixels to something the app will understand
	 * anyway.
	 */
	IGNORE_IOCTL(VIDIOC_QUERYBUF)

	/* These are miscellaneous features we shouldn't need to implement */
	IGNORE_IOCTL(VIDIOC_QUERYMENU)
	IGNORE_IOCTL(VIDIOC_G_CROP)
	IGNORE_IOCTL(VIDIOC_S_CROP)
	IGNORE_IOCTL(VIDIOC_G_JPEGCOMP)

	/*These should never apply to this device */
	IGNORE_IOCTL(VIDIOC_OVERLAY)
	IGNORE_IOCTL(VIDIOC_G_FBUF)
	IGNORE_IOCTL(VIDIOC_S_FBUF)
	IGNORE_IOCTL(VIDIOC_ENUMSTD)
	IGNORE_IOCTL(VIDIOC_S_STD)
	IGNORE_IOCTL(VIDIOC_G_PARM)
	IGNORE_IOCTL(VIDIOC_S_PARM)

	case VIDIOC_G_TUNER:
	case VIDIOC_S_TUNER:
	case VIDIOC_G_AUDIO:
	case VIDIOC_S_AUDIO:
	case VIDIOC_G_OUTPUT:
	case VIDIOC_S_OUTPUT:
	case VIDIOC_ENUMOUTPUT:
	case VIDIOC_G_AUDOUT:
	case VIDIOC_S_AUDOUT:
	case VIDIOC_G_MODULATOR:
	case VIDIOC_S_MODULATOR:
	case VIDIOC_G_FREQUENCY:
	case VIDIOC_S_FREQUENCY:
		/* These can never apply and so are ignored without
		 * any notification.
		 */
		return -EINVAL;

	default:
		printk("QuickCam VC: Unknown IOCTL cmd: %X\n", cmd);
		retval = -ENOIOCTLCMD;
		break;
    }
	
	return retval;
}

/*****************************************************************************
 QuickCam Init
*****************************************************************************/

static int qcamvc_camera_init(struct qcamvc_data *qcamvc)
{
	if (!qcamvc) return -1;

	long i;
	unsigned char *bigbuffer;
	unsigned char bitmask;

	memset(&qcamvc->misc_reg, 0, sizeof(qcamvc->misc_reg));
	
	bigbuffer=kmalloc(BIGBUFFER_SIZE, GFP_KERNEL);
	if (bigbuffer==NULL) 
	{
		printk("QuickCam VC: Init bigbuffer kmalloc error.\n");
		return -1;
	}
	
	qcamvc->camera_init = 1;
	
	/* setup a 128KB bit mask for 6bit/8bit per component */
	bitmask = 0xFF << qcamvc->bpc;
	for(i=0; i<BIGBUFFER_SIZE ; i+=2)
	{
		*(bigbuffer + i) = (unsigned char )(i >> 11);
		*(bigbuffer + i + 1) = bitmask;
	}
	
	get_camera_model(qcamvc);
	
	set_register(qcamvc, QCAM_VC_SET_BRIGHTNESS, 0x38);
	set_register(qcamvc, QCAM_VC_SET_BRIGHTNESS, 0x78);
	
	/* clear config bit & disable video streaming */
	set_register(qcamvc, QCAM_VC_SET_MISC, 0x00);
	
	/* clear 0x0e address (whatever this address actually does?) */
	set_register(qcamvc, 0x0E, 0x00);
	
	/* write 128KB of 6bit/8bit mask */
	set_registers(qcamvc, QCAM_VC_GET_FRAME, bigbuffer, BIGBUFFER_SIZE);
	
	/* set address 0x0e to 0x01 */
	set_register(qcamvc, 0x0E, 0x01);
	
	/* this might have somthing to do with setting the amount of compression - don't know how though. */
	set_register(qcamvc, QCAM_VC_SET_BRIGHTNESS, 0x78);
	set_register(qcamvc, QCAM_VC_SET_BRIGHTNESS, 0x78);
	set_register(qcamvc, QCAM_VC_SET_BRIGHTNESS, 0x58);

	/* set brightness & exposure - probably not needed here */
	qcamvc_set_brightness(qcamvc, qcamvc->brightness);
	qcamvc_set_exposure(qcamvc, qcamvc->exposure);

	/* set config bit. (whatever this bit actually does?) */
	set_register(qcamvc, QCAM_VC_SET_MISC, 0x01);
	
	/* set CCD columns & rows */
	qcamvc_set_ccd_area(qcamvc);

	/* set brightness & exposure */
	qcamvc_set_brightness(qcamvc, qcamvc->brightness);
	qcamvc_set_exposure(qcamvc, qcamvc->exposure);
	
	/* set the Light Sensitivity (no gain/no attenuation = 128 ) */
	qcamvc_set_light_sensitivity(qcamvc, qcamvc->light_sens);

	/* set Misc options & re-enable video streaming now */
	qcamvc->misc_reg.config = 1;
	qcamvc->misc_reg.multiplier = (qcamvc->ccd_area.multiplier ? 0 : 1);
	qcamvc->misc_reg.compression = (qcamvc->compression ? 1 : 0);
	qcamvc->misc_reg.video_enable = 1;
	set_registers(qcamvc, QCAM_VC_SET_MISC, (unsigned char*)&qcamvc->misc_reg, 1);
	
	/* send a get frame to clear the previous resolution */
	set_register(qcamvc, QCAM_VC_GET_FRAME, 0xFF);

	/* done with init */
	kfree(bigbuffer);
	
	return 0;
}

/*****************************************************************************/
/*                                                                           */
/* QuickCam Video Camera layer                                               */
/*                                                                           */
/*****************************************************************************/
static int qcamvc_initialise(struct qcamvc_data *qcamvc, char *camtype)
{
	strcpy(qcamvc->camtypedesc, camtype);
	
	/* defaults */
	qcamvc->camera_model	= 0;
	qcamvc->camera_type		= 0;

	qcamvc->width			= 352;
	qcamvc->height			= 288;
	qcamvc->bpc				= 8;
		
	qcamvc->compression		= 0;	//FIXME: on/off - don't know how to set amount of compression.
	qcamvc->light_sens		= 128;
	qcamvc->brightness		= 175;
	qcamvc->exposure		= 57;
	qcamvc->gamma_done		= 0;
	qcamvc->hue				= 128;
	qcamvc->saturation		= 128;
	qcamvc->contrast		= 128;

	qcamvc->pixelformat		= V4L2_PIX_FMT_RGB24;
	qcamvc->colordepth		= 24;
	qcamvc->flip_bgr		= 0;
	
	qcamvc->transfer_rate	= 0;
	qcamvc->curframe		= 0;
	qcamvc->frame_count		= 0;
	qcamvc->frame_waiting	= 0;
	
	qcamvc->raw_frame		= NULL;
	qcamvc->frame_buf		= NULL;
	free_frame_buf(qcamvc);
	
	qcamvc->proc_entry		= NULL;
	qcamvc->open_count		= 0;
	qcamvc->camera_init		= 0;
	
	qcamvc->vdev = video_device_alloc();
	if (qcamvc->vdev == NULL)
	{
		printk(KERN_ERR "QuickCam VC: video_device_alloc() returned NULL!");
		return -1;
	}
	*qcamvc->vdev = qcamvc_template;
	strcpy(qcamvc->vdev->name, qcamvc->camtypedesc);
	qcamvc->vdev->minor = -1;
	qcamvc->vdev->dev = NULL;
	qcamvc->vdev->release = video_device_release;
	qcamvc->vdev->priv = qcamvc;
	
	return 0;
}

struct qcamvc_data *qcamvc_register_camera(struct qcamvc_camera_ops *ops, void *lowlevel, char *camtype)
{
	struct qcamvc_data *qcamvc;

	if ((qcamvc = kmalloc(sizeof(struct qcamvc_data), GFP_KERNEL)) == NULL)
	{
		printk(KERN_ERR "QuickCam VC: Error allocating memory in qcamvc_register_camera\n");
		return NULL;
	}

	memset(qcamvc, 0, sizeof(struct qcamvc_data));
	qcamvc->ops = ops;
	qcamvc->lowlevel_data = lowlevel;
	
	init_MUTEX(&qcamvc->busy_lock);
	
	if (qcamvc_initialise(qcamvc, camtype) < 0)
	{
		kfree(qcamvc);
		return NULL;
	}

	if (video_register_device(qcamvc->vdev, VFL_TYPE_GRABBER, video_nr) == -1)
	{
		kfree(qcamvc);
		printk(KERN_ERR "QuickCam VC: video_register_device failed.\n"); 
		return NULL;
	}

#ifdef CONFIG_PROC_FS
	create_proc_qcamvc_cam(qcamvc);
#endif

	if (!qcamvc->ops->qcamvc_open(qcamvc->lowlevel_data))
	{
		qcamvc_camera_init(qcamvc);
		qcamvc->ops->qcamvc_close(qcamvc->lowlevel_data);
	}

	return qcamvc;
}

void qcamvc_unregister_camera(struct qcamvc_data *qcamvc)
{
#ifdef CONFIG_PROC_FS
	printk(KERN_INFO "QuickCam VC: destroying /proc/qcamvc/video%d\n", qcamvc->vdev->minor);
	destroy_proc_qcamvc_cam(qcamvc);
#endif	

	printk(KERN_INFO "QuickCam VC: unregistering video...\n");
	
	if (qcamvc->vdev)
	{
		if (-1 != qcamvc->vdev->minor)
			video_unregister_device(qcamvc->vdev);
		else
			video_device_release(qcamvc->vdev);
		qcamvc->vdev = NULL;
	}

	if (qcamvc->open_count)
	{
		put_cam(qcamvc->ops);
		printk(KERN_INFO "QuickCam VC: camera open -- setting ops to NULL\n");
		qcamvc->ops = NULL;
	}

	free_raw_frame(qcamvc);

	if (!qcamvc->open_count)
	{
		printk(KERN_INFO "QuickCam VC: freeing camera\n");
		kfree(qcamvc);
	}

}

/*****************************************************************************/
/*                                                                           */
/* QuickCam Video Camera layer                                               */
/* Module subs                                                               */
/*                                                                           */
/*****************************************************************************/

static int __init qcamvc_driver_init(void)
{
	printk("QuickCam VC: (C) 2001 by De Marchi Daniele, <demarchidaniele@libero.it>\n");
	printk("QuickCam VC: v4l level driver version %s registered.\n",QCAMVCVERSION);
#ifdef CONFIG_PROC_FS
	proc_qcamvc_create();
#endif
	
#ifdef CONFIG_KMOD
#ifdef CONFIG_VIDEO_QCAMVC_PP_MODULE
	request_module("qcamvc_pp");
#endif

#ifdef CONFIG_VIDEO_QCAMVC_USB_MODULE
	request_module("qcamvc_usb");
#endif
#endif	/* CONFIG_KMOD */
	
#ifdef CONFIG_VIDEO_QCAMVC_PP
	qcamvc_pp_init();
#endif
#ifdef CONFIG_VIDEO_QCAMVC_USB
	qcamvc_usb_init();
#endif

	return 0;
}

static void __exit qcamvc_driver_exit(void)
{
#ifdef CONFIG_PROC_FS
	proc_qcamvc_destroy();
#endif
	printk(KERN_INFO "QuickCam VC: v4l driver unregistered\n");
}

module_init(qcamvc_driver_init);
module_exit(qcamvc_driver_exit);

/* Exported symbols for modules. */
EXPORT_SYMBOL(qcamvc_register_camera);
EXPORT_SYMBOL(qcamvc_unregister_camera);

