/*
 *
 *    Connectix USB QuickCam VC Video Camera driver
 *
 *    (C) Copyright 2001 De Marchi Daniele
 *    (C) Copyright 2004 Terry Mohan
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of the
 *    License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Jan 2001 This driver develop started on the linux 
 *            kernel 2.4.0.
 *
 */

#undef _QCAMVC_DEBUG_ 
//#define _QCAMVC_DEBUG_ 
 
#undef QCAMVC_PARPORT_DMA
#define QCAMVC_PARPORT_DMA 1   /* comment this out to remove ECP DMA support */

#define CONFIG_PARPORT_PC 

#include <linux/config.h>
#include <linux/version.h>

#include <linux/module.h>
#include <linux/fs.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/spinlock.h>
#include <linux/pci.h>
#include <asm/dma.h>
 
#include <linux/parport.h>
#include <linux/parport_pc.h>
#include <linux/delay.h>
#include <linux/sched.h>

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,3))
#ifdef CONFIG_KMOD
#include <linux/kmod.h>
#endif
#else
#include <linux/kmod.h>
#endif

#include "qcamvc.h"


struct pp_cam_entry
{
	struct parport *port;
	struct pardevice *pdev;
	
	int open_count;
};


/* Magic numbers for defining port-device mappings */
#define QCAMVC_PARPORT_UNSPEC -4
#define QCAMVC_PARPORT_AUTO -3
#define QCAMVC_PARPORT_OFF -2
#define QCAMVC_PARPORT_NONE -1

#ifdef MODULE
static int parport_nr[PARPORT_MAX] = {[0 ... PARPORT_MAX - 1] = QCAMVC_PARPORT_UNSPEC};
static char *parport[PARPORT_MAX] = {NULL,};
MODULE_AUTHOR("De Marchi Daniele <demarchidaniele@libero.it>");
MODULE_DESCRIPTION("Connectix QuickCam VC(PP) Driver");
MODULE_LICENSE("GPL");
MODULE_PARM(parport, "1-" __MODULE_STRING(PARPORT_MAX) "s");
MODULE_PARM_DESC(parport, "'auto' or a list of parallel port numbers. Just like lp.");
#else
static int parport_nr[PARPORT_MAX] __initdata =
	{[0 ... PARPORT_MAX - 1] = QCAMVC_PARPORT_UNSPEC};
static int parport_ptr = 0;
#endif

/*****************************************************************************/
/*                                                                           */
/* Lowlevel driver for Connectix QuickCam VC(PP)                             */
/*                                                                           */
/*****************************************************************************/
static int		qcamvc_open(void *privdata);
static int		qcamvc_close(void *privdata);
static size_t	qcamvc_get_reg(void *privdata, unsigned char reg, void *buffer, size_t len);
static int		qcamvc_set_reg(void *privdata, unsigned char reg, void *buffer, size_t len);
static size_t	qcamvc_read(void *privdata, void *buffer, size_t len);
static int		qcamvc_write(void *privdata, void *buffer, size_t len);
static int		qcamvc_stream_start(void *privdata);
static int		qcamvc_stream_stop(void *privdata);
static size_t	qcamvc_stream_read(void *privdata, void *buffer, size_t len);

static struct qcamvc_camera_ops qcamvc_pp_ops = 
{
	qcamvc_open,
	qcamvc_close,
	qcamvc_write,
	qcamvc_read,
	qcamvc_set_reg,
	qcamvc_get_reg,
	qcamvc_stream_start,
	qcamvc_stream_stop,
	qcamvc_stream_read,
	THIS_MODULE,
};

static LIST_HEAD(cam_list);
static spinlock_t cam_list_lock_pp;

static unsigned short ppmode = IEEE1284_MODE_ECP;

#ifdef QCAMVC_PARPORT_DMA
/* forward declarations for IEEE1284 ECP DMA read support  */
static int change_mode(struct parport *, int );
static void frob_econtrol (struct parport *, unsigned char, unsigned char);


/* ----------------------------------------*/
/* Parallel Port DMA Code from CPiA driver */
/* ----------------------------------------*/

/* DMA support here thanks to: Blaise Gassend <blaise@gassend.com> */

/*ECP parport lowlevel IEEE1284 DMA read support: */

#define ECR_PS2 01
#define ECR_ECP 03

/* this an adaptation of code from parport_pc_ecp_read_block_pio() and
 * parport_pc_fifo_write_block_dma() */

static size_t qcamvc_ecp_read_block_dma (struct parport *port,
				       void *bufv, size_t length, int flags)
{
	int r = 0;
	unsigned long dmaflag;
	size_t left = length;
	const struct parport_pc_private *priv = port->physport->private_data;
	dma_addr_t dma_addr, dma_handle = 0;
	size_t maxlen = 0x10000; /* max 64k per DMA transfer */
	char *buf = bufv;
	unsigned long start = (unsigned long) buf;
	unsigned long end = (unsigned long) buf + length - 1;
	size_t count = length;
	
	if (length == 0)
	{
		DBG ("(length == 0)");
		return 0;
	}
	
	/* Special case: a timeout of zero means we cannot call schedule(). */
	if (!port->cad->timeout)
	{
		DBG ("cannot call schedule()");
		return parport_ieee1284_ecp_read_data (port, buf, length, flags);
	}
	
	port = port->physport;
	
	if (port->ieee1284.phase != IEEE1284_PH_REV_IDLE)
	{
		/* change to reverse-idle phase (must be in forward-idle) */
		
		/* Event 38: Set nAutoFd low (also make sure nStrobe is high) */ 
		parport_frob_control (port,
				      PARPORT_CONTROL_STROBE |PARPORT_CONTROL_AUTOFD,
				      PARPORT_CONTROL_AUTOFD);
		parport_pc_data_reverse (port); /* Must be in PS2 mode */
		udelay (5);
		
		/* Event 39: Set nInit low to initiate bus reversal */
		parport_frob_control (port,
				      PARPORT_CONTROL_INIT,
				      0);
		
		/* Event 40: Wait for  nAckReverse (PError) to go low */
		if ( (r = parport_wait_peripheral (port, 
						  PARPORT_STATUS_PAPEROUT, 
						  0)) )
		{
			printk (KERN_DEBUG "%s: PE timeout Event 40 (%d) "
				"in ecp_read_block_dma\n", port->name, r);
			return 0;
		}

		
		port->ieee1284.phase = IEEE1284_PH_REV_DATA;
		
		parport_frob_control (port,
				      PARPORT_CONTROL_STROBE |PARPORT_CONTROL_AUTOFD,
				      PARPORT_CONTROL_AUTOFD);

		if (change_mode (port, ECR_ECP))      /* ECP FIFO */
			printk (KERN_DEBUG "%s: Warning change_mode ECR_ECP failed\n", 
				port->name);
		
		/* the first byte must be collected manually */

		/* Event 43: Wait for nAck to go low */
		if (parport_wait_peripheral (port, 
					     PARPORT_STATUS_ACK, 
					     0))
		{
			/* timed out while reading -- no data */
			printk (KERN_DEBUG "DMA read timed out (initial byte)\n");
			goto out_no_data;
		}
#if 0   /* this is active in parport_pc_ecp_read_block_pio */
		/* read byte */
		*buf++ = inb (DATA (port));
	       	left--;
#endif
		
		/* Event 44: nAutoFd (HostAck) goes high to acknowledge */
		parport_pc_frob_control (port,
					 PARPORT_CONTROL_AUTOFD,
					 0);
		
#if 0  /* this is commented out  in parport_pc_ecp_read_block_pio */
		/* Event 45: Wait for nAck to go high */
		if (parport_wait_peripheral (port, 
					     PARPORT_STATUS_ACK, 
					     PARPORT_STATUS_ACK)) {
			/* timed out while waiting for peripheral to respond to ack */
			printk ("ECP DMA read timed out (waiting for nAck)\n");
			
			/* keep hold of the byte we've got already */
			goto out_no_data;
		}
#endif
	}
	
#if 0   /* active in parport_pc_ecp_read_block_pio */
	/* Event 46: nAutoFd (HostAck) goes low to accept more data */
	parport_pc_frob_control (port,
				 PARPORT_CONTROL_AUTOFD,
				 PARPORT_CONTROL_AUTOFD);

#endif
	
	if (!left) 
	{
		DBG ("(!left)\n");
		goto out_no_data;
	}
	
	/* now do the DMA transfer (adapted from parport_pc_fifo_write_block_dma) */
	
	if (end < MAX_DMA_ADDRESS)
	{
		/* If it would cross a 64k boundary, cap it at the end. */
		if ((start ^ end) & ~0xffffUL)
			maxlen = 0x10000 - (start & 0xffff);
		
		dma_addr = dma_handle = pci_map_single(priv->dev, (void *)buf, length,
						       PCI_DMA_FROMDEVICE);
	}
	else
	{
		/* above 16 MB we use a bounce buffer as ISA-DMA is not possible */
		maxlen   = PAGE_SIZE;          /* sizeof(priv->dma_buf) */
		dma_addr = priv->dma_handle;
	}
	
	DBG ("DMA buffer is ready\n");
	
	/* We don't want to be interrupted every character. */
	parport_pc_disable_irq (port);
	/* set nErrIntrEn and serviceIntr */
	frob_econtrol (port, (1<<4) | (1<<2), (1<<4) | (1<<2));
	
	/* Reverse mode. */
	parport_pc_data_reverse (port); /* Must be in PS2 mode */

	while (left)
	{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))
		long expire = jiffies + port->physport->cad->timeout;
#else
		unsigned long expire = jiffies + port->physport->cad->timeout;
#endif	
		//DBG ("start of while %i left\n", left);
		
		count = left;
		
		if (count > maxlen)
			count = maxlen;
		
		dmaflag = claim_dma_lock();
		disable_dma(port->dma);
		clear_dma_ff(port->dma);
		set_dma_mode(port->dma, DMA_MODE_READ);
		set_dma_addr(port->dma, dma_addr);
		set_dma_count(port->dma, count);
		
		/* Set DMA mode */
		frob_econtrol (port, 1<<3, 1<<3);
		
		/* Clear serviceIntr */
		frob_econtrol (port, 1<<2, 0);
		
		enable_dma(port->dma);
		release_dma_lock(dmaflag);
		
		/* Wait for interrupt. */
		//DBG ("about to wait\n");
false_alarm:
		if (parport_wait_event (port, HZ) < 0)
		{
			DBG ("(parport_wait_event (port, HZ) < 0)\n");
			break;
		}
		if (!time_before (jiffies, expire))
		{
			/* Timed out. */
			DBG("DMA read timed out\n");
			break;
		}
		/* Is serviceIntr set? */
		if (!(inb (ECONTROL (port)) & (1<<2)))
		{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))
			if (current->need_resched)
				schedule ();
#else
			cond_resched();
#endif			
			DBG("false_alarm\n");
			goto false_alarm;
		}
		
		dmaflag = claim_dma_lock();
		disable_dma(port->dma);
		clear_dma_ff(port->dma);
		count -= get_dma_residue(port->dma);
		release_dma_lock(dmaflag);

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))		
		if (current->need_resched)
			/* Can't yield the port. */
			schedule ();
#else
		cond_resched(); /* Can't yield the port. */
#endif
		/* update for possible DMA residue ! */
		DBG ("read %i in loop\n", count);
		
		if (dma_handle) 
			dma_addr += count;
		else 
			memcpy(buf, priv->dma_buf, count);
		buf  += count;
		left -= count;
	}
	
	frob_econtrol (port, 1<<2, 1<<2);
	/* Turn off DMA mode */
	frob_econtrol (port, 1<<3, 0);
	
	if (dma_handle)
		pci_unmap_single(priv->dev, dma_handle, length, PCI_DMA_TODEVICE);
	
out_no_data:	
	if (change_mode (port, ECR_PS2)) /* ECP FIFO */ 
		printk (KERN_DEBUG "%s: Warning change_mode ECR_PS2 failed\n", 
			port->name);
	
	/* Go to forward idle mode to shut the peripheral up (event 47). */
	parport_frob_control (port, PARPORT_CONTROL_INIT | PARPORT_CONTROL_AUTOFD, 
			      PARPORT_CONTROL_INIT | PARPORT_CONTROL_AUTOFD);
	
	/* event 49: PError goes high */
	if ( (r = parport_wait_peripheral (port,
					  PARPORT_STATUS_PAPEROUT,
					  PARPORT_STATUS_PAPEROUT)) )
	{
		printk (KERN_DEBUG
			"%s: PE timeout FWDIDLE (%d) in ecp_read_block_dma\n",
			port->name, r);
	}
	else
	{
		parport_data_forward (port);        
		port->ieee1284.phase = IEEE1284_PH_FWD_IDLE;
	}
	//DBG ("returning %i bytes read, endseen = %d\n", length - left, endseen);
	return length - left;
}
#endif /*QCAMVC_PARPORT_DMA */


static int qcamvc_open(void *privdata)
{
	struct pp_cam_entry *cam = (struct pp_cam_entry *)privdata;

	if (cam == NULL)
		return -EINVAL;

	if(cam->open_count == 0)
	{
		if (parport_claim_or_block(cam->pdev))
		{
			printk(KERN_ERR "failed to claim the port\n");
			return -EBUSY;
		}
		parport_negotiate(cam->port, IEEE1284_MODE_COMPAT);
		parport_data_forward(cam->port);
		parport_write_control(cam->port, PARPORT_CONTROL_SELECT);
		udelay(50);
		parport_write_control(cam->port, PARPORT_CONTROL_SELECT|PARPORT_CONTROL_INIT);
	}
	
	++cam->open_count;
	
	return 0;
}

static int qcamvc_close(void *privdata)
{
	struct pp_cam_entry *cam = privdata;
	if(!cam || !cam->open_count) return 0;
	
	if (--cam->open_count == 0)
	{
		parport_release(cam->pdev);
	}
	
	return 0;
}

static int qcamvc_stream_start(void *privdata)
{
	int ret = 0;
	struct pp_cam_entry *cam = privdata;
	if(!cam || !cam->open_count) return 0;
	
	struct parport *parport = cam->port;

	parport_pc_data_reverse (parport); /* Must be in PS2 mode */
	return ret;
}

static int qcamvc_stream_stop(void *privdata)
{
	int ret = 0;
	return ret;
}

static size_t qcamvc_stream_read(void *privdata, void *buffer, size_t len)
{
	struct pp_cam_entry *cam = privdata;
	if(!cam || !cam->open_count) return 0;
	
	struct parport *parport = cam->port;
	//unsigned char a = 0x0;

	/*if(parport_negotiate(parport, IEEE1284_DATA | ppmode) != 0)
	{
		printk(KERN_ERR "QuickCamVC: qcamvc_stream_read1: ieee1284 failed negotiation\n");
		return -1;
	}

	parport_write(parport, (void *) &a, 1);*/
	
#ifdef QCAMVC_PARPORT_DMA
	if (parport->dma != PARPORT_DMA_NONE)
	{
		int read_bytes = 0,  chunks = 0;
		size_t chunksize = len;
		u8 *buf = buffer, *buf_end = buffer + len;
		size_t datasize, bufsize;
		while (read_bytes < len)
		{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))
			if (current->need_resched)
				schedule ();
#else
			cond_resched();
#endif
			bufsize = chunksize;
			if(bufsize > (size_t)(buf_end - buf)) 
				bufsize = (size_t)(buf_end - buf);
	
			datasize = qcamvc_ecp_read_block_dma(parport, buf, bufsize, 0);
			if (datasize <= 0)
			{
				//printk("datasize <= 0\n");
				break;
			}
			chunks++;
	
			buf += datasize;
			read_bytes += (int) datasize;
		}
		
		return read_bytes;
	}
#endif
	
	unsigned long counter;
	int read, read_bytes = 0;
	for (counter=0;counter<len;counter++)
	{
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))
		if (current->need_resched)
			schedule ();
#else
		cond_resched();
#endif
		if ( (read=parport_read(parport, buffer + counter, 1)) != 1 )
		{
			printk(KERN_ERR "QuickCamVC: qcamvc_stream_read2: failed to read: error %i.\n", read);
			break;
		}
		read_bytes += read;
	}
	
	return read_bytes;
}

/********************************************************************/
/* qcamvc_read general purpose parport read interface		*/
/****************************************************************/
static size_t qcamvc_read(void *privdata, void *buffer, size_t len)
{
	struct pp_cam_entry *cam = privdata;
	if(!cam || !cam->open_count) return 0;
	
	struct parport *parport = cam->port;
	size_t ret=0;
	
	if(parport_negotiate(parport, IEEE1284_DATA | ppmode) != 0)
	{
		printk(KERN_ERR "QuickCamVC: qcamvc_read: ieee1284 failed negotiation\n");
		return -1;
	}

	parport_read(parport, buffer, len);
	
	return ret = len;
}

/******************************************************************/
/* qcamvc_write general purpose parport write interface		*/
/****************************************************************/
static int qcamvc_write(void *privdata, void *buffer, size_t len)
{
	struct pp_cam_entry *cam = privdata;
	if(!cam || !cam->open_count) return 0;
	
	struct parport *parport = cam->port;  
	int ret=0;
	
	if(parport_negotiate(parport, IEEE1284_DATA | ppmode) != 0)
	{
		printk(KERN_ERR "QuickCamVC: qcamvc_write: ieee1284 failed negotiation\n");
		return -1;
	}

	parport_write(parport, buffer, len);
	
	return ret = len;
}

/********************************************************************************/
/* qcamvc_set_reg register value set method for the parport version of the QuickCamVC */
/********************************************************************************/
static int qcamvc_set_reg(void *privdata, unsigned char reg, void *buffer, size_t len)
{
	struct pp_cam_entry *cam = privdata;
	if(!cam || !cam->open_count) return 0;
	
	struct parport *parport = cam->port;
	int ret=0;
	
	if ( parport_negotiate(parport, IEEE1284_ADDR | ppmode) != 0 )
	{
		printk(KERN_ERR "QuickCamVC:WCN1 qcamvc_set_reg1: ieee1284 failed negotiation\n");
		return -1;
	}

	if (parport_write(parport, &reg, 1) != 1)
		printk(KERN_INFO "QuickCamVC: qcamvc_set_reg: write failed #1\n");
	
	if(parport_negotiate(parport, IEEE1284_DATA | ppmode) != 0)
	{
		printk(KERN_ERR "QuickCamVC: qcamvc_set_reg2: ieee1284 failed negotiation\n");
		return -1;
	}

	if (parport_write(parport, buffer, len) != len)
		printk(KERN_ERR "QuickCamVC: qcamvc_set_reg: write failed #2\n");
	
	return ret = len;
}

/********************************************************************************/
/* qcamvc_get_reg register value retrieval method for the parport version of the
QuickCamVC */
/*********************************************************************************/
static size_t qcamvc_get_reg(void *privdata, unsigned char reg, void *buffer, size_t len)
{
	struct pp_cam_entry *cam = privdata;
	if(!cam || !cam->open_count) return 0;
	
	struct parport *parport = cam->port;
	int ret=0;
	
	if(parport_negotiate(cam->port, IEEE1284_ADDR | ppmode) != 0)
	{
		printk(KERN_ERR "QuickCamVC:WCN1 qcamvc_get_reg1: ieee1284 failed negotiation\n");
		return -1;
	}
	
	ret=parport_write(parport, &reg, 1);
	
	if (len == 0 || buffer == NULL)
		return ret;
	
	if(parport_negotiate(parport, IEEE1284_DATA | ppmode) != 0)
	{
		printk(KERN_ERR "QuickCamVC: qcamvc_get_reg2: ieee1284 failed negotiation\n");
		return -1;
	}

	parport_read(parport, buffer, len);
	
	return len;
}


/************************************************************************/
/* qcamvc_register registers the camera with the V4L driver and the parport driver
*/
/**************************************************************************/
static int qcamvc_register(struct parport *port) 
{
	struct pardevice *pdev = NULL;
	struct pp_cam_entry *cam;
	struct qcamvc_data *qcamvc;
	
	if (!(port->modes & PARPORT_MODE_ECP) &&
		!(port->modes & PARPORT_MODE_TRISTATE))
	{
		printk(KERN_ERR "QuickCam VC(PP): Port is not ECP capable!\n");
		return -ENXIO;
	}

	if (port->modes & PARPORT_MODE_DMA)
		printk(KERN_INFO "QuickCam VC(PP): Port is DMA capable.\n");
	
	cam = kmalloc(sizeof(struct pp_cam_entry), GFP_KERNEL);
	if (cam == NULL)
	{
		printk(KERN_ERR "QuickCam VC(PP): Failed to allocate pp_cam_entry.\n");
		return -ENOMEM;
	}
	
	memset(cam,0,sizeof(struct pp_cam_entry));
	pdev = parport_register_device(port, "Connectix QuickCam VC", NULL, NULL, NULL, 0, cam);
  
	if (!pdev)
	{
		//LOG("failed to parport_register_device\n");
		printk(KERN_ERR "Failed to parport_register_device.\n");
		kfree(cam);
		return -ENXIO;
	}
  
	cam->pdev = pdev;
	cam->port = port;

	if( (qcamvc = qcamvc_register_camera(&qcamvc_pp_ops, cam, "QuickCam VC(PP)")) == NULL )
	{
		printk(KERN_ERR "Failed to qcamvc_register_camera.\n");
		parport_unregister_device(pdev);
		kfree(cam);
		return -ENXIO;
	}

	printk(KERN_INFO "QuickCam VC(PP): sucessfully registered.\n");

	spin_lock( &cam_list_lock_pp );
	list_add( &qcamvc->cam_data_list, &cam_list );
	spin_unlock( &cam_list_lock_pp );

	return 0;
}

/*****************************************************************/
/* qcamvc_pp_detach Used to remove the driver from the kernel */
/****************************************************************/
static void qcamvc_pp_detach(struct parport *port)
{
	struct list_head *tmp;
	struct qcamvc_data *qcamvc = NULL;
	struct pp_cam_entry *cam = NULL;

	spin_lock( &cam_list_lock_pp );
	list_for_each (tmp, &cam_list)
	{
		qcamvc = list_entry(tmp, struct qcamvc_data, cam_data_list);
		cam = (struct pp_cam_entry *) qcamvc->lowlevel_data;
		if (cam && cam->port->number == port->number)
		{
			break;
		}
		qcamvc = NULL;
	}
	spin_unlock( &cam_list_lock_pp );

	if (!qcamvc || !cam)
	{
		printk(KERN_ERR "qcamvc_pp_detach failed to find qcamvc in cam_data_list\n");
		return;
	}

	if(cam->open_count > 0) 
		qcamvc_close(cam);
	parport_unregister_device(cam->pdev);
	qcamvc->lowlevel_data = NULL;
	kfree(cam);

}


/*****************************************************************/
/* qcamvc_pp_attach used to attach the driver to the kernel */
/**************************************************************/
static void qcamvc_pp_attach(struct parport *port)
{
	unsigned int i;

	switch (parport_nr[0])
	{
		case QCAMVC_PARPORT_UNSPEC:
		case QCAMVC_PARPORT_AUTO:
		if (port->probe_info[0].class != PARPORT_CLASS_MEDIA ||
			strncmp(port->probe_info[0].mfr, "Connectix", 9) != 0 ||
			strncmp(port->probe_info[0].model, "QuickCam VC", 11) != 0
			)
		{
			printk(KERN_ERR "qcamvc_pp_attach failed.\n");  
			return;
		}
		qcamvc_register(port);     
		break;
      
		default:
		for (i = 0; i < PARPORT_MAX; ++i) 
		{
			if (port->number == parport_nr[i])
			{
				qcamvc_register(port);
				break;
			}
		}
		break;
	}

}

static struct parport_driver qcamvc_pp_driver = 
{
	.name = "Connectix QuickCam VC",
	.attach = qcamvc_pp_attach,
	.detach = qcamvc_pp_detach,
};

static int qcamvc_pp_init(void)
{
	if(parport_nr[0] == QCAMVC_PARPORT_OFF)
	{
		printk("  disabled\n");
		return 0;
	}

	spin_lock_init( &cam_list_lock_pp );
		
	if (parport_register_driver (&qcamvc_pp_driver))
	{
		printk(KERN_ERR "unable to register with parport\n");
		return -EIO;
	}
	
	return 0;
}

#ifdef MODULE


/********************************************************************/
/* qcamvc_pp_init_module used by the wrapper below to insmod (or modprobe) */
/*********************************************************************/
static int __init qcamvc_pp_init_module (void)
{
	printk("QuickCam VC(PP): (C) 2001 by De Marchi Daniele, <demarchidaniele@libero.it>\n");
	printk("QuickCam VC(PP): lowlevel driver version %s registered.\n",QCAMVCVERSION);
	
	if (parport[0])
	{
		/* The user gave some parameters.  Let's see what they were. */
		if (!strncmp(parport[0], "auto", 4))
		{
			parport_nr[0] = QCAMVC_PARPORT_AUTO;
		}
		else
		{
			int n;
			for (n = 0; n < PARPORT_MAX && parport[n]; n++)
			{
				if (!strncmp(parport[n], "none", 4))
				{
					parport_nr[n] = QCAMVC_PARPORT_NONE;
				}
				else
				{
					char *ep;
					unsigned long r = simple_strtoul(parport[n], &ep, 0);
					if (ep != parport[n])
					{
						parport_nr[n] = r;
					}
					else
					{
						printk("bad port specifier `%s'\n", parport[n]);
						return -ENODEV;
					}
				}
			}
		}
	}
#if defined(CONFIG_KMOD) && defined(CONFIG_PNP_PARPORT_MODULE)
	if(parport_enumerate() && !parport_enumerate()->probe_info.model)
	{
		request_module("parport_probe");
	}
#endif
  
	return qcamvc_pp_init();
}


/***************************************************************************/
/* qcamvc_pp_exit_module used by the wrapper below to rmmod */
/**************************************************************************/
static void __exit qcamvc_pp_exit_module(void)
{
	parport_unregister_driver(&qcamvc_pp_driver);

	/*
	finally unregister video.
	can't do this in detach() because the call to qcamvc_unregister_camera()
	blocks and causes "bad: scheduling while atomic!" when unloading this module.
	*/
	struct list_head *tmp;
	struct qcamvc_data *qcamvc = NULL;

	spin_lock( &cam_list_lock_pp );
	list_for_each (tmp, &cam_list)
	{
		qcamvc = list_entry(tmp, struct qcamvc_data, cam_data_list);
		/*
		lowlevel_data will be NULL if it's the one we want
		ie. lowlevel_data=NULL in detach() from call to parport_unregister_driver()
		*/
		if (!qcamvc->lowlevel_data)
		{
			list_del(&qcamvc->cam_data_list);
			break;
		}
		qcamvc = NULL;
	}
	spin_unlock( &cam_list_lock_pp );

	if (!qcamvc)
		printk("qcamvc_pp_exit_module: failed to find qcamvc in cam_data_list\n");
	else	
		qcamvc_unregister_camera(qcamvc);

}


/**************************************/
/* wrappers used by insmod and rmmod to load and unload the driver as a module */
/**************************************/
module_init(qcamvc_pp_init_module);
module_exit(qcamvc_pp_exit_module);

#else /*!MODULE*/

/**************************************************/
/* used if compiled into the kernel */
/*************************************************/
static int __init qcamvc_pp_setup(char *str)
{
	if (!strncmp(str, "parport", 7)) {
		int n = simple_strtoul(str + 7, NULL, 10);
		if (parport_ptr < PARPORT_MAX) {
			parport_nr[parport_ptr++] = n;
		} else {
			LOG("too many ports, %s ignored.\n", str);
		}
	} else if (!strcmp(str, "auto")) {
		parport_nr[0] = QCAMVC_PARPORT_AUTO;
	} else if (!strcmp(str, "none")) {
		parport_nr[parport_ptr++] = QCAMVC_PARPORT_NONE;
	}

	return 0;
}

__setup("qcamvc_pp=", qcamvc_pp_setup);


#endif /* MODULE */


/* ----------------------------------------*/
/* Parallel Port DMA Code from CPiA driver */
/* ----------------------------------------*/

#ifdef QCAMVC_PARPORT_DMA
/*-----------------------------------------------------------------------*/
/* stuff below is static stuff 
 * copied from parport_pc.c (should be made public) */

#define DPRINTK(fmt,args...) do {} while(0)
/* ECR modes */
#define ECR_SPP 00
#define ECR_PS2 01
#define ECR_PPF 02
#define ECR_ECP 03
#define ECR_EPP 04
#define ECR_VND 05
#define ECR_TST 06
#define ECR_CNF 07
#define ECR_MODE_MASK 0xe0
#define ECR_WRITE(p,v) frob_econtrol((p),0xff,(v))
#define ECONTROL(p) ((p)->base_hi + 0x2)

/* frob_control, but for ECR */
static void frob_econtrol (struct parport *pb, unsigned char m,
			   unsigned char v)
{
	unsigned char ectr = 0;

	if (m != 0xff)
		ectr = inb (ECONTROL (pb));

	DPRINTK (KERN_DEBUG "frob_econtrol(%02x,%02x): %02x -> %02x\n",
		m, v, ectr, (ectr & ~m) ^ v);

	outb ((ectr & ~m) ^ v, ECONTROL (pb));
}


/* Safely change the mode bits in the ECR 
   Returns:
	    0    : Success
	   -EBUSY: Could not drain FIFO in some finite amount of time,
		   mode not changed!
 */
static int change_mode(struct parport *p, int m)
{
	const struct parport_pc_private *priv = p->physport->private_data;
	unsigned char oecr;
	int mode;

	DPRINTK(KERN_INFO "parport change_mode ECP-ISA to mode 0x%02x\n",m);

	if (!priv->ecr) {
		printk (KERN_DEBUG "change_mode: but there's no ECR!\n");
		return 0;
	}

	/* Bits <7:5> contain the mode. */
	oecr = inb (ECONTROL (p));
	mode = (oecr >> 5) & 0x7;
	if (mode == m) return 0;

	if (mode >= 2 && !(priv->ctr & 0x20)) {
		/* This mode resets the FIFO, so we may
		 * have to wait for it to drain first. */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))
		long expire = jiffies + p->physport->cad->timeout;
#else
		unsigned long expire = jiffies + p->physport->cad->timeout;
#endif
		int counter;
		switch (mode) {
		case ECR_PPF: /* Parallel Port FIFO mode */
		case ECR_ECP: /* ECP Parallel Port mode */
			/* Busy wait for 200us */
			for (counter = 0; counter < 40; counter++) {
				if (inb (ECONTROL (p)) & 0x01)
					break;
				if (signal_pending (current)) break;
				udelay (5);
			}

			/* Poll slowly. */
			while (!(inb (ECONTROL (p)) & 0x01)) {
				if (time_after_eq (jiffies, expire))
					/* The FIFO is stuck. */
					return -EBUSY;
				__set_current_state (TASK_INTERRUPTIBLE);
				schedule_timeout ((HZ + 99) / 100);
				if (signal_pending (current))
					break;
			}
		}
	}

	if (mode >= 2 && m >= 2) {
		/* We have to go through mode 001 */
		oecr &= ~(7 << 5);
		oecr |= ECR_PS2 << 5;
		ECR_WRITE (p, oecr);
	}

	/* Set the mode. */
	oecr &= ~(7 << 5);
	oecr |= m << 5;
	ECR_WRITE (p, oecr);
	return 0;
}
#endif /* QCAMVC_PARPORT_DMA */

