/*	Copyright (C) 2004 Bertrik Sikken
	Parts of this file were adapted from the sn9c102 driver by Luca Risolia.
	
	This file is part of the sonix-cam driver.

    The sonix-cam driver is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    The sonix-cam driver is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the sonix-cam driver; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "sonix.h"

struct sn9c102_device;

/* sensor structure */
enum sn9c102_i2c_frequency { /* sensors may support both the frequencies */
	SN9C102_I2C_100KHZ = 0x01,
	SN9C102_I2C_400KHZ = 0x02
};

enum sn9c102_i2c_interface {
	SN9C102_I2C_2WIRES,
	SN9C102_I2C_3WIRES
};

struct sn9c102_sensor {
	char name[32];

	/* These sensor capabilities must be provided if the SN9C10X controller
	   needs to communicate through the sensor serial interface by using
	   at least one of the i2c functions available.
	*/
	enum sn9c102_i2c_frequency frequency;
	enum sn9c102_i2c_interface interface;
	/* These identifiers must be provided if the image sensor implements
	   the standard I2C protocol.
	*/
	u8 slave_id;
	
	/* function pointer to probe for a specific sensor */
	int (*probe)(struct uvd *uvd);
	/* function pointer to sensor specific adjust_picture */
	int (*adjust_picture)(struct uvd *uvd);
	/* function pointer to sensor specific initialisation */
	int (*init)(struct uvd *uvd);
	/* function pointer to sensor specific post processing (e.g. AE calculation) */
	int (*post_process)(struct uvd *uvd);
	
	int exposure;		/* current exposure value */
	int exp_target;		/* target value for auto-exposure */

};

/* let the controller know that a pas106b sensor exists */
extern struct sn9c102_sensor sensor_pas106b;
extern struct sn9c102_sensor sensor_pas202b;
extern struct sn9c102_sensor sensor_tas5110c1b;

/* sensors need to know the following functions */
int sn9c102_write_reg(struct sn9c102_device* cam, u16 index, u8 value);
int sn9c102_read_reg(struct sn9c102_device* cam, u16 index);
int sn9c102_i2c_read(struct sn9c102_device* cam, u8 address);
int sn9c102_i2c_write(struct sn9c102_device* cam, u8 address, u8 value);




