/*	Copyright (C) 2004 Bertrik Sikken
	Parts of this file were adapted from the sn9c102 driver by Luca Risolia.
	
	This file is part of the sonix-cam driver.

    The sonix-cam driver is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    The sonix-cam driver is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the sonix-cam driver; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "sonix_sensor.h"


static int tas5110c1b_probe(struct uvd *uvd)
{
	/* Sensor detection is based on USB pid/vid */
	if (uvd->dev->descriptor.idProduct != 0x6001 &&
	    uvd->dev->descriptor.idProduct != 0x6005 &&
	    uvd->dev->descriptor.idProduct != 0x60ab)
		return -ENODEV;

	return 0;
}


static int tas5110c1b_init(struct uvd *uvd)
{
	struct sn9c102_device *cam = (struct sn9c102_device *)uvd->user_data;
	int err = 0;
	u8 h_start, v_start;

	err += sn9c102_write_reg(cam, 0x01, 0x01);	/* control: power down sensor */
	err += sn9c102_write_reg(cam, 0x01, 0x44);	/* control: 24 MHz system clock, transfer enable */
	err += sn9c102_write_reg(cam, 0x10, 0x00);	/* R_GAIN/B_GAIN */
	err += sn9c102_write_reg(cam, 0x11, 0x00);	/* G-GAIN */
	err += sn9c102_write_reg(cam, 0x14, 0x0a);	/* OFFSET for image data */
	err += sn9c102_write_reg(cam, 0x17, 0x60);	/* 12 MHz sensor clock */
	err += sn9c102_write_reg(cam, 0x18, 0x06);	/* VSYNC/HSYNC polarity */
	err += sn9c102_write_reg(cam, 0x19, 0xfb);	/* pixel clock timing */

	err += sn9c102_i2c_write(cam, 0x80, 0xc0);
	
	/*** set crop area ***/
	h_start = 69;
	v_start = 9;

	err += sn9c102_write_reg(cam, 0x12, h_start);	/* H_START */
	err += sn9c102_write_reg(cam, 0x13, v_start);	/* V_START */

	/* Don't change ! */
	err += sn9c102_write_reg(cam, 0x1a, 0x14);	/* HO_SIZE */
	err += sn9c102_write_reg(cam, 0x1b, 0x0a);	/* VO_SIZE */
	
	return err;
}


static int tas5110c1b_adjust_picture(struct uvd *uvd)
{
	struct sn9c102_device *cam = (struct sn9c102_device *)uvd->user_data;
	int err = 0;
	int gain;
	
	/* sensor gain is controlled by contrast slider */
	gain = 0xf6 - (uvd->vpic.contrast >> 8);
	if (gain < 0) {
		gain = 0;
	}
	err += sn9c102_i2c_write(cam, 0x20, gain);
	
	return err;
}


struct sn9c102_sensor sensor_tas5110c1b = {
	.name = "TAS5110C1B",
	
	.frequency = SN9C102_I2C_100KHZ,
	.interface = SN9C102_I2C_3WIRES,
	
	.probe = tas5110c1b_probe,
	.init = tas5110c1b_init,
	.adjust_picture = tas5110c1b_adjust_picture,
	.post_process = NULL
};
