# In order to cross compile the UNH iSCSI code, you usually need to change
#	KERNEL_DIR, CROSS_COMPILE, and CROSS_CFLAGS

# Example for IA32 architecture
#KERNEL_DIR = /lib/modules/`uname -r`/build
#CROSS_COMPILE   =
#CROSS_CFLAGS	= -march=i686 -mpreferred-stack-boundary=2

# Example for ARM(Xscale) architecture
#KERNEL_DIR = /root/linux-2.4.21
#CROSS_COMPILE	= /opt/crosstool/arm-unknown-linux-gnu/gcc-3.3.2-glibc-2.3.2/bin/arm-unknown-linux-gnu-
#CROSS_CFLAGS	= -march=armv5

AS              = $(CROSS_COMPILE)as
LD              = $(CROSS_COMPILE)ld
CC              = $(CROSS_COMPILE)gcc
AR              = $(CROSS_COMPILE)ar
NM              = $(CROSS_COMPILE)nm
STRIP           = $(CROSS_COMPILE)strip
OBJCOPY         = $(CROSS_COMPILE)objcopy
OBJDUMP         = $(CROSS_COMPILE)objdump
AWK             = awk

COMMON_CFLAGS = -g -D__KERNEL__ -I$(KERNEL_DIR)/include -I$(KERNEL_DIR)/drivers/scsi -Wall -Wstrict-prototypes -Wno-trigraphs -O2 -fomit-frame-pointer -fno-strict-aliasing -pipe -DMODULE $(CROSS_CFLAGS) -fsigned-char

# define ISCSI_STATS in order to compile in code that dynamically accumulates
# statistics about the number of pdus of each type send and received, etc.
# also need to define this to display those statistics via /proc
# you probably always want this defined
COMMON_CFLAGS += -DISCSI_STATS

# define ISCSI_CHECK_MEMORY in order to compile in code that
# tracks all calls to kmalloc and kfree to ensure that something is not
# freed if it has not been previously allocated, and, at the end, prints
# any allocated but unfreed memory and gives the maximum number of
# allocated memory slots -- see common/my_memory.h and common/my_memory.c.
# you want this defined during all development and testing
# undefine it when you are confident there are no memory leaks in the code!
#COMMON_CFLAGS += -DISCSI_CHECK_MEMORY

# define CONFIG_ISCSI_DEBUG in order to compile in code that enables dynamic
# tracing and printing -- see common/debug.h and common/debug.c for details.
# this code enables tracing to be dynamically and selectively enabled/disabled.
# you want this defined during all development and testing
# undefine it when you want top performance and have completely debugged
# both the target and the initiator.
COMMON_CFLAGS += -DCONFIG_ISCSI_DEBUG

# define ISCSI_CHECK_PDU_FORMAT in order to compile in code that checks
# the unused/reserved fields of every received pdu to ensure that they have
# been set to the proper value (normally 0, but sometimes 0xffffffff).
# Also checks for other inconsistencies between fields within a pdu,
# and other violations of the standard (such as sequences that are too
# long) which can safely be ignored without damage under the philosopy
# "conservative in what you send, liberal in what you receive".
# see initiator/initiator_rx.c for most uses of this symbol.
# you want this defined during all development and testing
# undefine it when you are sure the other side always builds pdus correctly.
COMMON_CFLAGS += -DISCSI_CHECK_PDU_FORMAT

# define UNH_LOCK_DEBUG in order to compile in code that checks the usage
# of the UNH_LOCK and UNH_UNLOCK macros, on both uniprocessors and smp
# machines (this helps detect lock/unlock mismatches even on uniprocessor
# machines where spinlocks are normally nops.
# you want this defined during all development, especially on SMP machines
# undefine it when all the locks have been thoroughly debugged.
#COMMON_CFLAGS += -DUNH_LOCK_DEBUG

# define K26 in order to compile in code for the 2.6 kernels (and to compile
# out code for the 2.4 kernels).  When defining this you should NOT be
# using this version of the Makefile (Makefile-24) -- use Makefile-26
# instead (see README).
# you do NOT want this defined when using this Makefile!
#COMMON_CFLAGS += -DK26
