/*	initiator/iscsi_config.c

	vi: set autoindent tabstop=4 shiftwidth=4 :

	This file contains the core functions to initiate session to the target.
	The file when compiled generates a user-level program which calls
	iscsi_initiator.c functions through /proc filesystem interface.

*/
/*
	Copyright (C) 2001-2003 InterOperability Lab (IOL)
	University of New Hampshier (UNH)
	Durham, NH 03824

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
	USA.

	The name of IOL and/or UNH may not be used to endorse or promote products
	derived from this software without specific prior written permission.
*/

#define	_POSIX_C_SOURCE	199506L

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <asm/param.h>
#include <asm/types.h>
#include <sys/param.h>
#include <scsi/scsi.h>
#include <sys/stat.h>

#include "iscsi_common.h"
#include "debug.h"

extern int ioctl(int, int, ...);

#define DEFAULT_HOST_NO		0
#define DEFAULT_TARGET_NO	0
#define DEFAULT_LUN_NO		-1
#define DEFAULT_CID_NO		0
#define WHITE_SPACE	" \t\v\f\n\r"

#define UNH_ISCSI_DEV_DIR	"/opt/unh/iscsi/dev"
#define MAX_LUNS			8

char server_node[MAXHOSTNAMELEN];
char target_dir[MAXPATHLEN];
char target_node[MAXPATHLEN];
char cmd[MAXPATHLEN];

int
create_device_nodes(char *target_dir, int host_no, int target_no, int lun_no)
{
	char disk_fmt[] = "/dev/sd%c";
	char tape_fmt[] = "/dev/st%d";
	char cd_fmt[] = "/dev/scd%d";
	char disk_name[10], disk_part[10];
	char tape_name[10], tape_subname[10];
	char cd_name[10];
	char tape_types[] = { 'a', 'l', 'm', 0 };
	int fd, ret = 0, id_buf[2], found = 0;
	int host, chan, lun, id, i = 0, p = 0;

	/* First do the disks */
	for (i = 'a'; i <= 'z'; i++) {
		sprintf(disk_name, disk_fmt, i);

		fd = open(disk_name, O_RDONLY);
		if (fd <= 0)
			continue;

		ret = ioctl(fd, SCSI_IOCTL_GET_IDLUN, &id_buf);
		close(fd);
		if (ret) {
			continue;
		}

		TRACE(TRACE_DEBUG,
			  "create_device_nodes disk_name: %s, id_buf: 0x%08x\n",
			  disk_name, id_buf[0]);

		host = (id_buf[0] >> 24) & 0xff;
		chan = (id_buf[0] >> 16) & 0xff;
		lun = (id_buf[0] >> 8) & 0xff;
		id = (id_buf[0] & 0xff);

		if (host != host_no || chan != 0 ||
				id != target_no || lun != lun_no) {
			continue;
		}

		found++;

		TRACE(TRACE_DEBUG,
			  "create_device_nodes found %d, target_dir: %s, "
			  "lun_no %d\n", found, target_dir, lun_no);

		sprintf(target_node, "%s/l%d", target_dir, lun_no);
		ret = link(disk_name, target_node);
		if (ret) {
			printf("Unable to link %s to %s \n", target_node,
				disk_name);
			goto out;
		}

		TRACE(TRACE_DEBUG, "Able to link %s to %s \n", target_node,
			disk_name);

		for (p = 1; p <= 15; p++) {
			sprintf(target_node, "%s/l%dp%d", target_dir,
				lun_no, p);
			sprintf(disk_part, "%s%d", disk_name, p);
			ret = link(disk_part, target_node);
			if (ret) {
				printf("Unable to link %s to %s \n",
					target_node, disk_part);
				goto out;
			}
		}
	}

	if (found)
		goto out;

	/* Check for SCSI tapes */
	for (i = 0; i <= 9; i++) {
		sprintf(tape_name, tape_fmt, i);

		fd = open(tape_name, O_RDONLY);
		if (fd <= 0)
			continue;

		ret = ioctl(fd, SCSI_IOCTL_GET_IDLUN, &id_buf);
		close(fd);
		if (ret) {
			continue;
		}

		TRACE(TRACE_DEBUG,
			  "create_device_nodes tape_name: %s, id_buf: 0x%08x\n",
			  tape_name, id_buf[0]);

		host = (id_buf[0] >> 24) & 0xff;
		chan = (id_buf[0] >> 16) & 0xff;
		lun = (id_buf[0] >> 8) & 0xff;
		id = (id_buf[0] & 0xff);

		if (host != host_no || chan != 0 || id != target_no ||
			lun != lun_no) {
			continue;
		}

		found++;

		TRACE(TRACE_DEBUG,
			  "create_device_nodes found %d, target_dir: %s, "
			  "lun_no %d\n", found, target_dir, lun_no);

		sprintf(target_node, "%s/l%d", target_dir, lun_no);
		ret = link(tape_name, target_node);
		if (ret) {
			printf("Unable to link %s to %s \n",
				target_node, tape_name);
			goto out;
		}

		TRACE(TRACE_DEBUG, "Able to link %s to %s \n", target_node,
			tape_name);

		for (p = 0; tape_types[p]; p++) {
			TRACE(TRACE_DEBUG, "p: %d, tape_types[p]: %c\n", p,
				tape_types[p]);
			sprintf(tape_subname, "%s%c", tape_name, tape_types[p]);
			sprintf(target_node, "%s/l%d%c", target_dir,
				lun_no, tape_types[p]);
			ret = link(tape_subname, target_node);
			if (ret) {
				printf("Unable to link %s to %s \n",
					target_node, tape_subname);
				goto out;
			}
			TRACE(TRACE_DEBUG, "Able to link %s to %s \n",
				  target_node, tape_subname);
		}
	}

	if (found)
		goto out;

	/* Check for SCSI CD-ROMs */
	for (i = 0; i <= 9; i++) {
		sprintf(cd_name, cd_fmt, i);

		fd = open(cd_name, O_RDONLY);
		if (fd <= 0)
			continue;

		ret = ioctl(fd, SCSI_IOCTL_GET_IDLUN, &id_buf);
		close(fd);
		if (ret) {
			continue;
		}

		TRACE(TRACE_DEBUG,
			  "create_device_nodes cd_name: %s, id_buf: 0x%08x\n",
			  cd_name, id_buf[0]);

		host = (id_buf[0] >> 24) & 0xff;
		chan = (id_buf[0] >> 16) & 0xff;
		lun = (id_buf[0] >> 8) & 0xff;
		id = (id_buf[0] & 0xff);

		if (host != host_no || chan != 0 || id != target_no ||
				lun != lun_no) {
			continue;
		}

		found++;

		TRACE(TRACE_DEBUG,
			  "create_device_nodes found %d, target_dir: %s, "
			  "lun_no %d\n", found, target_dir, lun_no);

		sprintf(target_node, "%s/l%d", target_dir, lun_no);
		ret = link(cd_name, target_node);
		if (ret) {
			printf("Unable to link %s to %s \n", target_node,
				cd_name);
			goto out;
		}
	}

  out:
	TRACE(TRACE_DEBUG, "create_device_nodes exiting\n");

	return (ret);
}

/*	creates the directory given by path, including all
 * 	directories in path
 *	path must be an absolute path name
 *	returns 0 on success, -1 on error
 */
int
make_full_path_dir(char *path)
{
	char *ptr, *partial_path = NULL;
	int len, mode;
	int retval = -1;			/* assume this function fails */

	if (path == NULL || (len = strlen(path)) <= 0) {
		/* path is bogus */
		goto early_out;
	}

	if (*path != '/') {
		/* path is relative, not absolute */
		goto early_out;
	}
	/* else path is absolute */

	if ((ptr = partial_path = malloc(len + 1)) == NULL)
		goto early_out;

	/* we need a copy of path because we are going to mess it up */
	strcpy(partial_path, path);

	mode = X_OK;	/* only need x access to intermediate directories */
	if (len == 1) {
		mode |= R_OK | W_OK;	/* need rwx access to final directory */
	}

	if (access("/", mode)) {
		/* we do not have proper access to root, leave with error */
		goto out;
	}

	if (len == 1) {
		/* the entire path is just "/" and we have rwx access to it */
		goto ok;
	}

	do {
		/* repair any previous damage to partial-path */
		*ptr++ = '/';
		if ((ptr = strchr(ptr, '/')) == NULL) {
			/* have last component in string, ptr is
				now NULL to exit loop */
			/* need rwx access to final directory */
			mode |= R_OK | W_OK;
		} else {
			/* still have subdirectories to follow,
				isolate this subdir */
			*ptr = '\0';
		}
		if (access(partial_path, mode)) {
			/* don't have desired access to this subdirectory */
			if (mkdir(partial_path, 0744)) {
				/* can't create this subdirectory,
					return an error */
				printf("Unable to create directory %s: %s\n",
					   partial_path, strerror(errno));
				goto out;
			}
		}
	}
	while (ptr);

  ok:
	retval = 0;

  out:
	free(partial_path);

  early_out:
	return retval;
}

/*	write a magic line to the proc file system to cause the scsi
 *	mid-level to redo the inquiry for our device
 */
void
do_inquiry(int add, int host_no, int target_no, int lun_no)
{
	int fd, n, len, ret = 0;
	static char *file_name = "/proc/scsi/scsi";
	char text_line[128];

	TRACE(TRACE_DEBUG, "Entering do_inquiry\n");

	if ((fd = open(file_name, O_RDWR)) < 0) {
		perror(file_name);
		return;
	}

	TRACE(TRACE_DEBUG, "do_inquiry file_name: %s\n", file_name);

	len = sprintf(text_line, "scsi %s-single-device %d 0 %d %d",
				  add ? "add" : "remove", host_no,
				  target_no, lun_no);

	len++;	/* for writing the last null character */

	TRACE(TRACE_DEBUG, "write to scsi: %s\n", text_line);

	if ((n = write(fd, text_line, len)) < 0) {
#ifdef CONF_ISCSI_DEBUG
		perror(file_name);
#endif							/* CONF_ISCSI_DEBUG */
	} else if (n != len && n != 0)
		printf("%s: wrote %d bytes, expected %d\n", file_name, n, len);
	/* FEATURE - ADD DEVICE ENTRY */
	else {
		if (add) {
			if (access(UNH_ISCSI_DEV_DIR, R_OK | W_OK | X_OK)) {
				ret = make_full_path_dir(UNH_ISCSI_DEV_DIR);
				//ret = mkdir(UNH_ISCSI_DEV_DIR, 0744);
				if (ret) {
					goto out;
				}
			}

			sprintf(target_dir, "%s/%d", UNH_ISCSI_DEV_DIR,
				target_no);
			if (access(target_dir, R_OK | W_OK | X_OK)) {
				ret = make_full_path_dir(target_dir);
				//ret = mkdir(target_dir, 0744);
				if (ret) {
					goto out;
				}
			}

			/* Remove old device files */
			sprintf(cmd, "/bin/rm -rf %s/%d/l%d*",
					UNH_ISCSI_DEV_DIR, target_no, lun_no);
			ret = system(cmd);
			if (ret) {
				printf("Unable to remove old device files.\n");
				goto out;
			}

			ret = create_device_nodes(target_dir, host_no,
					target_no, lun_no);
			if (ret) {
				printf("Unable to create device nodes!\n");
			}
		} else {
			sprintf(cmd, "/bin/rm -rf %s/%d",
				UNH_ISCSI_DEV_DIR, target_no);
			ret = system(cmd);
			if (ret)
				printf("Removal device files failed!\n");
		}
	}
  out:
	close(fd);

	TRACE(TRACE_DEBUG, "Leaving do_inquiry\n");
}

/*
 * Connects to a remote target by writing an ip address to a /proc file.
 * If the ip address is 0, a disconnect will result.
 * Returns	=0 if connect/disconnect worked ok for this target
 * <0 if connect/disconnect failed
 * >0 if just adding/removing another lun from this target
*/
int
do_connect(char *ip_address_str, int port_no, int host_no,
		   int target_no, int lun_no, int cid_no)
{
	int fd, n, len, result;
	static char file_name[64];
	static char text_line[128];

	TRACE(TRACE_DEBUG, "Entering do_connect\n");

	sprintf(file_name, "/proc/scsi/iscsi_initiator/%d", host_no);

	result = -1;
	if ((fd = open(file_name, O_RDWR)) < 0) {
		perror(file_name);
	} else { len =
		sprintf(text_line,"iscsi_initiator ip %s port %d target %d "
			"lun %d cid %d", ip_address_str, port_no, target_no,
			lun_no, cid_no);

		TRACE(TRACE_DEBUG, "writing string: %s of length: %d\n",
			  text_line, len);

		if ((n = write(fd, text_line, len)) < 0) {
			perror(file_name);
		} else if (n != len && n != 0)
			printf("%s: wrote %d bytes, expected %d\n",
				file_name, n, len);
		else
			result = n;
		close(fd);
	}

	TRACE(TRACE_DEBUG, "Leaving do_connect, result %d\n", result);
	return result;
}

int do_server(int action, int tag, char *ip_address, int port_no, int host_no) {
	int fd, n, len, result;
	static char file_name[64];
	static char text_line[128];

	TRACE(TRACE_DEBUG, "Entering do_server\n");

	sprintf(file_name, "/proc/scsi_target/iscsi_target/%d", host_no);

	result = -1;
	if ((fd = open(file_name, O_RDWR)) < 0) {
		perror(file_name);
	} else { len =
		sprintf(text_line,"iscsi_target ip %s port %d tag %d action %d",
			ip_address, port_no, tag, action);

		TRACE(TRACE_DEBUG, "writing string: %s of length: %d\n",
			  text_line, len);

		if ((n = write(fd, text_line, len)) < 0) {
			perror(file_name);
		} else if (n != len && n != 0)
			printf("%s: wrote %d bytes, expected %d\n",
				file_name, n, len);
		else
			result = n;
		close(fd);
	}

	TRACE(TRACE_DEBUG, "Leaving do_server, result %d\n", result);
	return result;
}

void
print_ip_address(int address, char *message)
{
	struct in_addr my_in_addr;

	TRACE(TRACE_DEBUG, "Entering print_ip_address\n");
	my_in_addr.s_addr = htonl(address);
	printf("%13s address: %#010x -- %s\n",
		   message, address, inet_ntoa(my_in_addr));

	TRACE(TRACE_DEBUG, "Leaving print_ip_address\n");
}

/* converts from a dns name to the ip address as a string
 * returns address family (AF_INET or AF_INET6) if ok, 0 on error
 */
int
cnv_from_dns_name(char *dns_name, char *ip_address_string, int length)
{
	struct hostent *node_ptr;

	TRACE(TRACE_DEBUG, "Entering cnv_from_dns_name, dns_name %s\n", dns_name);

	/*      get structure for host node with dns name */
	if ((node_ptr = gethostbyname(dns_name)) == NULL) {
		herror(dns_name);
		return 0;
	}

	strcpy(server_node, node_ptr->h_name);

	if (inet_ntop(node_ptr->h_addrtype, node_ptr->h_addr, ip_address_string,
															length) == NULL) {
		printf("Unable to convert IP address of \"%s\"\n",
															node_ptr->h_name);
		return 0;
	}

	TRACE(TRACE_DEBUG, "Leaving cnv_from_dns_name, ip address %s, family %d\n",
									ip_address_string, node_ptr->h_addrtype);
	return node_ptr->h_addrtype;
}

int
cnv_from_ip_address(int ip_address)
{
	struct hostent *node_ptr;
	struct in_addr my_in_addr;

	TRACE(TRACE_DEBUG, "Entering cnv_from_ip_address\n");
	my_in_addr.s_addr = htonl(ip_address);

	/*      get structure for host node with ip address */
	if ((node_ptr = gethostbyaddr((char *) &my_in_addr, 4,
		AF_INET)) == NULL) {
			/* this ip address is not known to the Name service.
			 * So just use the dotted-decimal notation as
			 * the name.
			 */
		sprintf(server_node, "%s", inet_ntoa(my_in_addr));
	} else {
		/* found a DNS name for this ip address, use it. */
		strcpy(server_node, node_ptr->h_name);
	}

	TRACE(TRACE_DEBUG, "Leaving cnv_from_ip_address, server_node: %s\n",
		  server_node);
	return ip_address;
}

#define	UP	0
#define	DOWN	1

/* returns 1 if ip_address_str is a valid ipv6 address,
		   0 if ip_address_str has wrong form for ipv6 address
   does not return if ip_address_str is invalid ipv6 address
*/
int
is_ipv6(char *ip_address_str, char *ipv6_address)
{
	int len, k;
	char *ptr = NULL;

	len = strlen(ip_address_str);
	if( *ip_address_str == '['  && *(ip_address_str+len-1) == ']') {
		/* have bracketed address, check only what is inside the brackets */
		ip_address_str++;
		len -= 2;
		if (strspn(ip_address_str, "0123456789abcdefABCDEF.:") != len)
			return 0;
		if ((ptr = malloc(len+2)) == NULL)
			return 0;
		strcpy(ptr, ip_address_str);
		*(ptr + len) = '\0';
		TRACE(TRACE_DEBUG, "scanning for IPV6 address\n");
		k = inet_pton(AF_INET6, ptr, ipv6_address);

	} else if (strspn(ip_address_str, "0123456789abcdefABCDEF.:") != len) {
		return 0;

	} else {
		TRACE(TRACE_DEBUG, "scanning for IPV6 address\n");
		k = inet_pton(AF_INET6, ip_address_str, ipv6_address);
	}

	if (k == 0) {
		printf("illegal ipv6 address \"%s\"\n", ip_address_str);
		exit(EXIT_FAILURE);
	} else if (k < 0) {
		printf("%s: %s\n", ip_address_str, strerror(errno));
		exit(EXIT_FAILURE);
	}

	if (ptr != NULL)
		free(ptr);
	return 1;
}

int
main(int argc, char *argv[])
{
	int tag;
	int ip_address, host_no, target_no, lun_no, port_no, cid_no;
	int updown, i, k, server;
	char *endptr, *ip_address_str = NULL;
	char ipv4_address[4];
	char ipv6_address[16];
	char ip_address_string[INET6_ADDRSTRLEN];

	if (argc < 2) {
		printf
			("Usage: iscsi_config up/down [server] [ip=address_or_name] "
			 "[host=number] [port=number] [target=number] [lun=number] "
			 "[cid=number] [tag=number]\n");
		exit(EXIT_FAILURE);
	}

	if (strcmp(argv[1], "up") == 0)
		updown = UP;
	else if (strcmp(argv[1], "down") == 0)
		updown = DOWN;
	else {
		printf
			("parameter 1 must be either \"up\" or \"down\", not \"%s\"\n",
			 argv[1]);
		exit(EXIT_FAILURE);
	}

	ip_address = 0;

	/*get the DNS name of the local host node on which we are running */
	/*****
	if( gethostname(local_node, MAXHOSTNAMELEN) < 0 )
		{
		perror("gethostname");
		exit(EXIT_FAILURE);
		}
	*****/

	port_no = ISCSI_WKP;
	host_no = DEFAULT_HOST_NO;
	target_no = DEFAULT_TARGET_NO;
	lun_no = DEFAULT_LUN_NO;
	cid_no = DEFAULT_CID_NO;
	tag = 0;
	server = 0;

	for (i = 2; i < argc; i++) {
		if (strncmp(argv[i], "ip=", 3) == 0) {
			/* dns name or ip address of remote server */
			ip_address_str = &argv[i][3];
			TRACE(TRACE_DEBUG, "scanning ip=\"%s\"\n", ip_address_str);
			if (strspn(ip_address_str, "0123456789.")==strlen(ip_address_str)) {
				/* contains only dots and decimal digits, assume ipv4 address */
				TRACE(TRACE_DEBUG, "scanning for IPV4 address\n");
				k = inet_pton(AF_INET, ip_address_str, ipv4_address);
				if (k == 0) {
					printf("illegal ipv4 address \"%s\"\n", ip_address_str);
					exit(EXIT_FAILURE);
				} else if (k < 0) {
					printf("%s: %s\n", ip_address_str, strerror(errno));
					exit(EXIT_FAILURE);
				}
				TRACE(TRACE_DEBUG, "valid IPV4 address\n");
			} else if (is_ipv6(ip_address_str, ipv6_address)) {
				TRACE(TRACE_DEBUG, "valid IPV6 address\n");
			} else {
				/* must be dns name, look it up to get ip address as a string */
				TRACE(TRACE_DEBUG, "looking up DNS name\n");
				if ((k = cnv_from_dns_name(ip_address_str, ip_address_string,
							INET6_ADDRSTRLEN)) != AF_INET && k != AF_INET6) {
					exit(EXIT_FAILURE);
				}
				ip_address_str = ip_address_string;
				if (k == AF_INET) {
					TRACE(TRACE_DEBUG, "valid IPV4 address\n");
				} else {
					TRACE(TRACE_DEBUG, "valid IPV6 address\n");
				}
			}
		} else if (strncmp(argv[i], "port=", 5) == 0) {
			/* port number of target */
			errno = 0;
			port_no = strtoul(&argv[i][5], &endptr, 0);
			if (errno != 0 || strspn(endptr, WHITE_SPACE) != strlen(endptr)
				|| port_no == 0 || port_no > MASK_16_BITS) {
				printf("illegal port number \"%s\"\n", &argv[i][5]);
				exit(EXIT_FAILURE);
			}
		} else if (strncmp(argv[i], "host=", 5) == 0) {
			/* number of host adaptor */
			errno = 0;
			host_no = strtoul(&argv[i][5], &endptr, 0);
			if (errno != 0 || strspn(endptr, WHITE_SPACE) !=
				strlen(endptr)) {
				printf("illegal host number \"%s\"\n", &argv[i][5]);
				exit(EXIT_FAILURE);
			}
		} else if (strncmp(argv[i], "target=", 7) == 0) {
			/* id number of target */
			errno = 0;
			target_no = strtoul(&argv[i][7], &endptr, 0);
			if (errno != 0 || strspn(endptr, WHITE_SPACE) !=
				strlen(endptr)) {
				printf("illegal target number \"%s\"\n", &argv[i][7]);
				exit(EXIT_FAILURE);
			}
		} else if (strncmp(argv[i], "lun=", 4) == 0) {
			/* number of lun */
			errno = 0;
			lun_no = strtoul(&argv[i][4], &endptr, 0);
			if (errno != 0 || strspn(endptr, WHITE_SPACE) !=
				strlen(endptr)
				|| lun_no > MAX_LUNS) {
				printf("illegal lun number \"%s\"\n", &argv[i][4]);
				exit(EXIT_FAILURE);
			}
		} else if (strncmp(argv[i], "cid=", 4) == 0) {
			/* number of cid */
			errno = 0;
			cid_no = strtoul(&argv[i][4], &endptr, 0);
			if (errno != 0 || strspn(endptr, WHITE_SPACE) != strlen(endptr)) {
				printf("illegal cid number \"%s\"\n", &argv[i][4]);
				exit(EXIT_FAILURE);
			}
		} else if (strncmp(argv[i], "tag=", 4) == 0) {
			/* number of portal group tag */
			errno = 0;
			tag = strtoul(&argv[i][4], &endptr, 0);
			if (errno != 0 || strspn(endptr, WHITE_SPACE) != strlen(endptr)) {
				printf("illegal tag number \"%s\"\n", &argv[i][4]);
				exit(EXIT_FAILURE);
			}
		} else if (strncmp(argv[i], "server", 6) == 0) {
			server = 1;
		} else {
			printf("illegal parameter \"%s\"\n", argv[i]);
			exit(EXIT_FAILURE);
		}
	}

	if (server == 1) {
		do_server(updown, tag, ip_address_str, port_no, host_no);
	} else if (updown == UP) {
		if (lun_no == DEFAULT_LUN_NO) {
			/* no LUN is specified, bring up all LUNs */
			for (i = 0; i < MAX_LUNS; i++) {
				if (do_connect(ip_address_str, port_no, host_no, target_no,
												i, cid_no) > 0) {
					do_inquiry(1, host_no, target_no, i);
				}
			}
		} else {
			/* LUN is specified, bring just that LUN up */
			if (do_connect(ip_address_str, port_no, host_no, target_no,
											lun_no, cid_no) > 0) {
				do_inquiry(1, host_no, target_no, lun_no);
			}
		}
	} else {
		if (lun_no != DEFAULT_LUN_NO) {
			/* LUN is specified, bring just that LUN down */
			if (do_connect(ip_address_str, 0, host_no, target_no,
											lun_no, cid_no) > 0) {
				do_inquiry(0, host_no, target_no, lun_no);
			}
		} else {
			/* Process all LUNs for connection tear down */
			do_connect(ip_address_str, 0, host_no, target_no, 0, cid_no);
			for (i = 0; i < MAX_LUNS; i++) {
				do_inquiry(0, host_no, target_no, i);
			}
		}
	}

	exit(EXIT_SUCCESS);
}
