#ifndef h_Zc030x_Cameras_h
#define h_Zc030x_Cameras_h

/* Declaration file for camera specific */
/* Zc030x     -- Driver --      Zc030x  */
/* This file is under GPL               */
/* Copyright :
         Martin     Braun
         Cyril      Russo
----------------------------------------*/

/* Include the header */
#include "zc030x_kerneldef.h"
#include "zc030x_i2c.h"
#include "zc030x.h"
#include "zc030x_sensorsig.h"


/* Product ID and Vendor ID of "supported" cameras */
typedef struct 
{
    const __u16 ProductID;          /* The product ID */
    const __u16 VendorID;           /* The vendor ID */
    const __u16 Index;              /* The index to the other tables */
    const char  Name[30];           /* The camera name */
    const I2CInitSequence * pI2CInitSeq;    /* The I2C init sequence */
} CameraList;


/* And the supported camera */
extern const CameraList CamList[];

/* Declare the static usb_device_id
   Sadly, this can't be dynamic !            */
extern const struct usb_device_id zc030x_table[];

/* Declare the sensor list */
typedef struct 
{
    const __u16              Index;          /* The index to the other tables */
    const __u8               Address;        /* The address in the bridge */
    const char               Name[30];       /* The sensor name */
    const signature *        pSignature;     /* The I2C sensor signature */
} SensorList;

/* Declare the sensor size */
typedef struct 
{
    const __u16              Width;          /* Width of the sensor */
    const __u16              Height;         /* Height of the sensor */
    const __u8               FirstPixel;     /* First pixel type */
    const __u8               Enable;         /* Chip select Enable ?*/
} SensorSize;

/* And the supported sensor sizes */
extern const SensorSize SensorSizeList[];

/* And the supported sensors */
extern const SensorList SensList[];

/* And the sensors config */
extern const SensorConfig * SensConfList[];

/* Detect the camera */
int zc030x_detect_camera(struct usb_zc030x * dev);
    
/* Configure the camera */
int zc030x_configure_camera(struct usb_zc030x * dev, int width, int height, int quality);

/* Send the sharpness sequence */
int zc030x_send_sharpness(struct usb_device *udev, const __u8 * value);
    
/* Send the gamma sequence */
int zc030x_send_gamma(struct usb_device *udev, const __u16 brightness, const __u16 contrast, const __u16 gamma);
    
/* Send the RGB to RGB conversion matrix */
int zc030x_send_RGBmatrix(struct usb_device *udev, __u8 matrix[3][4]); 



    
#endif
