#ifndef h_Zc030x_Frame_h
#define h_Zc030x_Frame_h

/* Kernel wait queues */
#include <linux/wait.h>
/* Kernel semaphore */
#include <linux/interrupt.h>

/* States for each frame buffer. */
enum { 
    FRAME_UNUSED,           /* Unused (no MCAPTURE) */
    FRAME_READY,            /* Ready to start grabbing */
    FRAME_GRABBING,         /* In the process of being grabbed into */
    FRAME_DONE,             /* Finished grabbing, but not been synced yet */
    FRAME_ERROR,            /* Something bad happened while processing */
    FRAME_ABORTING,         /* Aborting everything. Caused by hot unplugging.*/
};

/* Structure to hold frame info & data */
typedef struct 
{
    unsigned char * CompData;        /* Compressed frame buffer */
    int             CompLen;         /* Compressed size len */
    unsigned char * Data;            /* Decoded frame buffer */
    int             SeqNumber;       /* Frame sequence number */
    
    int             Depth;           /* Bytes per pixel */
    int             Width;           /* Width application is expecting */
    int             Height;          /* Height */

    int             HeaderWidth;     /* Width the frame actually is */
    int             HeaderHeight;    /* Height */

    unsigned int    Format;          /* Format asked by apps for this frame */

    volatile int    GrabState;       /* State of grabbing */
    int             ScanState;       /* State of scanning */

    long            ScanLength;      /* uncompressed, raw data length of frame */
    long            bytes_read;      /* amount of scanlength that has been read from *data */

    wait_queue_head_t wq;            /* Processes waiting */

    int             last_packet;     /* sequence number for last packet */

    struct semaphore Lock;           /* Frame lock */
} zc030x_frame;

#endif
