#ifndef h_Zc030x_Reg_h
#define h_Zc030x_Reg_h

/* This file implement the bridge register communication      */
/* This project is under GPL                                  */
/* - Driver file -            Zc030x       - Driver file -    */ 
/* Copyright: 
               Martin   Braun
               Cyril    Russo

--------------------------------------------------------------*/

/* Include definitions */
#include "zc030x_kerneldef.h"

/* Error definitions */
#define REG_Error   -1
#define REG_Warning  0
#define REG_OK       1


/* Reset the chip */
int zc030x_reg_reset(struct usb_device *udev);

/* Read a register from the bridge chip */
int zc030x_reg_read(struct usb_device *udev, __u16 addr, __u16 value, __u16 length);

/* Write a register on the bridge chip */
int zc030x_reg_write(struct usb_device *udev, __u16 addr, __u16 value);

/* Set picture size */
int zc030x_reg_setsize(struct usb_zc030x *dev, __u16 width, __u16 height, __u8 mask2);


/* Define the register map */
#define ZcReg_SystemControl     0x0000
#define ZcReg_SystemOperating   0x0001
/* Picture size */
#define ZcReg_ClockSelect       0x0002
#define ZcReg_FrameWidthHigh    0x0003
#define ZcReg_FrameWidthLow     0x0004
#define ZcReg_FrameHeightHigh   0x0005
#define ZcReg_FrameHeightLow    0x0006
/* JPEG control */
#define ZcReg_ClockSetting      0x0008
/* Test mode */
#define ZcReg_TestModeControl   0x000b
/* Frame retreiving */
#define ZcReg_LastAcqTime       0x000c
#define ZcReg_MonitorRes        0x000d
#define ZcReg_TimestampHigh     0x000e
#define ZcReg_TimestampLow      0x000f
#define ZcReg_FrameLost         0x0018
#define ZcReg_AutoAdjustFPS     0x0019
#define ZcReg_LastFrameState    0x001a
#define ZcReg_DataCounter       0x0025
/* Stream and sensor specific */
#define ZcReg_CMOSSensorSelect  0x0010
#define ZcReg_VideoStatus       0x0011
#define ZcReg_VideoControlFunc  0x0012
/* Horizontal and vertical synchros */
#define ZcReg_HSYNC_0           0x001d
#define ZcReg_HSYNC_1           0x001e
#define ZcReg_HSYNC_2           0x001f
#define ZcReg_HSYNC_3           0x0020
/* Target picture size in byte */
#define ZcReg_TargetPictSize_0  0x0022
#define ZcReg_TargetPictSize_1  0x0023
#define ZcReg_TargetPictSize_2  0x0024
/* Audio registers */
#define ZcReg_AudioADC          0x0030
#define ZcReg_AudioStreamStatus 0x0031
#define ZcReg_AudioStatus       0x0032

/* Sensor interface */
#define ZcReg_HBlankHigh        0x0080
#define ZcReg_HBlankLow         0x0081
#define ZcReg_ResetLevelAddr    0x0082
#define ZcReg_RGainAddr         0x0083
#define ZcReg_GGainAddr         0x0084
#define ZcReg_BGainAddr         0x0085
#define ZcReg_ExpTimeHigh       0x0086
#define ZcReg_ExpTimeMid        0x0087
#define ZcReg_ExpTimeLow        0x0088
#define ZcReg_ResetBlackHigh    0x0089
#define ZcReg_ResetWhiteHigh    0x008a
#define ZcReg_I2CDeviceAddr     0x008b
#define ZcReg_I2CIdleAndNAck    0x008c
#define ZcReg_CompabilityMode   0x008d
#define ZcReg_CompabilityMode2  0x008e
/* I2C control */
#define ZcReg_I2CCommand        0x0090
#define ZcReg_I2CStatus         0x0091
#define ZcReg_I2CAddressSelect  0x0092
#define ZcReg_I2CSetValue       0x0093
#define ZcReg_I2CWriteACK       0x0094
#define ZcReg_I2CRead           0x0095
#define ZcReg_I2CReadACK        0x0096
/* Window inside the sensor array */
#define ZcReg_WinYStartHigh     0x0097
#define ZcReg_WinYStartLow      0x0098
#define ZcReg_WinXStartHigh     0x0099
#define ZcReg_WinXStartLow      0x009a
#define ZcReg_WinHeightHigh     0x009b
#define ZcReg_WinHeightLow      0x009c
#define ZcReg_WinWidthHigh      0x009d
#define ZcReg_WinWidthLow       0x009e
#define ZcReg_FirstYHigh        0x0119
#define ZcReg_FirstYLow         0x011a
#define ZcReg_FirstXHigh        0x011b
#define ZcReg_FirstXLow         0x011c
/* Max sensor array size */
#define ZcReg_MaxXHigh          0x009f
#define ZcReg_MaxXLow           0x00a0
#define ZcReg_MaxYHigh          0x00a1
#define ZcReg_MaxYLow           0x00a2
#define ZcReg_ExposureTimeHigh  0x00a3
#define ZcReg_ExposureTimeLow   0x00a4
#define ZcReg_ExposureGain      0x00a5
#define ZcReg_ExposureBlackLvl  0x00a6

/* Other registers */
#define ZcReg_OperationMode     0x0100
#define ZcReg_SensorCorrection  0x0101
/* Gains */
#define ZcReg_RGain             0x0116
#define ZcReg_GGain             0x0117
#define ZcReg_BGain             0x0118
#define ZcReg_GlobalGain        0x011d
#define ZcReg_DigitalGain       0x01a8
#define ZcReg_DigitalLimitDiff  0x01a9
#define ZcReg_DigitalGainStep   0x01aa


/* Auto correction */
#define ZcReg_AutoCorrectEnable 0x0180
#define ZcReg_WinXStart         0x0181
#define ZcReg_WinXWidth         0x0182
#define ZcReg_WinXCenter        0x0183
#define ZcReg_WinYStart         0x0184
#define ZcReg_WinYWidth         0x0185
#define ZcReg_WinYCenter        0x0186
/* Gain range */
#define ZcReg_MaxGain           0x0187
#define ZcReg_MinGain           0x0188
/* Auto exposure and white balance */
#define ZcReg_AWBStatus         0x0189
#define ZcReg_AWBFreeze         0x018a
#define ZcReg_AEStatus          0x018b
#define ZcReg_AEFreeze          0x018c
#define ZcReg_AEUnfreeze        0x018f
#define ZcReg_ExposureLimitHigh 0x0190
#define ZcReg_ExposureLimitMid  0x0191
#define ZcReg_ExposureLimitLow  0x0192
#define ZcReg_AntiFlickerHigh   0x0195
#define ZcReg_AntiFlickerMid    0x0196
#define ZcReg_AntiFlickerLow    0x0197
/* What is this ? */
#define ZcReg_YTarget           0x018d
#define ZcReg_ResetLvl          0x018e
/* Color */
#define ZcReg_RedMeanAfterAGC   0x01a0
#define ZcReg_GreenMeanAfterAGC 0x01a1
#define ZcReg_BlueMeanAfterAGC  0x01a2
#define ZcReg_RedMeanAfterAWB   0x01a3
#define ZcReg_GreenMeanAfterAWB 0x01a4
#define ZcReg_BlueMeanAfterAWB  0x01a5
#define ZcReg_YMeanAfterAE      0x01a6
#define ZcReg_CalcGlobalMean    0x01a7

#define ZcReg_BlueMeanAfterAGC  0x01a2



/* Matrixes */

/* Color matrix is like : 
R' = R * RGB00 + G * RGB01 + B * RGB02 + RGB03
G' = R * RGB10 + G * RGB11 + B * RGB22 + RGB13
B' = R * RGB20 + G * RGB21 + B * RGB12 + RGB23   */
#define ZcReg_RGB00             0x010a
#define ZcReg_RGB01             0x010b
#define ZcReg_RGB02             0x010c
#define ZcReg_RGB03             0x0113
#define ZcReg_RGB10             0x010d
#define ZcReg_RGB11             0x010e
#define ZcReg_RGB12             0x010f
#define ZcReg_RGB13             0x0114
#define ZcReg_RGB20             0x0110
#define ZcReg_RGB21             0x0111
#define ZcReg_RGB22             0x0112
#define ZcReg_RGB23             0x0115


/* Gamma matrix */
#define ZcReg_Gamma00           0x0120
#define ZcReg_Gamma01           0x0121
#define ZcReg_Gamma02           0x0122
#define ZcReg_Gamma03           0x0123
#define ZcReg_Gamma04           0x0124
#define ZcReg_Gamma05           0x0125
#define ZcReg_Gamma06           0x0126
#define ZcReg_Gamma07           0x0127
#define ZcReg_Gamma08           0x0128
#define ZcReg_Gamma09           0x0129
#define ZcReg_Gamma0a           0x012a
#define ZcReg_Gamma0b           0x012b
#define ZcReg_Gamma0c           0x012c
#define ZcReg_Gamma0d           0x012d
#define ZcReg_Gamma0e           0x012e
#define ZcReg_Gamma0f           0x012f
#define ZcReg_Gamma10           0x0130 /* Here is the gradient */
#define ZcReg_Gamma11           0x0131
#define ZcReg_Gamma12           0x0132
#define ZcReg_Gamma13           0x0133
#define ZcReg_Gamma14           0x0134
#define ZcReg_Gamma15           0x0135
#define ZcReg_Gamma16           0x0136
#define ZcReg_Gamma17           0x0137
#define ZcReg_Gamma18           0x0138
#define ZcReg_Gamma19           0x0139
#define ZcReg_Gamma1a           0x013a
#define ZcReg_Gamma1b           0x013b
#define ZcReg_Gamma1c           0x013c
#define ZcReg_Gamma1d           0x013d
#define ZcReg_Gamma1e           0x013e
#define ZcReg_Gamma1f           0x013f
/* Luminance gamma */
#define ZcReg_YGamma00          0x0140
#define ZcReg_YGamma01          0x0141
#define ZcReg_YGamma02          0x0142
#define ZcReg_YGamma03          0x0143
#define ZcReg_YGamma04          0x0144
#define ZcReg_YGamma05          0x0145
#define ZcReg_YGamma06          0x0146
#define ZcReg_YGamma07          0x0147
#define ZcReg_YGamma08          0x0148
#define ZcReg_YGamma09          0x0149 /* Here is the gradient */ 
#define ZcReg_YGamma0a          0x014a
#define ZcReg_YGamma0b          0x014b
#define ZcReg_YGamma0c          0x014c
#define ZcReg_YGamma0d          0x014d
#define ZcReg_YGamma0e          0x014e
#define ZcReg_YGamma0f          0x014f
#define ZcReg_YGamma10          0x0150
#define ZcReg_YGamma11          0x0151


#define ZcReg_SharpnessMode     0x01c5
#define ZcReg_Sharpness00       0x01c6
#define ZcReg_Sharpness01       0x01c7
#define ZcReg_Sharpness02       0x01c8
#define ZcReg_Sharpness03       0x01c9
#define ZcReg_Sharpness04       0x01ca
#define ZcReg_Sharpness05       0x01cb
/* Synchronization */
#define ZcReg_Sync00Low         0x0190
#define ZcReg_Sync00Mid         0x0191
#define ZcReg_Sync00High        0x0192
#define ZcReg_Sync01Low         0x0195
#define ZcReg_Sync01Mid         0x0196
#define ZcReg_Sync01High        0x0197

/* Dead pixels */
#define ZcReg_DeadPixelsMode    0x0250
/* EEPROM */
#define ZcReg_EEPROMConfig      0x0300
#define ZcReg_EEPROMAccess      0x0301
#define ZcReg_EEPROMStatus      0x0302

/* Define register map macro */
#define ZR(X)   ZcReg_##X



/* Define gain macro */
//#define MakeGain(X,Y)     ((X << 6) | (Y & 0x3f))
#define MakeGain(X,Y)     ((X << 6) | ((int)((float)Y * 6.4) & 0x3f))

/* System control */
#define ZcVal_Reset             0x01
/* System operating */
#define ZcVal_Stop              0x00
        /* It seems that stop then start reset the chip... 
           What is the purpose of the previous register then ? */
#define ZcVal_Start             0x01  
/* ClockSelection */
        /* This seems needed for some sensors (HDCS, PB, ICM and PAS) */
#define ZcVal_DoubleClock       0x40
#define ZcVal_InvertClock       0x20
#define ZcVal_Fullscale         0x00
#define ZcVal_Downsampled       0x10  /* When on, this double the PCLCK */
#define ZcVal_PCLCK_6Mhz        0x00
#define ZcVal_PCLCK_3Mhz        0x01
#define ZcVal_PCLCK_1500Khz     0x02
#define ZcVal_PCLCK_750Khz      0x03
/* BRC control */
/* Clock setting */
#define ZcVal_RawMode           0x20  /* Maybe in uncompressed mode */
#define ZcVal_StandardHeader    0x10  /* Else the header is very simple */
#define ZcVal_WhiteAndBlack     0x80
#define ZcVal_FullQuant         0x00
#define ZcVal_HalfQuant         0x02
#define ZcVal_QuarterQuant      0x04
#define ZcVal_EighthQuant       0x06
#define ZcVal_Rounding          0x01
/* Test control */
#define ZcVal_Zc301Plus         0x80
#define ZcVal_Zc302             0x00
#define ZcVal_DisRemoteWakeUp   0x40
#define ZcVal_EepromSize        0x20
#define ZcVal_DebugModeOn       0x10
#define ZcVal_DebugAudioOn      0x0a
#define ZcVal_DebugVideoOn      0x04
#define ZcVal_SuspendControl    0x01
/* Acquisition time */
#define MakeAcqTime(X)          (__u8)(((X) + 169) / 170)
#define DefaultAcqTime          10710  /* Default ACQ time is 10.710ms */
/* Sensor ID */
#define ZcVal_HV7131B           0x00
#define ZcVal_HV7121B           0x01
#define ZcVal_TAS5130C          0x02
#define ZcVal_TAS5110B          0x03
#define ZcVal_CS2102            0x04
#define ZcVal_CS2103            0x05
#define ZcVal_OV7620            0x06
#define ZcVal_OVCIF             0x07
#define ZcVal_HDCS2020          0x08
#define ZcVal_HDCS1020          0x09
#define ZcVal_PB0330            0x0a
#define ZcVal_PB0111            0x0b
#define ZcVal_ICM105A           0x0c
#define ZcVal_ICM102A           0x0d
#define ZcVal_PAS202B           0x0e
#define ZcVal_PAS106B           0x0f
/* Video status */
        /* Those register are only valid when read once... ???? */
#define ZcVal_EndOfFrame        0x04 
#define ZcVal_SnapshotRequest   0x02
#define ZcVal_VideoOverflow     0x01
/* Sensor status - Video Contol */
#define ZcVal_EnableSensor      0x04
#define ZcVal_ResetSensor       0x02
#define ZcVal_SensorTurnOn      0x01
#define ZcVal_SensorTurnOff     0x00
/* Auto adjust FPS */
#define ZcVal_EnableAutoFPS     0x01
/* Last frame state */
#define LastFSMask              0x03
#define ZcVal_U0                0x00   /* Still need to know what this is */
#define ZcVal_U1                0x01
#define ZcVal_U2                0x02
#define ZcVal_U3                0x03
/* HSync */
#define ZcVal_DefaultHS0        0x01
#define ZcVal_DefaultHS1        0x0a
#define ZcVal_DefaultHS2        0x1b
#define ZcVal_DefaultHS3        0x3c
/* Target picture size */
#define ZcVal_24Kb              0x30
#define ZcVal_40Kb              0x50
#define ZcVal_56Kb              0x70
#define MakeSize(X)             ((X) > 0x200)
/* Default H Blank */
#define ZcVal_DefaultHBlankHigh 0x20
#define ZcVal_DefaultHBlankLow  0x21
#define ZcVal_DefaultResetLevel 0x30
#define ZcVal_DefaultRGainAddr  0x31
#define ZcVal_DefaultGGainAddr  0x32
#define ZcVal_DefaultBGainAddr  0x33
#define ZcVal_DefaultExpTimeHi  0x25
#define ZcVal_DefaultExpTimeMi  0x26
#define ZcVal_DefaultExpTimeLo  0x27
#define ZcVal_DefaultBlackLevel 0x57
#define ZcVal_DefaultWhiteLevel 0x59

/* I2C Device address */
#define ZcVal_I2CCheckAck       0x80
#define ZcVal_I2CDeviceAddress  0x7F
/* I2C state */
#define ZcVal_HDCSNoAck         0x04
#define ZcVal_I2CIdle           0x02
#define ZcVal_I2CNoAck          0x01
/* I2C compatibility mode */
#define ZcVal_OVSensor          0x08
#define ZcVal_TASxxxxC          0x04
#define ZcVal_Round9To8Bit      0x02
#define ZcVal_InvertSensorClk   0x01
/* I2C function */
#define ZcVal_I2CRead           0x02
#define ZcVal_I2CWrite          0x01

/* Window size - Start address */
/* For Y */
#define ZcVal_HalfscalY_ICM     0x01
#define ZcVal_FullscalY_ICM     0x02
#define ZcVal_HalfscalY_TASCVGA 0x06
#define ZcVal_FullscalY_TASCVGA 0x07
#define ZcVal_HalfscalY_TASCCIF 0x00
#define ZcVal_FullscalY_TASCCIF 0x01
#define ZcVal_HalfscalY_Default 0x00
#define ZcVal_FullscalY_Default 0x00
/* And X */
#define ZcVal_HalfscalX_ICM     0x01
#define ZcVal_FullscalX_ICM     0x02
#define ZcVal_HalfscalX_TASCVGA 0x01
#define ZcVal_FullscalX_TASCVGA 0x02
#define ZcVal_HalfscalX_TASCCIF 0x17
#define ZcVal_FullscalX_TASCCIF 0x18
#define ZcVal_HalfscalX_PAS     0x02
#define ZcVal_FullscalX_PAS     0x03
#define ZcVal_HalfscalX_Default 0x00
#define ZcVal_FullscalX_Default 0x00
/* Width and height */
#define ZcVal_FullScaleInc      0x06
#define ZcVal_HalfScaleInc      0x08
/* Max value - X */
#define ZcVal_XMaxHighDefault   0x02
#define ZcVal_XMaxLowVGADefault 0xf1
#define ZcVal_XMaxLowCIFDefault 0xdf
/* And Y */
#define ZcVal_YMaxHighDefault   0x01
#define ZcVal_YMaxLowVGADefault 0xf4
#define ZcVal_YMaxLowCIFDefault 0x2e

/* Expsure time */
#define ZcVal_DefExpTimeVGAHigh 0x01
#define ZcVal_DefExpTimeCIFHigh 0x00
#define ZcVal_DefExpTimeVGALow  0x90
#define ZcVal_DefExpTimeCIFLow  0xc8
/* Exp Gain */
#define ZcVal_DefExpGainVGA     0x07
#define ZcVal_DefExpGainCIF     0xFF


/* Operation mode */
#define ZcVal_DigitalColorGain  0x08
#define ZcVal_YGammaGain        0x04
#define ZcVal_DigitalGlobalGain 0x01

/* Sensor correction enable - Bayer pattern */
#define ZcVal_FirstPixelIsGreen 0x80
#define ZcVal_FirstLineIsGBGBG  0x40
#define ZcVal_RBInterpolAvg     0x20
#define ZcVal_GInterpolMedian   0x10
#define ZcVal_2PixInterpolation 0x08
#define ZcVal_GammaCorrection   0x04
#define ZcVal_ColorCorrection   0x02
#define ZcVal_EnableInterpol    0x01

/* Auto correct enable */
#define ZcVal_AutoResetLevel    0x10
#define ZcVal_AutoColorGain     0x08
#define ZcVal_AutoWhiteBalance  0x02
#define ZcVal_AutoExpDisable    0x00
#define ZcVal_AutoExpNormal     0x01
#define ZcVal_AutoExpAntiFlick  0x41

/* Auto exposure */
#define ZcVal_DefaultWinXStart  0x05
#define ZcVal_DefaultWinXWidth  0x0f
#define ZcVal_DefaultWinXCenter 0x28
#define ZcVal_DefaultWinYStart  0x05
#define ZcVal_DefaultWinYWidth  0x0a
#define ZcVal_DefaultWinYCenter 0x1e

/* Analog gains */
#define ZcVal_DefaultMaxGain    0x3f
#define ZcVal_DefaultMinGain    0x14

/* AWB mode */
#define ZcVal_DefaultAWBStatus  0x76
#define ZcVal_DefaultAWBFreeze  0xb5
#define ZcVal_EnableAWBFreeze   0x80
#define MakeAWBLockUnlockRange(X,Y)       ((((X) & 0x7) << 4) | ((Y) & 0x0f)) 

/* AE mode */
#define ZcVal_BlockWeightMode   0x00
#define ZcVal_WindowWeightMode  0x40
#define ZcVal_WinAndBlockWeight 0x80
#define ZcVal_Disabled          0xc0
#define ZcVal_LargeCenterWindow 0x20
#define ZcVal_SmallYRange       0x10
#define MakeAELockRange(X)          ((X) & 0x7f)
#define MakeAEUnlockRange(X)        ((X) & 0x7f)

#define ZcVal_DefaultDGain      0x01
#define MakeDigitalGain(X,Y)    ((X * 0x60) | (Y & 0x5f))
#define MakeDigitalLimitDiff(X) ((X) & 0x7f)
#define MakeDigitalGainStep(Step,DiffStep) ( (((Step) & 0x0f) << 4) | ((DiffStep) & 0x0f))

/* Synchronization (Exp time + Antiflicker) */
#define ZcVal_DefaultSync00Low  0x14
#define ZcVal_DefaultSync00Mid  0x58
#define ZcVal_DefaultSync00High 0x55
#define ZcVal_DefaultSync01Low  0x01
#define ZcVal_DefaultSync01Mid  0xb2
#define ZcVal_DefaultSync01High 0x07

/* Y target */
#define ZcVal_DefaultYTarget    0x80
#define ZcVal_DefaultResetLvl   0x20

/* Sharpening */
#define ZcVal_UseAvgValue       0x02
#define ZcVal_EnableSharpening  0x01
#define MakeSharpness(X)        ((X) * 0x0a)

/* Dead pixel */
#define ZcVal_RemoveDeadPixels  0x08
#define ZcVal_DetectDeadPixels  0x04

/* EEPROM */
#define ZcVal_EEPROMSize32Kb    0x02
#define ZcVal_EEPROMPresent     0x01
#define ZcVal_AutoWrite         0x10
#define ZcVal_BatchReadControl  0x08
#define ZcVal_BatchWriteControl 0x04
#define ZcVal_ReadMode          0x02
#define ZcVal_WriteMode         0x01






/* Define register specific value */

/* Define register value macro */
#define ZV(X)   ZcVal_##X

#define WriteDefaultValue(X, Y)  zc030x_reg_write(X, ZcReg_##Y, ZcVal_Default##Y) 


#endif
