/*
    struct video_device zc030x_fill = 
    {
        .owner =         THIS_MODULE,
        .name =          "ZC030X Webcam",
        .type =          VID_TYPE_CAPTURE,
        .hardware =      VID_HARDWARE_ZC030X,
        .fops =          pFops,
        .minor=          -1                      // minor 
    #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) 
        .initialize =    NULL,                   // initialize
    #endif
    };
*/

/* Include headers */
#include <linux/module.h>
/* Include declarations */
#include "zc030x_v4l.h"
/* Include camera */
#include "zc030x_cameras.h"

#include "zc030x_reg.h"

/* Check if the palette is valid */
inline int zc030x_v4l_checkpalette(int Palette)
{
	return (Palette == VIDEO_PALETTE_RGB24)    || 
		   (Palette == VIDEO_PALETTE_RGB32)||
//		   (Palette == VIDEO_PALETTE_YUV422)||
//		   (Palette == VIDEO_PALETTE_UYVY) ||
    0;
    /*
           (Palette == VIDEO_PALETTE_RGB32) ||
*/
}

/* Get the palette name */
inline const char * zc030x_v4l_getpalettename(int Palette)
{
	if (Palette == VIDEO_PALETTE_RGB24)  return "VIDEO_PALETTE_RGB24";
	if (Palette == VIDEO_PALETTE_RGB32)  return "VIDEO_PALETTE_RGB32";
	if (Palette == VIDEO_PALETTE_UYVY)   return "VIDEO_PALETTE_UYVY";
	if (Palette == VIDEO_PALETTE_YUV422) return "VIDEO_PALETTE_YUV422";
	return "internal error !";
}

/* Get the depth of the pixel */
int zc030x_v4l_getdepth(int Palette)
{
	if (Palette == VIDEO_PALETTE_RGB24)  return 24;
	if (Palette == VIDEO_PALETTE_RGB32)  return 32;
	if (Palette == VIDEO_PALETTE_UYVY)   return 16;
	if (Palette == VIDEO_PALETTE_YUV422) return 16;
    return 0;
}


/* Functions for pre 2.4.24 kernel compatibility */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 24)
inline void * video_get_drvdata(struct video_device *vdev)
{
    return vdev->priv;
}

inline void video_set_drvdata(struct video_device *vdev, void *data)
{
        vdev->priv = data;
}

inline struct video_device *video_device_alloc(void)
{
    struct video_device *vdev;
    
    vdev = kmalloc(sizeof(*vdev),GFP_KERNEL);
    if (NULL == vdev)
    return NULL;
    memset(vdev,0,sizeof(*vdev));
    return vdev;
}

inline void video_device_release(struct video_device *vdev)
{
    kfree(vdev);
}
#endif

/* Fill the video dynamic device object */
int zc030x_v4l_initdev(struct file_operations * pFops, struct usb_zc030x * pDev, const char * Name, struct usb_device * udev)
{
    /* Check arguments */
    if (pFops == NULL || pDev == NULL || Name == NULL)
        return V4L_Error;
    
    /* Okay, init and create the device */
    PDEBUG (5, ">> [ZCV4L] Registering video for linux device");
    
    /* Alloc video device (v4l) */
    pDev->pVideoDevice = video_device_alloc();
    /* Check allocation */
    if (pDev->pVideoDevice == NULL) 
        return V4L_Error;
    
    /* Fill it with the right value */
    memset(pDev->pVideoDevice, 0, sizeof(*pDev->pVideoDevice));
    pDev->pVideoDevice->owner       = THIS_MODULE;
    pDev->pVideoDevice->type        = VID_TYPE_CAPTURE;
    pDev->pVideoDevice->hardware    = VID_HARDWARE_ZC030X; 
    pDev->pVideoDevice->fops        = pFops;
    pDev->pVideoDevice->minor       = -1;
    /* And save the device name */
    strcpy(pDev->pVideoDevice->name, Name);

    /* Set driver data */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
    pDev->pVideoDevice->dev = &udev->dev;
#endif
    video_set_drvdata(pDev->pVideoDevice, pDev);
    
    /* Register video device */
    if (video_register_device(pDev->pVideoDevice, VFL_TYPE_GRABBER, -1) < 0)
    {
        PDEBUG(4, "Error registering the video device!");
        return V4L_Error;
    }
    
    /* Okay, return */
    PDEBUG (5, "<< [ZCV4L] Registered as video%d.\n", pDev->pVideoDevice->minor & 0x3F);
    return V4L_OK;
}

/* Release video device */
int zc030x_v4l_releasedev(struct video_device * pVideoDevice)
{
    /* Check argument */
    if (pVideoDevice != NULL)
    {
        /* Check if the device is used */
        if(pVideoDevice->minor == -1)
            video_device_release(pVideoDevice);
        else
            video_unregister_device(pVideoDevice);
    }

    /* Okay return */
    return V4L_OK;
}

/* Video for Linux specific IOCTLs */
int zc030x_v4l_ioctl (struct inode *inode, struct file *file, unsigned int cmd, void *arg)
{
    /* Return object */
    int ret = -ENOTTY;
    /* Iterators */
    int i = 0;

    /* Get the video device */
    struct video_device *vdev = video_devdata(file);
    /* And the USB device too */
    struct usb_zc030x *dev;
    
    /* Debug this function */
    PDEBUG(3,">> zc030x_v4l_ioctl()");

    /* Check parameter */
    if (vdev == NULL)
    {
        PDEBUG (2, "vdev is NULL, returning");
        return 0;
    }
    
    /* Get USB device too */
    dev = video_get_drvdata(vdev);
    if (dev == NULL)
    {
        PDEBUG (2, "dev is NULL, returning");
        return 0;
    }
    
    /* Lock device object */
    down (&dev->sem);

    /* Verify that the device wasn't unplugged */
    if (!dev->present) 
    {
        up (&dev->sem);
        return -ENODEV;
    }

    /* Debug this function */
    PDEBUG(4,"zc030x - minor %d, cmd 0x%.4x, arg %p",dev->minor, cmd, arg);

    /* Get the IOCTL command */
    switch (cmd) 
    {
        /* User is asking for capture capabilities */
        case VIDIOCGCAP:
        {
            /* Convert the argument */
            struct video_capability *b = arg;
            /* Debug it */
            PDEBUG (4, "VIDIOCGCAP %p :",b);
            /* Clear it */
            memset(b, 0, sizeof(struct video_capability));
            /* Create the card name */
            snprintf(b->name, 32, "%s with sensor %s\n", CamList[dev->type_id].Name, SensList[dev->sensor_id].Name);
            /* Set the type */
            b->type = VID_TYPE_CAPTURE;
            /* The number of channels */
            b->channels = 1;
            /* No audio yet */
            b->audios = 0;
            /* And the size */
            /* TODO: Fix this hack */
/*
            b->minwidth = SensorSizeList[dev->sensor_id].Width>>1;
            b->minheight = SensorSizeList[dev->sensor_id].Height>>1;
            b->maxwidth = SensorSizeList[dev->sensor_id].Width;
            b->maxheight = SensorSizeList[dev->sensor_id].Height;
            */
            b->minwidth = currentwidth;
            b->minheight = currentheight;
            b->maxwidth = currentwidth;
            b->maxheight = currentheight; 
            /* Return */
            ret = 0;
            break;
        }
        /* Get source channel */
        case VIDIOCGCHAN:
        {
            /* Convert the argument */
            struct video_channel *v = arg;
            /* Set the name */
            snprintf(v->name, 32, "zc030x video bridge\n");
            /* No special support */
            v->flags = 0;
            /* No tuner */
            v->tuners = 0;
            /* Set the type */
            v->type = VIDEO_TYPE_CAMERA;
            /* Return */
            ret = 0;
            break;
        }
        /* Set source channel */
        case VIDIOCSCHAN:
        {
            /* Return */
            ret = 0;
            break;
        }
        /* Get image properties */
        case VIDIOCGPICT:
        {
            /* Convert the argument */
            struct video_picture *p = arg;
            /* Set the bit depth of a pixel */
            p->depth = 24;
            /* Set the palette type */
            /* Possible values are VIDEO_PALETTE_YUV422 for large pictures */
            /* Or VIDEO_PALETTE_RGB24 for smaller one */
            p->palette = dev->VideoFormat;
            p->brightness = dev->Brightness;
            p->contrast   = dev->Contrast;
            
            /* Return */
            ret = 0;
            break;
        }
        /* Get the subcapture, if any */
        case VIDIOCGCAPTURE:
        {
            /* Convert the argument */
            int *vf = arg;
            /* Debug this */
            PDEBUG (4, "VIDIOCGCAPTURE");
            /* No subcapture */
            *vf = 0;
            /* Return */
            ret = -EINVAL;
            break;
        }
        /* Set the subcapture */
        case VIDIOCSCAPTURE:
        {
            /* Not implemented */
            ret = -EINVAL;
            break;
        }
        /* Set the capture window */
        case VIDIOCSWIN:
        {
            /* Convert the argument */
            struct video_window *vw = arg;
            /* Set the return code */
            ret = 0;
            /* Debug this */
            PDEBUG (4, "VIDIOCSWIN: width=%d, height=%d, flags=%d",
                 vw->width, vw->height,vw->flags);
            /* We don't accept capture window inside the captured area */
            if (vw->x) ret = -EINVAL;
            if (vw->y) ret = -EINVAL;
            if (vw->width  != currentwidth) ret = -EINVAL;
            if (vw->height != currentheight) ret = -EINVAL;
            break;
        }
        /* Get the capture window */
        case VIDIOCGWIN:
        {
            /* Convert the argument */
            struct video_window *vw = arg;
            /* Clear it */
            memset(vw, 0, sizeof(struct video_window));
            /* Fill it */
            vw->x      = 0;
            vw->y      = 0;
            vw->width  = currentwidth;
            vw->height = currentheight;
            vw->flags  = 0;
            /* Debug this */
            PDEBUG (4, "VIDIOCGWIN: %dx%d", vw->width, vw->height);
            /* Return  */
            ret = 0;
            break;
        }
        /* Get the video buffer */ 
        case VIDIOCGMBUF:
        {
            /* Convert the argument */
            struct video_mbuf *vm=arg;
            /* Debug this */
            PDEBUG (4, "VIDIOCGMBUF: %p - Set to %d frames", vm, ZC030X_NUMFRAMES);
            /* Clear it */
            memset(vm, 0, sizeof(struct video_mbuf));
            /* Set the size */
            vm->size = ZC030X_NUMFRAMES * MAX_VIDEODATA_SIZE;
            /* And the number of frames */
            vm->frames = ZC030X_NUMFRAMES;
            /* Set the offset for each frame */
            for(i = 0; i < ZC030X_NUMFRAMES; i++)
            {
                vm->offsets[i] = MAX_VIDEODATA_SIZE * i;
            }
            /* Okay, return */
            ret = 0;
            break;
        }
        /* Check the asked flags, change gamma & bn & constrast */
        case VIDIOCSPICT:
        {
            /* Convert the argument */
            struct video_picture *p = arg;
            /* Check if the asked format is supported */
            ret = zc030x_v4l_checkpalette(p->palette) ? 0 : -EINVAL;
            /* Now get an approximate number of the asked brightness */
            dev->Brightness = p->brightness;
            dev->Contrast   = p->contrast;
            zc030x_send_gamma(dev->udev, dev->Brightness, dev->Contrast, dev->Gamma);
            break;
        }
        /* Start the capture */
        case VIDIOCMCAPTURE:
        {
            /* Convert the argument */
            struct video_mmap * vm = arg;
            zc030x_frame      * frame = NULL;
            /* Debug this */
            PDEBUG(4, "VIDEOCMCAPTURE, format is %s", zc030x_v4l_getpalettename(vm->format));
            PDEBUG(4, "frame: %d, size: %dx%d, format: %d", vm->frame, vm->width, vm->height, vm->format);
            /* Check if asked palette is supported */
            if (!zc030x_v4l_checkpalette(vm->format))
            {
                /* Invalid palette */
                PDEBUG(1, "VIDIOCMCAPTURE: invalid format (%d)", vm->format);
                up (&dev->sem);
                return -EINVAL;
            }
            /* Set the video format to what is asked */
            dev->VideoFormat = vm->format;
            /* Check for the asked frame */
            if ((vm->frame < 0) || (vm->frame > ZC030X_NUMFRAMES))
            {
                PDEBUG(1,"VIDIOCMCAPTURE: invalid frame (%d)", vm->frame);
                up (&dev->sem);
                return -EINVAL;
            }
            /* Check for the asked height */
            /* TODO Handle change in height */
            if (vm->width != currentwidth  || vm->height != currentheight) 
            {
/*                if ((vm->width == SensorSizeList[dev->sensor_id].Width || vm->width == SensorSizeList[dev->sensor_id].Width / 2) 
                 && (vm->height == SensorSizeList[dev->sensor_id].Height || vm->height == SensorSizeList[dev->sensor_id].Height / 2)) 
                {
                    PDEBUG(1,"VIDIOCMCAPTURE: wrong dimensions. Changing to asked size %dx%d", vm->width, vm->height);
                    
                    if (zc030x_reg_setsize(dev, vm->width, vm->height, 0) != REG_OK)
                    {
                        PDEBUG(1, "Error while changing the size... LEAVING");
                        up (&dev->sem);
                        return -EINVAL;
                    }
                } else
                {
                */
                    PDEBUG(1,"VIDIOCMCAPTURE: wrong dimensions. Sensor doesn't allow size %dx%d", vm->width, vm->height);
                    up (&dev->sem);
                    return -EINVAL;/*
                }*/
                
            }
#if 0            
            /* TODO: Handle this better */
            /* Mark all other frame as unavailable for grabbing */
            down(&dev->buf_lock);
            /* Get the frame */
            frame = &dev->frame[vm->frame];
            frame->Format= vm->format;
            /* Set this frame to unused */
            down(&frame->Lock);
            /* Check if the frame is unused */
            if (frame->GrabState != FRAME_GRABBING)
            {
                /* Now this frame is ready to be grabbed into */
                frame->GrabState = FRAME_READY;
            }
            up(&frame->Lock);
            up(&dev->buf_lock);
            
            /* TODO: Remove this */
            /* Dumping the buffer state */
            for (i = 0; i < ZC030X_NUMFRAMES; i++)
            {
                frame = &dev->frame[i];
                down(&frame->Lock);
                PDEBUG(4, "DUMPASK: [%d] Frame %d state is %d", vm->frame, i, frame->GrabState);
                up(&frame->Lock);
            }
#endif
/* Remove previous algorithm */
#if 1
            /* Get the frame */
            frame = &dev->frame[vm->frame];
            /* Lock it */
//            down(&frame->Lock);

            /* If the current frame is being grabbed, wait until it's done */ 
            if (frame->GrabState == FRAME_GRABBING)
            {
                PDEBUG(4, "VIDEOCMCAPTURE: already grabbing");
                /* Schedule until it's done */
                /* Release it while waiting for it */
//                up(&frame->Lock);
                ret = wait_event_interruptible(frame->wq, (frame->GrabState != FRAME_GRABBING));
                /* Were we interrupted ? */
                if (ret)
                {
                    up (&dev->sem);
                    return -EINTR;
                }
                /* Lock it */
                down(&frame->Lock);
                /* Set to ready */
                frame->GrabState = FRAME_READY;
                /* Unlock it */
                up(&frame->Lock);
            } else
            {
                /* Set to ready */
                frame->GrabState = FRAME_READY;
                /* Unlock it */
//                up(&frame->Lock);
            }
#endif
            /* Return */
            ret = 0;
            break;
        }
        /* Synchronize to the specified frame */
        case VIDIOCSYNC:
        {
            /* Convert the argument */
            unsigned int frameIndex = *((unsigned int *) arg);
            zc030x_frame      * frame = NULL;
            /* Debug this */
            PDEBUG(4, "VIDIOCSYNC: syncing to frame %d", frameIndex);
            /* Check the argument */
            if (frameIndex >= ZC030X_NUMFRAMES)
            {
                up (&dev->sem);
                return -EINVAL;
            }
            
            /* TODO: Remove this */
            /* Dumping the buffer state */
            for (i = 0; i < ZC030X_NUMFRAMES; i++)
            {
                frame = &dev->frame[i];
                down(&frame->Lock);
                PDEBUG(4, "DUMPGET: [%d] Frame %d state is %d", frameIndex, i, frame->GrabState);
                up(&frame->Lock);
            }
            
            /* Get the Frame with the specified index */
            frame = &dev->frame[frameIndex];
            /* Lock it */
//            down(&frame->Lock);
            /* Check grabbing state */
//            redo:
            PDEBUG(4, "frame state %d", frame->GrabState);
            switch (frame->GrabState) 
            {
                /* This frame is not used */
                case FRAME_UNUSED:
                {
//                    up (&frame->Lock);
                    up (&dev->sem);
                    return -EINVAL;
                }
                /* This frame is currently aborting */
                case FRAME_ABORTING:
                {
//                    up (&frame->Lock);
                    up (&dev->sem);
                    return -ENODEV;
                }
                /* Is the frame ready, grabbing or in error ? */
                case FRAME_READY:
                case FRAME_GRABBING:
                case FRAME_ERROR:
                {
                redo:
                    /* Check if the device is still present */
                    if (!dev->present)
                    {
//                        up (&frame->Lock);
                        up (&dev->sem);
                        return -EIO;
                    }
                    /* Release the lock before waiting for a frame */
//                    up(&frame->Lock);
                    ret = wait_event_interruptible(frame->wq, (frame->GrabState == FRAME_DONE));
                    /* Were we interrupted ? */
                    if (ret)
                    {
                        up (&dev->sem);
                        return -EINTR;
                    }
                    
                    /* Synchronization is done */
//                    down(&frame->Lock);
                    PDEBUG(5,"Synched on frame %d, grabstate = %d", frameIndex, frame->GrabState);
                    
                    /* Relock the frame */
                    if (frame->GrabState == FRAME_ERROR)
                        goto redo;

                    /* Fallthrough.
                    * We have waited in state FRAME_GRABBING until it
                    * becomes FRAME_DONE, so now we can move along.
                    */
                }
                /* Frame is already uncompressed */
                case FRAME_DONE:
                {
                    PDEBUG(3,"Already synched %d\n", frameIndex);
                    /* Release the current frame. This means that it
                    * will be reused as soon as all other frames are
                    * full, so the app better be done with it quickly.
                    * Can this be avoided somehow?
                    */
                    frame->GrabState = FRAME_UNUSED;
                    break;
                }
            } 
            /* Unlock the frame */
//            up(&frame->Lock);
            
            /* Return */
            ret = 0;
            break;
        }
        default:
        {
            break;
        }
    }
    
    /* unlock the device */
    up (&dev->sem);
    
    /* Okay, return */
    PDEBUG(3,"<< zc030x_v4l_ioctl() : %d", ret);
    return ret;
}

