/*
 * Zoran 364xx based USB webcam module version 0.62
 *
 * Allows you to use your USB webcam with V4L applications
 * This is still in heavy developpement !
 *
 * Copyright (C) 2004  Antoine Jacquet <royale@zerezo.com>
 * http://royale.zerezo.com/zr364xx/
 *
 * Heavily inspired by "usb-skeleton.c", "vicam.c", "cpia.c" and "spca50x.c" drivers
 * It also uses the JPEG decoder of the "bootsplash" project (a bit hacked)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
#include <linux/wrapper.h>
#endif
#include <linux/module.h>
#include <linux/init.h>
#include <linux/videodev.h>
#include <media/v4l2-common.h>
#include <linux/usb.h>
#include <linux/vmalloc.h>
#include <linux/slab.h>
#include <linux/proc_fs.h>

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,19))
#define NEW_V4L_INTERFACE 1
#else
#undef NEW_V4L_INTERFACE
#endif

#include "decode-jpg.h"
#include "decode-jpg.c"
#include "osd.c"
//#include "usbvideo.h" /* not used, instead we declare the usbvideo_ functions */


/* Version Information */
#define DRIVER_VERSION "v0.62"
#define DRIVER_AUTHOR "Antoine Jacquet, http://royale.zerezo.com/"
#define DRIVER_DESC "Zoran 364xx"


/* Camera */
#define VID_HARDWARE_ZORAN 0xff
#define ZORAN_FRAMES 2
#define ZORAN_MAX_FRAME_SIZE 1024*1024*3
#define VIDEO_PALETTE_JPEG 21
#define VIDEO_PALETTE_JPEG_DEPRECATED 66

#define BUFFER_SIZE 0x1000
#define CTRL_TIMEOUT 500


/* debug macro */
#define DBG(x...) if (debug) info(x)


/* Module parameters */
static int debug=0;
static int osd=0;
static int mode=0;
static int hardbright=0;


/* Module parameters interface */
MODULE_PARM(debug, "i");
MODULE_PARM_DESC(debug, "Debug level");
MODULE_PARM(osd, "i");
MODULE_PARM_DESC(osd, "OSD frame count");
MODULE_PARM(mode, "i");
MODULE_PARM_DESC(mode, "0 = 320x240, 1 = 160x120, 2 = 640x480");
MODULE_PARM(hardbright, "i");
MODULE_PARM_DESC(hardbright, "Enable hardware brightness (disables autoexposure and software brightness)");


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
/* devices supported by this driver */
static struct usb_device_id device_table[] = 
{
  {USB_DEVICE (    -1,     -1)}, // add your custom entry here ...
  {USB_DEVICE (0x08ca, 0x0109)},
  {USB_DEVICE (0x041e, 0x4024)},
  {USB_DEVICE (0x0d64, 0x0108)},
  {USB_DEVICE (0x0546, 0x3187)},
  {USB_DEVICE (0x0d64, 0x3108)},
  {USB_DEVICE (0x0595, 0x4343)},
  {USB_DEVICE (0x0bb0, 0x500d)},
  {USB_DEVICE (0x0feb, 0x2004)},
  {USB_DEVICE (0x055f, 0xb500)},
  {USB_DEVICE (0x08ca, 0x2062)},
  {USB_DEVICE (0x052b, 0x1a18)},
  {USB_DEVICE (0x04c8, 0x0729)},
  {USB_DEVICE (0x04f2, 0xa208)},
  {USB_DEVICE (0x0784, 0x0040)},
  {USB_DEVICE (0x06d6, 0x0034)},
  {}  /* Terminating entry */
};

MODULE_DEVICE_TABLE (usb, device_table);
#endif


/* devices supported by this driver */
static struct
{
  int vendor;
  int product;
  int method; /* which init method this camera uses */
  const char *name;
} devices[]={
  {     -1,     -1, 0, "Custom camera" }, // ... and here !
  { 0x08ca, 0x0109, 0, "Aiptek PocketDV 3300 or Maxell Maxcam PRO DV3" },
  { 0x041e, 0x4024, 0, "Creative PC-CAM 880" },
  { 0x0d64, 0x0108, 0, "Aiptek Fidelity 3200 or Praktica DCZ 1.3 S or Genius Digital Camera" },
  { 0x0546, 0x3187, 0, "Polaroid iON 230" },
  { 0x0d64, 0x3108, 0, "Praktica Exakta DC 2200" },
  { 0x0595, 0x4343, 0, "Concord Eye-Q Duo 1300 or Concord Eye-Q Duo 2000 or Ricoh RDC-6000 or Fujifilm EX-10 or Digitrex DSC 1300 or Firstline FDC 2000" },
  { 0x0bb0, 0x500d, 0, "Concord EyeQ Go Wireless" },
  { 0x0feb, 0x2004, 0, "CRS Electronic 3.3 Digital Camera or Packard Bell DSC-300" },
  { 0x055f, 0xb500, 0, "Mustek MDC 3000" },
  { 0x08ca, 0x2062, 2, "Aiptek PocketDV 5700" },
  { 0x052b, 0x1a18, 1, "Chiphead Megapix V12" },
  { 0x04c8, 0x0729, 0, "Konica Revio 2" },
  { 0x04f2, 0xa208, 0, "Creative PC-CAM 850" },
  { 0x0784, 0x0040, 1, "Traveler Slimline X5" },
  { 0x06d6, 0x0034, 0, "Trust Powerc@m 750" }, 
  {      0,      0, 0, "" } // end entry
};


/* Camera stuff */
struct zoran
{
  struct usb_device *udev;      /* save off the usb device pointer */
  struct usb_interface *interface;    /* the interface for this device */
  struct video_device *vdev;  // v4l video device
  u8 *framebuf;
  int nb;
  unsigned char *buffer;
  unsigned char *jpeg;
  struct jpeg_decdata decdata;
  int skip;
  int brightness;
  int contrast;
  int width;
  int height;
  int palette;
  int method;
};


static int
send_control_msg(struct usb_device *udev, u8 request, u16 value, u16 index,
     unsigned char *cp, u16 size)
{
  int status;

  // for reasons not yet known to me, you can't send USB control messages
  // with data in the module (if you are compiled as a module).  Whatever
  // the reason, copying it to memory allocated as kernel memory then
  // doing the usb control message fixes the problem.
  //
  // (comment by the author of the vicam driver)

  unsigned char *transfer_buffer = kmalloc(size, GFP_KERNEL);
  memcpy(transfer_buffer, cp, size);

  status = usb_control_msg(udev,
                           usb_sndctrlpipe(udev, 0),
                           request,
                           USB_DIR_OUT | USB_TYPE_VENDOR |
                           USB_RECIP_DEVICE, value, index,
                           transfer_buffer, size, CTRL_TIMEOUT);

  kfree(transfer_buffer);

  if (status < 0) {
    info("Failed sending control message, error %d.",status);
  }

  return status;
}


/* Control messages sent to the camera to initialize it and launch the capture */
typedef struct {
  unsigned int value;
  unsigned int size;
  unsigned char *bytes;
} message;

/* method 1 */
static unsigned char m1d1[]={0,0,0,0,0,0,0,0,0,0};
static unsigned char m1d2[]={0,0,0,0,0,0};
static unsigned char m1d3[]={0,0};
static message m1[]={
  {0x1f30, 0, NULL},
  {0xd000, 0, NULL},
  {0x3370, sizeof(m1d1), m1d1},
  {0x2000, 0, NULL},
  {0x2f0f, 0, NULL},
  {0x2610, sizeof(m1d2), m1d2},
  {0xe107, 0, NULL},
  {0x2502, 0, NULL},
  {0x1f70, 0, NULL},
  {0xd000, 0, NULL},
  {0x9a01, sizeof(m1d3), m1d3},
  {-1,-1,NULL}
};

/* method 2 */
static unsigned char m2d1[]={0xff,0xff};
static unsigned char m2d2[]={0x00,0x00};
static message m2[]={
  {0x1f30, 0, NULL},
  {0xd000, 0, NULL},
  {0xf000, 0, NULL},
  {0x2000, 0, NULL},
  {0x2f0f, 0, NULL},
  {0x2650, 0, NULL},
  {0xe107, 0, NULL},
  {0x2502, sizeof(m2d1), m2d1},
  {0x1f70, 0, NULL},
  {0xd000, 0, NULL},
  {0xd000, 0, NULL},
  {0xd000, 0, NULL},
  {0x9a01, sizeof(m2d2), m2d2},
  {-1,-1,NULL}
};

/* method 3 */
static unsigned char m3d1[]={0xff,0xff};
static message m3[]={
  {0x1f30, 0, NULL},
  {0xf000, 0, NULL},
  {0x2000, 0, NULL},
  {0x2f0f, 0, NULL},
  {0x2650, 0, NULL},
  {0xe107, 0, NULL},
  {0x2502, sizeof(m3d1), m3d1},
  {0x1f70, 0, NULL},
  {-1,-1,NULL}
};

/* init table */
static message *init[3]={m1,m2,m3};


/* JPEG fixed data in header (Huffman table, etc) */

static unsigned char header1[]={
                          0xFF,0xD8, /* 0xFF,0xE0,0x00,0x10,'J','F','I','F',0x00,0x01,0x01,0x00,0x33,0x8A,0x00,0x00,0x33,0x88, */
                          0xFF,0xDB,0x00,0x84
                        };
static unsigned char header2[]={
                          0xFF,0xC4,0x00,0x1F,0x00,0x00,0x01,0x05,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0xFF,0xC4,0x00,0xB5,0x10,0x00,0x02,0x01,0x03,0x03,0x02,0x04,0x03,0x05,0x05,0x04,0x04,0x00,0x00,0x01,0x7D,0x01,0x02,0x03,0x00,0x04,0x11,0x05,0x12,0x21,0x31,0x41,0x06,0x13,0x51,0x61,0x07,0x22,0x71,0x14,0x32,0x81,0x91,0xA1,0x08,0x23,0x42,0xB1,0xC1,0x15,0x52,0xD1,0xF0,0x24,0x33,0x62,0x72,0x82,0x09,0x0A,0x16,0x17,0x18,0x19,0x1A,0x25,0x26,0x27,0x28,0x29,0x2A,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6A,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0xA2,0xA3,0xA4,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFF,0xC4,0x00,0x1F,0x01,0x00,0x03,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0xFF,0xC4,0x00,0xB5,0x11,0x00,0x02,0x01,0x02,0x04,0x04,0x03,0x04,0x07,0x05,0x04,0x04,0x00,0x01,0x02,0x77,0x00,0x01,0x02,0x03,0x11,0x04,0x05,0x21,0x31,0x06,0x12,0x41,0x51,0x07,0x61,0x71,0x13,0x22,0x32,0x81,0x08,0x14,0x42,0x91,0xA1,0xB1,0xC1,0x09,0x23,0x33,0x52,0xF0,0x15,0x62,0x72,0xD1,0x0A,0x16,0x24,0x34,0xE1,0x25,0xF1,0x17,0x18,0x19,0x1A,0x26,0x27,0x28,0x29,0x2A,0x35,0x36,0x37,0x38,0x39,0x3A,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6A,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0xA2,0xA3,0xA4,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,
                          0xFF,0xC0,0x00,0x11,0x08,0x00,0xF0,0x01,0x40,0x03,0x01,0x21,0x00,0x02,0x11,0x01,0x03,0x11,0x01,
                          0xFF,0xDA,0x00,0x0C,0x03,0x01,0x00,0x02,0x11,0x03,0x11,0x00,0x3F,0x00
                        };
static unsigned char header3;



#include "memory.c"


#ifdef NEW_V4L_INTERFACE
static int zoran_open(struct inode *inode, struct file *file)
{
  struct video_device *dev = video_devdata(file);
#else
static int zoran_open(struct video_device *dev, int flags)
{
#endif
  struct zoran *cam = dev->priv;
  struct usb_device *udev = cam->udev;
  int i,n;

  DBG("zoran_open");

  cam->palette=VIDEO_PALETTE_RGB24;
  cam->skip=2;

  if (!cam->framebuf) {  /* we do lazy allocation */
    cam->framebuf =
        (u8*)/*usbvideo_*/rvmalloc(ZORAN_MAX_FRAME_SIZE * ZORAN_FRAMES);
    if (!cam->framebuf) {
      //up(&cam->busy_lock);
      return -ENOMEM;
    }
  }

  for (i=0;init[cam->method][i].size!=-1;i++)
  {
    n=send_control_msg(udev, 1, init[cam->method][i].value, 0, init[cam->method][i].bytes, init[cam->method][i].size);
    if (n<0)
    {
      info("error during open sequence: %d",i);
      return n;
    }
  }

#ifdef NEW_V4L_INTERFACE
  file->private_data = cam;
#endif

  return 0;
}

#ifdef NEW_V4L_INTERFACE
static int zoran_close(struct inode *inode, struct file *file)
{
  struct zoran *cam = file->private_data;
#else
static void zoran_close(struct video_device *dev)
{
  struct zoran *cam = dev->priv;
#endif
  struct usb_device *udev = cam->udev;
  int i,n;

  DBG("zoran_close");

  for (i=0;i<2;i++)
  {
    n=send_control_msg(udev, 1, init[cam->method][i].value, 0, init[i][cam->method].bytes, init[cam->method][i].size);
    if (n<0)
    {
      info("error during close sequence");
#ifdef NEW_V4L_INTERFACE
      return n;
#else
      return;
#endif
    }
  }

#ifdef NEW_V4L_INTERFACE
  file->private_data = NULL;
  return 0;
#else
  return;
#endif
}

static int read_frame(struct zoran *cam, int framenum);

#ifdef NEW_V4L_INTERFACE
static ssize_t zoran_read(struct file *file, char *buf, size_t cnt, loff_t *ppos)
{
  unsigned long count = cnt;
  struct zoran *cam = file->private_data;
#else
static long zoran_read(struct video_device *dev,char *buf,unsigned long count,int noblock)
{
  struct zoran *cam = dev->priv;
#endif
  DBG("zoran_read: read %d bytes.", (int) count);

  if (!buf)
    return -EINVAL;

  if (!count)
    return -EINVAL;

  count=read_frame(cam,0);
  
  if (copy_to_user(buf,cam->framebuf,count))
    return -EFAULT;

  return count;
}


#ifdef NEW_V4L_INTERFACE
static int zoran_mmap(struct file *file, struct vm_area_struct *vma)
{
  // TODO: allocate the raw frame buffer if necessary
  unsigned long start = vma->vm_start;
  unsigned long size = vma->vm_end - vma->vm_start;
  struct zoran *cam = file->private_data;
#else
static int zoran_mmap(struct video_device *dev, const char *adr, unsigned long size)
{
  unsigned long start = (unsigned long)adr;
  struct zoran *cam = dev->priv;
#endif
  unsigned long page, pos;

  if (!cam)
    return -ENODEV;

  DBG("zoran_mmap: %ld", size);

  /* We let mmap allocate as much as it wants because Linux was adding 2048 bytes
   * to the size the application requested for mmap and it was screwing apps up.
   if (size > VICAM_FRAMES*VICAM_MAX_FRAME_SIZE)
   return -EINVAL;
   */

  /* make this _really_ smp-safe */
  /*if (down_interruptible(&cam->busy_lock))
    return -EINTR;*/

  pos = (unsigned long) (cam->framebuf);
  while (size > 0) {
    page = /*usbvideo_*/kvirt_to_pa(pos);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,10)
    /* version >= 2.6.10 */
    if (remap_pfn_range(vma, start, page >> PAGE_SHIFT, PAGE_SIZE, PAGE_SHARED))
#else
    /* 2.5.0 <= version < 2.6.10 */
    if (remap_page_range(vma, start, page, PAGE_SIZE, PAGE_SHARED))
#endif
#else
    /* version < 2.5.0 */
    if (remap_page_range(start, page, PAGE_SIZE, PAGE_SHARED))
#endif
    {
      //up(&cam->busy_lock);
      return -EAGAIN;
    }
    start += PAGE_SIZE;
    pos += PAGE_SIZE;
    if (size > PAGE_SIZE)
      size -= PAGE_SIZE;
    else
      size = 0;
  }

  //up(&cam->busy_lock);

  return 0;
}

#ifndef NEW_V4L_INTERFACE
static int zoran_video_init(struct video_device *vdev)
{
  DBG("zoran_video_init");
  return 0;
}

/*
static long zoran_write(struct video_device *v,const char *buf,unsigned long count,int nonblock)
{
  DBG("zoran_write");
  return -EINVAL;
}
*/
#endif

static int read_frame(struct zoran *cam, int framenum)
{
  int i,n,temp;
  int head;
  int actual_length;
  unsigned char *ptr=NULL;
  unsigned char swap;
  unsigned char text[20]="FRAME : 000000";
  unsigned char *pos;

redo:
  if (hardbright)
  {
    /* hardware brightness */
    n=send_control_msg(cam->udev, 1, 0x2001, 0, NULL, 0);
    temp=(0x60<<8)+(256-cam->brightness)/2;
    n=send_control_msg(cam->udev, 1,   temp, 0, NULL, 0);
  }
  head=0;
  do
  {
    n=usb_bulk_msg(cam->udev, usb_rcvbulkpipe(cam->udev, 0x81), cam->buffer, BUFFER_SIZE, &actual_length, CTRL_TIMEOUT);
    DBG("buffer : %d %d",cam->buffer[0],cam->buffer[1]);
    DBG("bulk : n=%d size=%d",n,actual_length);
    if (n<0) 
    {
      info("error reading bulk msg");
      return 0;
    }
    if (actual_length<0 || actual_length>BUFFER_SIZE)
    {
      info("wrong number of bytes");
      return 0;
    }
    
    /* swap to good indian if camera needs it */
    if (cam->method==0) for (i=0;i<BUFFER_SIZE;i+=2)
    {
      swap=cam->buffer[i];
      cam->buffer[i]=cam->buffer[i+1];
      cam->buffer[i+1]=swap;
    }
    
    /* write the JPEG header */
    if (!head)
    {
      DBG("jpeg header");
      ptr=cam->jpeg;
      memcpy(ptr,header1,sizeof(header1));
      ptr+=sizeof(header1);
      header3=0;
      memcpy(ptr,&header3,1);
      ptr++;
      memcpy(ptr,cam->buffer,64);
      ptr+=64;
      header3=1;
      memcpy(ptr,&header3,1);
      ptr++;
      memcpy(ptr,cam->buffer+64,64);
      ptr+=64;
      memcpy(ptr,header2,sizeof(header2));
      ptr+=sizeof(header2);
      memcpy(ptr,cam->buffer+128,actual_length-128);
      ptr+=actual_length-128;
      head=1;
      
      DBG("header : %d %d %d %d %d %d %d %d %d",cam->buffer[0],cam->buffer[1],cam->buffer[2],cam->buffer[3],cam->buffer[4],cam->buffer[5],cam->buffer[6],cam->buffer[7],cam->buffer[8]);
    }
    else
    {
      memcpy(ptr,cam->buffer,actual_length);
      ptr+=actual_length;
    }
  }
  while (actual_length==BUFFER_SIZE);

  if (cam->skip)
  {
    cam->skip--;
    goto redo;
  }
  
  temp=ptr-cam->jpeg;
  ptr-=2;
  while (ptr>cam->jpeg)
  {
    if (*ptr==0xFF && *(ptr+1)==0xD9 && *(ptr+2)==0xFF) break;
    ptr--;
  }
  if (ptr==cam->jpeg) info("No EOI marker");
  
  while (ptr>cam->jpeg)
  {
    if (*ptr==0xFF && *(ptr+1)==0xFF && *(ptr+2)==0xFF) break;
    ptr--;
  }
  if (ptr!=cam->jpeg)
  {
    info("Bogus frame ? %d",cam->nb);
    goto redo;
  }
  
  DBG("jpeg : %d %d %d %d %d %d %d %d %d",cam->jpeg[0],cam->jpeg[1],cam->jpeg[2],cam->jpeg[3],cam->jpeg[4],cam->jpeg[5],cam->jpeg[6],cam->jpeg[7],cam->jpeg[8]);
  
  if (cam->palette == VIDEO_PALETTE_JPEG || cam->palette == VIDEO_PALETTE_JPEG_DEPRECATED)
  {
    memcpy(cam->framebuf+framenum*ZORAN_MAX_FRAME_SIZE,cam->jpeg,temp);
    return temp;
  }
  
  if ((n=jpeg_decode(cam->jpeg,cam->framebuf+framenum*ZORAN_MAX_FRAME_SIZE,cam->width,cam->height,24,&cam->decdata))!=0)
    info("error decoding jpeg frame : %d",n);
  
  DBG("pic : %d %d %d %d %d %d %d %d %d",cam->framebuf[0],cam->framebuf[1],cam->framebuf[2],cam->framebuf[3],cam->framebuf[4],cam->framebuf[5],cam->framebuf[6],cam->framebuf[7],cam->framebuf[8]);

  if (!hardbright)
  {
    /* software brightness */
    DBG("brigthness=%d", cam->brightness);
    pos=cam->framebuf+framenum*ZORAN_MAX_FRAME_SIZE;
    for (i=0;i<cam->width*cam->height*3;i++,pos++)
    {
      temp=*pos+cam->brightness*2-256;
      if (temp<0) temp=0;
      if (temp>0xff) temp=0xff;
      *pos=(char)temp;
    }
  }
  
  /* software contrast */
  DBG("contrast=%d", cam->contrast);
  pos=cam->framebuf+framenum*ZORAN_MAX_FRAME_SIZE;
  for (i=0;i<cam->width*cam->height*3;i++,pos++)
  {
    temp=(*pos-128)*cam->contrast/128+128;
    if (temp<0) temp=0;
    if (temp>0xff) temp=0xff;
    *pos=(char)temp;
  }
  
  cam->nb++;
  
  temp=cam->nb;
  pos=text+13;
  while (temp)
  {
    *pos=temp%10+'0';
    temp=temp/10;
    pos--;
  }
  
  if (osd) osd_text(cam->framebuf+framenum*ZORAN_MAX_FRAME_SIZE,2,2,text,cam->width);

  return cam->width*cam->height*3;
}


#ifdef NEW_V4L_INTERFACE
static int zoran_ioctl(struct inode *inode, struct file *file, unsigned int ioctlnr, unsigned long ul_arg)
{
  void *arg = (void *)ul_arg;
  struct zoran *cam = file->private_data;
#else
static int zoran_ioctl(struct video_device *dev, unsigned int ioctlnr, void *arg)
{
  struct zoran *cam = dev->priv;
#endif
  int retval=0;

  if (!cam)
    return -ENODEV;

  /* make this _really_ smp-safe */
/*  if (down_interruptible(&cam->busy_lock))
    return -EINTR;*/


  switch (ioctlnr)
  {

    /* query capabilites */
    case VIDIOCGCAP:
    {
      struct video_capability b;

      DBG("VIDIOCGCAP");
      strcpy(b.name, DRIVER_DESC " webcam");
      b.type = VID_TYPE_CAPTURE;
      b.channels = 1;
      b.audios = 0;
      b.maxwidth = cam->width;  /* VIDEOSIZE_CIF */
      b.maxheight = cam->height;
      b.minwidth = cam->width;  /* VIDEOSIZE_48_48 */
      b.minheight = cam->height;

      if (copy_to_user(arg, &b, sizeof (b)))
        retval = -EFAULT;

      break;
    }

    /* get/set video source - we are a camera and nothing else */
    case VIDIOCGCHAN:
    {
      struct video_channel v;

      DBG("VIDIOCGCHAN");
      if (copy_from_user(&v, arg, sizeof (v))) {
        retval = -EFAULT;
        break;
      }
      if (v.channel != 0) {
        retval = -EINVAL;
        break;
      }

      v.channel = 0;
      strcpy(v.name, "ZR364XX");
      v.tuners = 0;
      v.flags = 0;
      v.type = VIDEO_TYPE_CAMERA;
      v.norm = 0;

      if (copy_to_user(arg, &v, sizeof (v)))
        retval = -EFAULT;
      break;
    }

    case VIDIOCSCHAN:
    {
      int v;

      if (copy_from_user(&v, arg, sizeof (v)))
        retval = -EFAULT;
      DBG("VIDIOCSCHAN %d", v);

      if (retval == 0 && v != 0)
        retval = -EINVAL;

      break;
    }

    /* image properties */
    case VIDIOCGPICT:
    {
      struct video_picture vp;
      DBG("VIDIOCGPICT");
      memset(&vp, 0, sizeof (struct video_picture));
      vp.brightness = cam->brightness << 8;
      vp.contrast = cam->contrast << 8;
      vp.depth = 24;
      vp.palette = cam->palette;
      if (copy_to_user
          (arg, &vp, sizeof (struct video_picture)))
        retval = -EFAULT;
      break;
    }

    case VIDIOCSPICT:
    {
      struct video_picture vp;
      
      if(copy_from_user(&vp, (struct video_picture *) arg,
        sizeof(struct video_picture)))
        retval = -EFAULT;

      else
      {
        DBG("VIDIOCSPICT depth = %d, pal = %d", vp.depth,
            vp.palette);

        cam->brightness = vp.brightness >> 8;
        cam->contrast = vp.contrast >> 8;
        
        if (vp.palette == VIDEO_PALETTE_JPEG_DEPRECATED)
          info("warning : 66 for VIDEO_PALETTE_JPEG is DEPRECATED, please use 21");

        if (vp.depth != 24
            || (vp.palette != VIDEO_PALETTE_RGB24 && vp.palette != VIDEO_PALETTE_JPEG && vp.palette != VIDEO_PALETTE_JPEG_DEPRECATED))
          retval = -EINVAL;
        else
          cam->palette = vp.palette;
      }

      break;
    }

    /* get/set capture window */
    case VIDIOCGWIN:
    {
      struct video_window vw;
      vw.x = 0;
      vw.y = 0;
      vw.width = cam->width;
      vw.height = cam->height;
      vw.chromakey = 0;
      vw.flags = 0;
      vw.clips = NULL;
      vw.clipcount = 0;

      DBG("VIDIOCGWIN");

      if (copy_to_user
          ((void *) arg, (void *) &vw, sizeof (vw)))
        retval = -EFAULT;

      // I'm not sure what the deal with a capture window is, it is very poorly described
      // in the doc.  So I won't support it now.
      break;
    }

    case VIDIOCSWIN:
    {

      struct video_window *vw = (struct video_window *) arg;
      DBG("VIDIOCSWIN %d x %d", vw->width, vw->height);

      if ( vw->width != cam->width || vw->height != cam->height )
        retval = -EFAULT;
      
      break;
    }

    /* mmap interface */
    case VIDIOCGMBUF:
    {
      struct video_mbuf vm;
      int i;

      DBG("VIDIOCGMBUF");
      memset(&vm, 0, sizeof (vm));
      vm.size =
          ZORAN_MAX_FRAME_SIZE * ZORAN_FRAMES;
      vm.frames = ZORAN_FRAMES;
      for (i = 0; i < ZORAN_FRAMES; i++)
        vm.offsets[i] = ZORAN_MAX_FRAME_SIZE * i;

      if (copy_to_user
          ((void *) arg, (void *) &vm, sizeof (vm)))
        retval = -EFAULT;

      break;
    }

    case VIDIOCMCAPTURE:
    {
      struct video_mmap vm;
      // int video_size;

      if (copy_from_user
          ((void *) &vm, (void *) arg, sizeof (vm))) {
        retval = -EFAULT;
        break;
      }

      DBG("VIDIOCMCAPTURE frame=%d, height=%d, width=%d, format=%d.",vm.frame,vm.width,vm.height,vm.format);

      if ( vm.frame >= ZORAN_FRAMES || (vm.format != VIDEO_PALETTE_RGB24 && vm.format != VIDEO_PALETTE_JPEG && vm.format != VIDEO_PALETTE_JPEG_DEPRECATED) )
        retval = -EINVAL;

      // in theory right here we'd start the image capturing
      // (fill in a bulk urb and submit it asynchronously)
      //
      // Instead we're going to do a total hack job for now and
      // retrieve the frame in VIDIOCSYNC

      break;
    }

    case VIDIOCSYNC:
    {
      int frame;

      if (copy_from_user((void *) &frame, arg, sizeof (int))) {
        retval = -EFAULT;
        break;
      }
      DBG("VIDIOCSYNC: %d", frame);

      read_frame(cam, frame);
      
      break;
    }

      /* pointless to implement overlay with this camera */
    case VIDIOCCAPTURE:
    case VIDIOCGFBUF:
    case VIDIOCSFBUF:
    case VIDIOCKEY:
      retval = -EINVAL;
      break;
  
      /* tuner interface - we have none */
    case VIDIOCGTUNER:
    case VIDIOCSTUNER:
    case VIDIOCGFREQ:
    case VIDIOCSFREQ:
      retval = -EINVAL;
      break;
  
      /* audio interface - we have none */
    case VIDIOCGAUDIO:
    case VIDIOCSAUDIO:
      retval = -EINVAL;
      break;
    default:
      retval = -ENOIOCTLCMD;
      break;
  }

  //up(&cam->busy_lock);
  return retval;
}


/* v4l driver */
#ifdef NEW_V4L_INTERFACE
static struct file_operations zoran_fops =
{
  .owner    = THIS_MODULE,
  .open     = zoran_open,
  .release  = zoran_close,
  .read     = zoran_read,
  .mmap     = zoran_mmap,
  .ioctl    = zoran_ioctl,
  .llseek   = no_llseek,
};
#endif

static struct video_device zoran_template=
{
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,3))
  .owner    = THIS_MODULE,
#endif
  .name     = DRIVER_DESC,
  .type     = VID_TYPE_CAPTURE,
  .hardware = VID_HARDWARE_ZORAN,
#ifdef NEW_V4L_INTERFACE
  .fops     = &zoran_fops,
#else
  .open       = zoran_open,
  .close      = zoran_close,
  .read       = zoran_read,
  .ioctl      = zoran_ioctl,
  .mmap       = zoran_mmap,
  .initialize = zoran_video_init,
#endif
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
  .release    = video_device_release,
#endif
  .minor    = -1
};


#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
#define ZORAN_NODEV -ENODEV
#else
#define ZORAN_NODEV NULL
#endif

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
static int zoran_probe(struct usb_interface *intf, const struct usb_device_id *id)
{
  struct usb_device *udev = interface_to_usbdev(intf);
#else
static void * zoran_probe(struct usb_device *udev,unsigned int ifnum,const struct usb_device_id *id)
{
#endif
  struct zoran *cam=NULL;
  int i;
  /*
  const struct usb_interface_descriptor *interface;
  const struct usb_endpoint_descriptor *endpoint;
  */
  
  DBG("probing...");
  
  for (i=0;devices[i].vendor;i++)
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,11)
    if ((udev->descriptor.idVendor==devices[i].vendor) && (udev->descriptor.idProduct==devices[i].product))
#else
    if ((le16_to_cpu(udev->descriptor.idVendor) == devices[i].vendor) && 
	(le16_to_cpu(udev->descriptor.idProduct) == devices[i].product))
#endif
	break;
  
  if (!devices[i].vendor) return ZORAN_NODEV;

  info(DRIVER_DESC " webcam plugged");
  info("model %s detected",devices[i].name);

  if ((cam=kmalloc(sizeof(struct zoran),GFP_KERNEL))==NULL)
  {
    info("cam: out of memory !");
    return ZORAN_NODEV;
  }
  memset(cam,0x00,sizeof(struct zoran));
  cam->method=devices[i].method; /* save the init method used by this camera */
  
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
  cam->vdev=video_device_alloc();
#else
  cam->vdev=kmalloc(sizeof(struct video_device),GFP_KERNEL);
#endif
  if (cam->vdev==NULL)
  {
    info("cam->vdev: out of memory !");
    return ZORAN_NODEV;
  }
  memcpy(cam->vdev,&zoran_template,sizeof(zoran_template));
  cam->vdev->priv=cam;
  
  cam->udev=udev;

  if ((cam->buffer=kmalloc(BUFFER_SIZE,GFP_KERNEL))==NULL)
  {
    info("cam->buffer: out of memory !");
    return ZORAN_NODEV;
  }
  
  if ((cam->jpeg=kmalloc(100000,GFP_KERNEL))==NULL)
  {
    info("cam->jpeg: out of memory !");
    return ZORAN_NODEV;
  }
  
  switch (mode)
  {
    case 1:
      info("160x120 mode selected");
      cam->width=160;
      cam->height=120;
      break;
    case 2:
      info("640x480 mode selected");
      cam->width=640;
      cam->height=480;
      break;
    default:
      info("320x240 mode selected");
      cam->width=320;
      cam->height=240;
      break;
  }
  
  m1d1[0]=mode;
  m2[2].value=0xf000+mode;
  header2[437]=cam->height/256;
  header2[438]=cam->height%256;
  header2[439]=cam->width/256;
  header2[440]=cam->width%256;
  
  cam->nb=0;
  cam->brightness=128;
  cam->contrast=128;
  
  /*
  cam->bulkEndpoint = bulkEndpoint;
  info("%d",bulkEndpoint);
  */

  if (video_register_device(cam->vdev, VFL_TYPE_GRABBER, -1) == -1)
  {
    kfree(cam);
    info("video_register_device failed");
    return ZORAN_NODEV;
  }
  
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
  usb_set_intfdata(intf, cam);
#endif

  info(DRIVER_DESC " controlling video device %d",cam->vdev->minor);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
  return cam;
#else
  return 0;
#endif
}


#if LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0)
static void zoran_disconnect(struct usb_interface *intf)
{
  struct zoran *cam = usb_get_intfdata(intf);
  dev_set_drvdata(&intf->dev, NULL);
#else
static void zoran_disconnect(struct usb_device *udev,void *ptr)
{
  struct zoran *cam=ptr;
#endif
    
  info(DRIVER_DESC " webcam unplugged");
  video_unregister_device(cam->vdev);
  kfree(cam->buffer);
  kfree(cam->jpeg);
  if (cam->framebuf) /*usbvideo_*/rvfree(cam->framebuf,ZORAN_MAX_FRAME_SIZE * ZORAN_FRAMES);
  kfree(cam);
}


static struct usb_driver zoran_driver=
{
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,5,70))
//  .owner      = THIS_MODULE,
#endif
  .name       = "zoran",
  .probe      = zoran_probe,
  .disconnect = zoran_disconnect,
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0))
  .id_table   = device_table
#endif
};


static int __init zoran_init(void)
{
  if (usb_register(&zoran_driver)<0)
    return -1;
  
  info(DRIVER_DESC " module loaded");
  return 0;
}


static void __exit zoran_exit(void)
{
  info(DRIVER_DESC " module unloaded");
  usb_deregister(&zoran_driver);
}


module_init(zoran_init);
module_exit(zoran_exit);

MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL");
