/*
 * Philips saa7145 controller driver, for the
 * Fast Multimedia AV Master.
 *
 * This part handles device access (PCI/I2C/codec/...)
 * 
 * Copyright (C) 2000 Serguei Miridonov <mirsev@cicese.mx>
 *
 * Currently maintained by:
 *   Luca Bonissi <lucabon@interfree.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __AVMASTER_DEVICE_H__
#define __AVMASTER_DEVICE_H__

/* general purpose I/O */
extern void GPIO(struct avmaster *avms,
		 int bit,
		 unsigned int value);

/* codec (or actually: guest bus) access */
extern int post_office_wait(struct avmaster *avms);
extern int post_office_write(struct avmaster *avms,
			     unsigned guest,
			     unsigned reg,
			     unsigned value);
extern int post_office_read(struct avmaster *avms,
			    unsigned guest,
			    unsigned reg);

extern void detect_guest_activity(struct avmaster *avms);

extern void jpeg_codec_sleep(struct avmaster *avms,
			     int sleep);
extern int jpeg_codec_reset(struct avmaster *avms);

/* zr360x7 access to raw capture */
extern void saa7145_overlay(struct avmaster *avms,
			    int on);
extern void write_overlay_mask(struct file *file,
			       struct video_clip *vp,
			       int count);
extern void saa7145_set_memgrab(struct avmaster *avms,
				int mode);
extern int wait_grab_pending(struct avmaster *avms);

/* interrupts */
extern u32 print_interrupts(struct avmaster *avms);
extern void clear_interrupt_counters(struct avmaster *avms);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
extern void 
#else
extern irqreturn_t 
#endif
		avmaster_irq(int irq, void *dev_id, struct pt_regs *regs);

/* JPEG codec access */
extern void jpeg_start(struct avmaster *avms);
extern void saa7145_enable_jpg(struct avmaster *avms,
			       enum avmaster_codec_mode mode);
extern void avmaster_feed_stat_com(struct avmaster *avms);

/* general */
extern void avmaster_set_pci_master(struct avmaster *avms,
				 int set_master);
extern void avmaster_init_hardware(struct avmaster *avms);
extern void saa7145_restart(struct avmaster *avms);
extern void saa7145_init_vfe(struct avmaster *avms);

/* i2c */
extern int decoder_command(struct avmaster *avms,
			   int cmd,
			   void *data);
extern int encoder_command(struct avmaster *avms,
			   int cmd,
			   void *data);

#endif				/* __AVMASTER_DEVICE_H__ */
