/*
 * saa7145.h
 * This is a driver that accesses the registers of an SAA7145.
 *
 * Copyright (C) 1999,2000  Michael Stickel
 * Copyright (C) 2004 Luca Bonissi <lucabon@interfree.it>
 *
 * Currently maintained by:
 *   Luca Bonissi <lucabon@interfree.it>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#ifndef __SAA7145_H__
#define __SAA7145_H__

/* 
 *  This file contains register definitions and usefull macros
 *  for the Philips SAA7145 (Pantera) Multimedia PCI-Bridge.
 */

#define PCI_CLOCK	33000000

/* SAA7145 Register SET.  */
#define VDBA		(0x00)		// Video DMA Base Address

#define VDPA		(0x04)		// Video DMA Protection Address
# define VDPA_MASK	(0x3fffff<<2)	// Video DMA Protection Address Mask
# define VDPA_VDPEN	(1)		// Enable Video DMA Protection Mask

#define VDMAC		(0x08)		// Video DMA Control
# define VDMAC_EVID	(1<<31)		// Enable Video Capture and Transfer
# define VDMAC_VBS	(7<<25)		// Video Data Burst Transfer Size Mask (power of 2, 7=unlimited)
#  define VDMAC_VBS_1	(0<<25)
#  define VDMAC_VBS_2	(1<<25)
#  define VDMAC_VBS_4	(2<<25)
#  define VDMAC_VBS_8	(2<<25)
#  define VDMAC_VBS_16	(4<<25)
#  define VDMAC_VBS_32	(5<<25)
#  define VDMAC_VBS_64	(6<<25)
#  define VDMAC_VBS_INF	(7<<25)
# define VDMAC_VTHR	(7<<19)		// Video FIFO Threshold Mask (1,16,32,48,64,80,96,112)
#  define VDMAC_VTHR_1		(0<<21)
#  define VDMAC_VTHR_16		(1<<21)
#  define VDMAC_VTHR_32		(2<<21)
#  define VDMAC_VTHR_48		(3<<21)
#  define VDMAC_VTHR_64		(4<<21)
#  define VDMAC_VTHR_80		(5<<21)
#  define VDMAC_VTHR_96		(6<<21)
#  define VDMAC_VTHR_112	(7<<21)
# define VDMAC_VEND	(3<<19)		// Endianness Controls (00=none, 01=1234->3412, 10=1234->4321)
#  define VDMAC_VEND_NONE	(0<<19)
#  define VDMAC_VEND_3412	(1<<19)
#  define VDMAC_VEND_4321	(2<<19)
# define VDMAC_PFMT	(7<<16)		// Output Format
#  define VDMAC_PFMT_RGB32	(0<<16) // RGB32: aRGB
#  define VDMAC_PFMT_RGB24	(1<<16)	// RGB24: RGB
#  define VDMAC_PFMT_RGB16	(2<<16) // RGB16: 565
#  define VDMAC_PFMT_RGB15	(3<<16) // RGB15: a555
#  define VDMAC_PFMT_YUV16	(4<<16) // YUV422: CIRR (UYVY)
#  define VDMAC_PFMT_GRAY	(5<<16) // Grayscale: Y
#  define VDMAC_PFMT_RGB16d	(6<<16) // RGB16: 565 with dither
#  define VDMAC_PFMT_RGB15d	(7<<16) // RGB15: a555 with dither
# define VDMAC_VFLIP	(1<<15)		// Vertical Flip
# define VDMAC_VPCH	(0x1fff)	// Line Pitch (difference between 2 vertical pixel)

#define VPCTL		(0x0c)		// Video Port Control
# define VPCTL_TEST	(1<<15)		// 1=Test, 0=Normal
# define VPCTL_EVENDD	(1<<11)		// Turn off even fields
# define VPCTL_ODDD	(1<<10)		// Turn off odd fields
# define VPCTL_REVS	(1<<9)		// Reference Edge for Vertical Sync
# define VPCTL_REHS	(1<<8)		// Reference Edge for Horizontal Sync
# define VPCTL_TWOS	(1<<7)		// U and V components in Two's Complements
# define VPCTL_SF	(1<<6)		// Swap fields
# define VPCTL_FODD	(1<<5)		// Force Odd
# define VPCTL_FD	(3<<3)		// Field Detect Mode (00=Normal, 01=Noise Limited, 10=Force Toggle, 11=External Input)
# define VPCTL_SOE	(1<<2)		// Sync Output Enable
# define VPCTL_XSS	(1<<1)		// External Sync Source (1=pins, 0=stream)
# define VPCTL_VPZ	(1<<0)		// Video Port size (1=16 bits, 0=8 bits)

#define XWC		(0x10)		// Horizontal Window Control
# define XWC_XWS_M	(0x3ff)		// Horizontal Window Start Mask
# define XWC_XWS_O	(16)		//                         Offset
# define XWC_XWZ_M	(0x3ff)		// Horizontal Window Size  Mask
# define XWC_XWZ_O	(0)		//                         Offset

#define YWC		(0x14)		// Vertical Window Control
# define YWC_YWS_M	(0x3ff)		// Vertical Window Start Mask
# define YWC_YWS_O	(16)		//                       Offset
# define YWC_YWZ_M	(0x3ff)		// Vertical Window Size  Mask
# define YWC_YWZ_O	(0)		//                       Offset

#define FSCTL		(0x18)		// Filter and Scaler Control
# define FSCTL_ILO	(1<<31)		// Interlace Output
# define FSCTL_FMODE_M	(7)		// Horizontal Filter Mode Mask
# define FSCTL_FMODE_O  (28)		//                        Offset
# define FSCTL_XSCI_M	(0x7ff)		// Horizontal Scaling Increment Mask
# define FSCTL_XSCI_O	(16)		//                              Offset
# define FSCTL_YSCI_M	(0x7ff)		// Vertical Scaling Increment Mask
# define FSCTL_YSCI_O	(0)		//                            Offset

#define YPR		(0x1C)		// Vertical Phase
# define YPR_YPO_M	(0x3ff)		// Vertical Phase Offset Odd Mask
# define YPR_YPO_O	(16)		//                           Offset
# define YPR_YPE_M	(0x3ff)		// Vertical Phase Offset Even Mask
# define YPR_YPE_O	(0)		//                            Offset

#define CDBA		(0x20)		// Clip Mask DMA Base Address

#define CDF		(0x24)		// Clip Data Format
# define CDF_CPO_M	(0x1f)		// Pixel Offset into Dword Mask
# define CDF_CPO_O	(24)		//                         Offset
# define CDF_CPIX_M	(0x3f)		// Number of Dwords per Line of Clip Mask
# define CDF_CPIX_O	(16)		//                                   Offset
# define CDF_CLINE_M	(0x3ff)		// Number of Line of Clip Mask
# define CDF_CLINE_O	(0)		//                        Offset

#define CDMAC		(0x28)		// Clipper DMA Control
# define CDMAC_ECLIP	(1<<31)		// Enable Clipper
# define CDMAC_CEND	(3<<19)		// Endianness Control
#  define CDMAC_CEND_NONE_BE	(0<<19)	// No swap, Big Endian
#  define CDMAC_CEND_SWAP_BE	(1<<19)	// Swap 4321 => 1234, Big Endian
#  define CDMAC_CEND_NONE_LE	(2<<19)	// No swap, Little Endian
#  define CDMAC_CEND_SWAP_LE	(3<<19)	// Swap 4321 => 1234, Little Endian
# define CDMAC_CPOL	(1<<16)		// Clipper Polarity
# define CDMAC_CPCH	(0x3ff)		// Line Pitch

#define LCR		(0x2C)		// Line Counter Register
# define LCR_SLCT_M	(0x3ff)		// Source line counter threshold Mask
# define LCR_SLCT_O	(16)		//                               Offset
# define LCR_TLCT_M	(0x3ff)		// Target line counter threshold Mask
# define LCR_TLCT_O	(0)		//                               OffseT

#define ADBA		(0x30)		// Audio DMA Base Address
# define ADBA_MASK	(0x3fffffff<<2)	// Audio DMA Base Address Mask

#define ADPA		(0x34)		// Audio DMA Protection Address
# define ADPA_MASK	(0x7fff<<2)	// Audio DMA Protection Address Mask

#define ADMAC		(0x38)		// Audio DMA Control
# define ADMAC_EAUD	(1<<31)		// Enable Audio DMA
# define ADMAC_AEND	(1<<19)		// Audio Data Endianness
# define ADMAC_AFMT	(3<<16)		// Audio Format Control
# define ADMAC_ARNG	(0xf)		// DMA Interrupt Range

#define RPSPTR		(0x3C)		// RPS Pointer Address
# define RPSPTR_MASK	(0x3fffffff<<2)	// RPS Pointer Address Mask
# define RPSPTR_ERPS	(1)		// Enable RPS

#define RPSR		(0x40)		// RPS Page Register
# define RPSR_RPSP	(0x1fffff<<11)	// RPS Page Address
# define RPSR_ERPSP	(1)		// RPS Page Error Enable

#define RPSTO		(0x44)		// RPS Time Out
# define RPSTO_TOSEL	(3<<28)		// RPS Time Out Selector
#  define RPSTO_TOSEL_NONE	(0<<28)	// No Time Out
#  define RPSTO_TOSEL_PCI	(1<<28)	// PCI Clock Counter
#  define RPSTO_TOSEL_VSYNC	(2<<28)	// V Sync Counter
#  define RPSTO_TOSEL_BOTH	(3<<28)	// PCI & V Sync Counters
# define RPSTO_TOV_M	(0xf)		// RPS Video Time Out Value Mask
# define RPSTO_TOV_O	(24)		//                          Offset
# define RPSTO_TOP_M	(0xffffff)	// RPS PCI Time Out Value Mask
# define RPSTO_TOP_O	(0)		//                        Offset

# define RPS_FLG_ANY	(1<<15)		// AND / OR logical for the flags
# define RPS_FLG_INV	(1<<14)		// Invert Flag
# define RPS_FLG_RPS_S	(1<<13)		// RPS Semaphore
# define RPS_FLG_VFE	(1<<12)		// Video FIFO Empty
# define RPS_FLG_GPIO3	(1<<11)		// GPIO3 High
# define RPS_FLG_GPIO2	(1<<10)		// GPIO2 High
# define RPS_FLG_GPIO1	(1<<9)		// GPIO1 High
# define RPS_FLG_GPIO0	(1<<8)		// GPIO0 High
# define RPS_FLG_EAW	(1<<7)		// End of Active Window
# define RPS_FLG_EOL	(1<<6)		// End Of Line
# define RPS_FLG_HS	(1<<5)		// Horizontal count Source
# define RPS_FLG_HT	(1<<4)		// Horizontal count Target
# define RPS_FLG_ODD	(1<<3)		// Odd field active
# define RPS_FLG_EVEN	(1<<2)		// Even field active
# define RPS_FLG_DEBID	(1<<1)		// DEBI Done
# define RPS_FLG_IICD	(1<<0)		// I2C Done

#define ISR		(0x48)		// Interrupt Status

#define IER		(0x4C)		// Interrupt Enable

#define IMR		(0x50)		// Interrupt Monitor

# define ISRC_PPER    (1<<31)		// PCI Parity Error on Read
# define ISRC_PABO    (1<<30)		// PCI Acces Abort
# define ISRC_PPEW    (1<<29)		// PCI Parity Error on Write
# define ISRC_DEBI_D  (1<<28)		// DEBI Done
# define ISRC_DEBI_I  (1<<27)		// DEBI Interrupt
# define ISRC_DEBI_TO (1<<26)		// DEBI Time Out Error
# define ISRC_IIC_E   (1<<25)		// I2C Error
# define ISRC_IIC_D   (1<<24)		// I2C Done
# define ISRC_ARI     (1<<23)		// Audio Range Interrupt
# define ISRC_UPLD    (1<<22)		// RPS Upload
# define ISRC_RPS_S   (1<<21)		// RPS Semaphore
# define ISRC_RPS_I   (1<<20)		// RPS INTERRUPT command
# define ISRC_RPS_TO  (1<<19)		// RPS Time Out
# define ISRC_RPS_L   (1<<18)		// RPS Late Error
# define ISRC_RPS_PE  (1<<17)		// RPS Page Error
# define ISRC_VFE     (1<<16)		// Video FIFO empty
# define ISRC_VBI     (1<<15)		// Vertical Blank Interval
# define ISRC_HBI     (1<<14)		// Horizontal Blank Interval
# define ISRC_FIDN    (1<<13)		// Field ID Not
# define ISRC_FID     (1<<12)		// Field ID
# define ISRC_AFO     (1<<11)		// Audio FIFO Overflow
# define ISRC_CFU     (1<<10)		// Clipper FIFO Underflow
# define ISRC_VPE     (1<<9)		// Video DMA Address Protection Error
# define ISRC_VFO     (1<<8)		// Video FIFO Overflow
# define ISRC_GP3     (1<<7)		// GPIO 3 
# define ISRC_GP2     (1<<6)		// GPIO 2
# define ISRC_GP1     (1<<5)		// GPIO 1
# define ISRC_GP0     (1<<4)		// GPIO 0
# define ISRC_EC2     (1<<2)  		// Event Counter 1
# define ISRC_EC1     (1<<1)		// Event Counter 2
# define ISRC_MASTER  (1<<0)		// MASTER

#define ECR		(0x54)		// Event Counter
# define ECR_MASK	(0xfffff)	// Event Counter Mask

#define ECT		(0x58)		// Event Counter Threshold
# define ECT_MASK	(0xfffff)	// Event Counter Threshold Mask

#define ECC		(0x5C)		// Event Counter Control
# define ECC_MASK	(0xffff)	// Event Counter Control Mask

#define GPIOC		(0x60)		// GPIO Control

#define DEBICFG		(0x64)		// DEBI Configuration 
# define DEBICFG_XIRQPOL	(1<<29)	// XIRQ Monitor
# define DEBICFG_SSDS		(1<<28)	// 1=Smart Slave, 0=Dump Slave
# define DEBICFG_XIRQ_WAIT	(1<<27)	// If 1, DEBI is hold until cleared
# define DEBICFG_TIMEOUT	(0xf<<22)// Terminate the transfer after this many PCI clocks
# define DEBICFG_SWAP		(3<<20)	// Endian swapping (00=none, 01=1234->3412, 10=1234->4321)
# define DEBICFG_SLAVE16	(1<<19) // 1=16 bits, 0=8 bits
# define DEBICFG_INCR		(1<<18) // Enables address increment for block transfer
# define DEBICFG_INMO		(1<<17) // Bus style, 1=Intel, 0=Motorola
# define DEBICFG_TIEN		(1<<16) // Timer enable

#define DEBIDAT		(0x68)		// DEBI Data (if BLOCKLEN<=4) or Address of Data

#define DEBICMD		(0x6C)		// DEBI Command
//# define DEBICMD_BLOCKLEN(len) (((len)&0x7fff)<<17)
# define DEBICMD_BLOCKLEN (0x7fff<<17)	// Data block length in bytes
# define DEBICMD_WRITE_N  (1<<16)	// Transfer direction. 1=Read, 0=Write
# define DEBICMD_A16_IN   (0xffff)	// Slave Target Address

#define IICTFR		(0x70)		// I2C Transfer Control
# define IICTFR_BYTE2	(0xff<<24)	// Data/Address Byte 2
# define IICTFR_BYTE1	(0xff<<16)	// Data/Address Byte 1
# define IICTFR_BYTE0	(0xff<<8)	// Data/Address Byte 0
# define IICTFR_ATTR2	(3<<6)		// Attribute information for BYTE2 (00=nop, 01=end, 10=continue, 11=start)
# define IICTFR_ATTR1	(3<<4)		// Attribute information for BYTE1
# define IICTFR_ATTR0	(3<<2)		// Attribute information for BYTE0
#  define IIC_START	(3)
#  define IIC_CONT	(2)
#  define IIC_STOP	(1)
#  define IIC_NOP	(0)
# define IICTFR_ERR	(1<<1)		// General Error Flag
# define IICTFR_BUSY	(1)		// Transfer in progress

#define IICSC		(0x74)		// I2C Status and Clock
# define IICSC_IICCC	(7<<8)		// I2C Clock Control
#  define IICCC_DIV120	(0<<8)
#  define IICCC_DIV3200	(1<<8)
#  define IICCC_DIV80	(2<<8)
#  define IICCC_DIV8	(3<<8)		// Don't use this rate - for ref only 
#  define IICCC_DIV480	(4<<8)
#  define IICCC_DIV6400	(5<<8)
#  define IICCC_DIV320	(6<<8)
#  define IICCC_DIV12	(7<<8)		// Don't use this rate - for ref only 
# define IICSC_IICIRQ	(1<<7)		// Interupt Request (Like ABORT for SAA7146?)
# define IICSC_APERR	(1<<5)		// Address Phase Error
# define IICSC_DTERR	(1<<4)		// Data Transmition Error
# define IICSC_ERR	(1<<1)		// General Error Flag
# define IICSC_BUSY	(1<<0)		// Transfer in progress

#define UTIL		(0x78)		// Utility Register

#define RSVD		(0x7C)		// Reserved for future use (it seems a copy of util)

/*--------------[ Macros ]------------------- */

#define RPS_LDREG(ptr,reg,val)  *(ptr++) = 0x50000100 | (reg&0xff); *(ptr++) = val;
#define RPS_STREG(ptr,reg,addr) *(ptr++) = 0x60000100 | (reg&0xff); *(ptr++) = val;
#define RPS_STOP(ptr)           *(ptr++) = 0x30000000;
#define RPS_JUMP(ptr,flag,addr) *(ptr++) = 0x40000000 | (flag&0xffff); *(ptr++) = addr;
#define RPS_IRQ(ptr,flag)       *(ptr++) = 0x20000000 | (flag&0xffff);
#define RPS_WAIT(ptr,flag)      *(ptr++) = 0x90000000 | (flag&0xffff);
#define RPS_CHECK(ptr,flag)     *(ptr++) = 0x80000000 | (flag&0xffff);
#define RPS_PAUSE(ptr,flag)     *(ptr++) = 0xA0000000 | (flag&0xffff);
#define RPS_SET(ptr,flag)       *(ptr++) = 0x10000000 | (flag&0xffff);
#define RPS_CLR(ptr,flag)       *(ptr++) = 0x00000000 | (flag&0xffff);

/*--------------[ Constants ]------------------- */
#define SAA7145_RPS_MEM         4096    /* max memory per message */
#define SAA7145_I2C_MEM         4096    /* max memory per message */
#define SAA7145_I2C_TIMEOUT     100     /* i2c-timeout-value in ms */
#define SAA7145_I2C_RETRIES     3       /* how many times shall we retry an i2c-operation? */
#define SAA7145_I2C_DELAY       5       /* time we wait after certain i2c-operations */

#endif /* __SAA7145_H__ */

